package AmiRNA::Config::ConfigParser;

use strict;
use warnings;
use XML::Smart;
use AmiRNA::Error;

our $VERSION = '3.1';

sub new {
	my ($self, $configfile) = @_;
	my $stdconfloc = "/etc/AmiRNA/AmiRNA.xml";

	$configfile = $stdconfloc if not defined $configfile;
	my $XML;

 	eval {
 		if(not -e $configfile) {
 			die AmiRNA::Error->new( { errno => $!, msg => "Configuration file not found", subject => $configfile });
 		}
 		eval {
			$XML = XML::Smart->new($configfile);
			$XML = $XML->{config};
		};
		if($@) {
			die AmiRNA::Error->new( { errno => $!, msg => "Invalid configuration file. Please check $configfile with xmllint.", subject => $configfile });
		}
 	};
	die if $@;

	$self = {
		configfile => $configfile,
		conf => $XML,
	};
	bless($self);
	return $self;
}

### Get index path with fasta transcript libraries and index files
sub get_index_path {
	my ($self) = @_;
	return( $self->{conf}->{index_path} );
}

### Get tmp path
sub get_tmp_path {
	my ($self) = @_;
	return( $self->{conf}->{tmp_path} );
}

### Get executable path for "mapping" tool and "folding" tool
sub get_executable_path {
	my ($self, $name) = @_;
	return( $self->{conf}->{external_programs}{$name}{path} );
}

### Get name of "mapping" tool and "folding" tool
sub get_executable_name {
	my ($self, $name) = @_;
	return( $self->{conf}->{external_programs}{$name}{binary} );
}

### Get database engine specified in config file
sub get_db_engine {
        my ($self) = @_;
        return( $self->{conf}->{database}{engine} );
}

### Get database DSN specified in config file
sub get_db_dsn {
	my ($self) = @_;
	return( $self->{conf}->{database}{dsn} );
}

### Get database user specified in config file
sub get_db_user {
	my ($self) = @_;
	return( $self->{conf}->{database}{user} );
}

### Get database password specified in config file
sub get_db_passwd {
	my ($self) = @_;
	return( $self->{conf}->{database}{password} );
}

### Get maximum number of jobs in queue
sub get_mjn {
	my ($self) = @_;
	return( $self->{conf}->{MAX_JOB_NUMBER} );
}

### Get maximum number of jobs per IP
sub get_mci {
	my ($self) = @_;
	return( $self->{conf}->{MAX_CALCULATIONS_PER_IP} );
}

### Get maximum number of jobs per email address
sub get_mce {
	my ($self) = @_;
	return( $self->{conf}->{MAX_CALCULATIONS_PER_EMAIL} );
}
1;

__END__

=head1 NAME

 AmiRNA::Config::ConfigParser

=head1 SYNOPSIS

 # Create ConfigParser object
 my $cp = AmiRNA::Config::ConfigParser->new($config_file);
	
 # Get some configuration parameter from config file
 my $index_path = $cp->get_index_path();

=head1 DESCRIPTION

 Module for reading of the main AmiRNA configuration file (e.g. /etc/AmiRNA/AmiRNA.xml)


=head1 METHODS

=over

=item AmiRNA::Config::ConfigParser-E<gt>new($config_file)

 Create ConfigurationParser object.
 $configfile is optional, default is /etc/AmiRNA/AmiRNA.xml


=item AmiRNA::Config::ConfigParser-E<gt>get_index_path()

 Returns directory path of transcript library indices.


=item AmiRNA::Config::ConfigParser-E<gt>get_tmp_path()

 Returns path of tmp directory.


=item AmiRNA::Config::ConfigParser-E<gt>get_executable_path($type)

 Returns executable path of mapping or folding tool. $type can be "mapping" or "folding".


=item AmiRNA::Config::ConfigParser-E<gt>get_executable_name($type)

 Returns executable name of mapping tool. Currently $type can only be "mapping". This version
 of AmiRNA supports only the ViennaRNA package (RNAcofold and RNAup) for folding and therefore the
 folding tool cannot be specified.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_engine()

 Returns database engine (mysql, postgres, sqlite).


=item AmiRNA::Config::ConfigParser-E<gt>get_db_dsn()

 Returns database DSN.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_user()

 Returns database user name.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_passwd()

 Returns database user password.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_mjn()

 Returns maximum number of jobs in queue.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_mci()

 Returns maximum number jobs per IP.


=item AmiRNA::Config::ConfigParser-E<gt>get_db_mce()

 Returns maximum number jobs per email address.

=back


=head1 DEPENDENCIES

=over

=item L<XML::Smart>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
