package AmiRNA::Config::UserParams;

use strict;
use warnings;

use AmiRNA::Error;

our $VERSION = '3.1';

sub new {
	my ( $self ) = @_;
	$self = {};
	bless($self);
	return $self;
}

### Check if all required parameters are set
sub check {
	my ($self, $allowed_params, $required_params, $user_param) = @_;

	my @allowed_params = @{$allowed_params};
	my @required_params = @{$required_params};
	my %user_param = %{$user_param};

	eval {
		foreach my $key (sort keys %user_param) {
			my $regex = "^$key\$";
			if( scalar(grep(/$regex/, @allowed_params)) == 0 ) {
				die AmiRNA::Error->new( { errno => "Parameter '$key' not allowed: " . scalar(grep(/$regex/, @allowed_params)),
					msg => "Parameter '$key' not allowed", subject => "Wrong user parameters" } );
			}
			
			if( (! defined $user_param{$key}) || ($user_param{$key} eq "") ) {
				if( scalar(grep(/$regex/, @required_params)) != 0 ) {
					die AmiRNA::Error->new( { errno => "Missing required parameter '$key' in job constructor", 
						msg => "Missing Parameter '$key'", subject => "Wrong user parameters" } );
					}
				}
			}

			if(scalar(keys %user_param) != scalar(@allowed_params) ) {

				# from perlfaq4
				my (@union, @intersection, @difference);
				@union = @intersection = @difference = ();
				my %count = ();
				foreach my $element (@allowed_params, keys %user_param) { $count{$element}++ }
				foreach my $element (keys %count) {
					push @union, $element;
					push @{ $count{$element} > 1 ? \@intersection : \@difference }, $element;
				}

				die AmiRNA::Error->new( { errno => "Wrong number of parameters (required " . scalar(@allowed_params) 
					. ", got " .scalar(keys %user_param) ."). Missing: @difference.", msg => "Wrong number of parameters", subject => "Wrong user parameters" } );
			}
	};
	die if $@;

	return(1);
}
1;

__END__

=head1 NAME

 AmiRNA::Config::UserParams

=head1 SYNOPSIS

 # Create UserParams object
 my $userParams = AmiRNA::Config::UserParams->new();
        
 # Check if user parameters of any function are complete
 if( $userParams->check($allowed_params, $required_params, $user_param) ) {
       # use user params
 }


=head1 DESCRIPTION

 Module to check if user parameters for any AmiRNA function are complete.


=head1 METHODS

=over

=item AmiRNA::Config::ConfigParser-E<gt>new()

 Create UserParams object.


=item AmiRNA::Config::ConfigParser-E<gt>check()

 Returns 1 if user params are complete, otherwise it dies 
 with an AmiRNA::Error.

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2006 by Max Planck Institute for Developmental Biology, 
Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
