package AmiRNA::DB::Base;

use strict;
use warnings;

use DBI;
use AmiRNA::Error;
use AmiRNA::Config::ConfigParser;
use AmiRNA::DB::Job;

our $VERSION = '3.1';

sub new {
	my ( $self, $configfile ) = @_;

	my $configParser;
	eval {
		$configParser = AmiRNA::Config::ConfigParser->new($configfile);
	};
	die if $@;

	$self = {
		MAX_JOB_NUMBER             => $configParser->get_mjn(),
		MAX_CALCULATIONS_PER_IP    => $configParser->get_mci(),
		MAX_CALCULATIONS_PER_EMAIL => $configParser->get_mce(),
		engine => $configParser->get_db_engine(),
		dsn    => $configParser->get_db_dsn(),
		user   => $configParser->get_db_user(),
		passwd => $configParser->get_db_passwd(),
		dbh    => "",
	};
	bless $self;
	return $self;
}


### Connect to a database, supported databases are MySQL, PostgreSQL and SQLite
sub connect {
	my ($self) = @_;

	eval {
		if( (! defined $self->{dsn}) || (! defined $self->{user}) || (! defined $self->{passwd}) ) {
			die AmiRNA::Error->new( { errno => "Missing login credentials", msg => "Cannot connect to database.", subject => "dsn, user or passwd" } );
		}
	};
	die if $@;

	eval {
		$self->{dbh} = DBI->connect( $self->{dsn}, $self->{user}, "$self->{passwd}", {PrintError=>0, RaiseError=>0} ) or 
			die AmiRNA::Error->new( { errno => DBI->errstr, msg => "Cannot connect to database.", subject => "Database connection failed" } );
	};
	die if $@;
	
	return 1;
}


### Disconnect from database
sub disconnect {
	my ($self) = @_;

	eval {
		if(! defined $self->{dbh}) {
			die AmiRNA::Error->new( { errno => $!, msg => "DB handler does not exist.", subject => "DB disconnect failed." } );
		}

		$self->{dbh}->disconnect or die AmiRNA::Error->new( { errno => $!, msg => "DB disconnect failed.", subject => "DB disconnect failed." } );
	};
	die if $@;

	return 1;       
}


### Returns an autoincrement id field for the database engine
sub autoincrement_id {
	my ($self) = @_;

	if ( $self->{engine} eq 'SQLite' ) {
		return "id\tINTEGER PRIMARY KEY";
	}
	elsif ( $self->{engine} eq 'PostgreSQL' ) {
		return "id\tSERIAL UNIQUE NOT NULL";
	}
	elsif ( $self->{engine} eq 'MySQL' ) {
		return "id\tINTEGER PRIMARY KEY AUTO_INCREMENT";
	}
	else {
		return "";
	}
}


### Extract gene names from fasta file
sub get_seq_ids_from_sequence {
	my ( $self, $job ) = @_;
	my @seq_ids = ();
	my @lines = split( /\n/, $job->{sequences});

	if( $job->{sequences} =~ /^>/ ) {
		foreach my $line( @lines ) {
			if( $line =~ /^>/ ) {
				chomp($line);
				$line =~ s/>//g;
				push( @seq_ids, $line );
			}
		}
	}
	else {
		foreach my $line( @lines ) {
			chomp($line);
			push( @seq_ids, $line );
		}
	}

	return sort @seq_ids;
}

1;

__END__

=head1 NAME

 AmiRNA::DB::Base 


=head1 SYNOPSIS

 # Create Base object
 my $db = AmiRNA::DB::Base->new($configfile);

 # Connect to database
 $db->connect();

 # Use Base object to create Queue and Result objects
 my $queue = AmiRNA::DB::Queue->new($db);
 my $results = AmiRNA::DB::Results->new($db);

 # Disconnect
 $db->disconnect();


=head1 DESCRIPTION

 The database module for the designer database schema. Responsible for
 connecting and disconnecting and other DB support functions.


=head1 METHODS

=over

=item AmiRNA::DB::Base-E<gt>new($configfile)

 Construct a basic database object. 
 $configfile is optional, default '/etc/AmiRNA/AmiRNA.conf'


=item AmiRNA::DB::Base-E<gt>connect()

 Connects to the database specified in the config file.
 The following parameters are requested from the config file for  MySQL, PostgreSQL or SQLite:

    <config><database><engine>
    <config><database><dsn>
    <config><database><user>
    <config><database><password>


=item AmiRNA::DB::Base-E<gt>disconnect()

Disconnect from database.


=item AmiRNA::DB::Base-E<gt>autoincrement()

Get autoincrement SQL code for database engine specified in config file.


=item AmiRNA::DB::Base-E<gt>get_seq_ids_from_sequence($job)

 Returns gene names from job object if job->{sequences} is in fasta or gene-list format.

=back


=head1 DEPENDENCIES

=over

=item L<DBI>

=back


=head1 SEE ALSO

=over

=item L<DBI> 

=item L<DBD::SQLite>

=item L<DBD::Mysql>

=item L<DBD::Pg>

=item L<AmiRNA::Error>

=item L<AmiRNA::DB::Job>

=item L<AmiRNA::DB::Queue>

=item L<AmiRNA::DB::Results>

=item L<AmiRNA::DB::Sequences>

=item L<AmiRNA::Config::ConfigParser>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
