package AmiRNA::DB::Job;

use strict;
use warnings;

our $VERSION = '3.1';

sub new {
	my ( $self, %job_param ) = @_;

	### Check user parameters
	my @allowed_params  = qw/sequences email transcript_library target_count musthave ip trusted annotated offtargets description/;
	my @required_params = qw/sequences email transcript_library target_count musthave ip trusted annotated offtargets description/;
	
	eval {
		my $param_check = new AmiRNA::Config::UserParams();
        $param_check->check(\@allowed_params, \@required_params, \%job_param);
	};
    die if $@;
	
	$self = {
		now                => time,
		sequences          => $job_param{'sequences'},
		email              => $job_param{'email'},
		transcript_library => $job_param{'transcript_library'},
		target_count       => $job_param{'target_count'},
		musthave           => $job_param{'musthave'},
		ip                 => $job_param{'ip'},
		trusted            => $job_param{'trusted'},
		annotated          => $job_param{'annotated'} || "0",
		offtargets         => $job_param{'offtargets'} || "0",
		description        => $job_param{'description'},
		host               => '',
		started            => 0,
		email_sent         => 0,
		id                 => 0
	};
	bless $self;
	return $self;
}

1;

__END__

=head1 NAME

 AmiRNA::DB::Job


=head1 SYNOPSIS

 my $job = AmiRNA::DB::Job->new( ( sequences => "AT1G74280.1,AT1G74290.1,AT1G74300.1", email => "myname\@mail.com",
              transcript_library => "TAIR8_cdna", target_count => 3, musthave => 2, ip => "10.39.123.251", trusted => 1,
              annotated => 1, offtargets => "", description => "A.thaliana, Multi Target: AT1G74280.1,AT1G74290.1,AT1G74300.1" ) );


=head1 DESCRIPTION

 Data structure for AmiRNA-Designer jobs


=head1 METHODS

=over

=item AmiRNA::DB::Job-E<gt>new($sequences, $email, $db, $musthave, $ip, $trusted, $annotated, $offtargets, $description)

 This function constructs a Job object.

 $sequences: string in FASTA format or gene-list (comma seperated gene IDs) format
 $email: user specified email address
 $db: database name
 $musthave: minimum number of genes that have to be targeted by a multi-amiRNA
 $ip: internet protocol adress
 $trusted: indicates that the sequence comes from the internal fasta database instead of being user defined
 $annotated: indicates if the gene is annotated in the selected fasta file
 $offtargets: specify an unlimited amount of accepted off-target genes
 $description: user specified job description

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::DB::Queue>

=item L<AmiRNA::DB::Result>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
