package AmiRNA::DB::Queue;

use strict;
use warnings;

use Sys::Hostname;
use AmiRNA::Error;
use AmiRNA::DB::Job;
use AmiRNA::DB::Base;

our $VERSION = '3.1';

### Constructor
sub new {
	my ( $self, $db ) = @_;
	$self = {
		db => $db,
	};
	bless $self;
	return $self;
}


### Create table 'queue' storing requests for the AmiRNADesigner
sub create_table {
	my ( $self ) = @_;

	my $autoinc = $self->{db}->autoincrement_id();
	
	my $sql     = <<SQL;
CREATE TABLE IF NOT EXISTS queue (
	$autoinc,
	time	           INTEGER,
	sequences          TEXT,
	email	           VARCHAR(100),
	transcript_library VARCHAR(100),
	target_count       INTEGER,
	musthave           INTEGER,
	ip	           VARCHAR(100),
	started            INTEGER,
	host               VARCHAR(50),
	description        VARCHAR(250),
	trusted            INTEGER,
	annotated          INTEGER,
	offtargets         VARCHAR(500)
)
SQL

	eval {
		$self->{db}{dbh}->do($sql) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", 
				subject => "Create table queue for engine " . $self->{db}{engine} . " failed" } );
	};
	die if $@;

	return 1;
}


### Drop table queue
sub drop_table {
	my ($self) = @_;

	eval {
		$self->{db}{dbh}->do('DROP TABLE queue') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Drop table queue failed" } );
	};
	die if $@;

	return 1;
}


### Get total number of jobs in queue
sub get_number_of_jobs_in_queue {
	my ( $self ) = @_;

	my $result = 0;
	eval {
		my $sth = $self->{db}{dbh}->prepare( 'SELECT COUNT(*) FROM queue' ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue" } );
		my $res  = $sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue" } );
		my $data = $sth->fetchrow_arrayref();

		if ( defined $data ) { $result = $data->[0]; }
	};
	die if $@;

	return $result;
}

### Get number of jobs in queue for specified IP adress
sub get_number_of_jobs_for_ip {
	my ( $self, $ip ) = @_;

	my $result = 0;
	eval {
		my $sth = $self->{db}{dbh}->prepare("SELECT COUNT(*) FROM queue WHERE ip=?") or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue WHERE ip=" } );
		my $res = $sth->execute($ip) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue WHERE ip=$ip" } );
		my $data = $sth->fetchrow_arrayref();
		if ( defined $data ) { $result = $data->[0]; }
	};
	die if $@;

	return $result;
}

### Get number of jobs in queue for specified email address
sub get_number_of_jobs_for_email {
	my ( $self, $email ) = @_;

	my $result = 0;
	eval {
		my $sth = $self->{db}{dbh}->prepare("SELECT COUNT(*) FROM queue WHERE email=?") or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue WHERE email=?" } );
		my $res = $sth->execute($email) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "SELECT COUNT(*) FROM queue WHERE email=$email" } );
		my $data = $sth->fetchrow_arrayref();
		if ( defined $data ) { $result = $data->[0]; }
	};
	die if $@;

	return $result;
}

### Get number of unstarted jobs
sub get_number_of_pending_jobs {
	my $self = shift;

	my $result = 0;
	eval {
		my $sth  = $self->{db}{dbh}->prepare('SELECT COUNT(*) FROM queue WHERE started = 0') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "prepare('SELECT COUNT(*) FROM queue WHERE started = 0')" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "execute('SELECT COUNT(*) FROM queue WHERE started = 0')" } );
		my $data = $sth->fetchrow_arrayref();
		if (defined $data ) { $result = $data->[0]; }
	};
	die if $@;

	return $result;
}


### Reset job with specified ID from queue, e.i. set started flag to 0
sub reset_job {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('UPDATE queue SET started=0 WHERE id=?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "UPDATE queue SET started=0 WHERE id=?" } );
		$sth->execute($id) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "UPDATE queue SET started=0 WHERE id=$id" } );
	};
	die if $@;

	return 1;
}

### Delete job with specified ID from queue
sub delete_job {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('DELETE FROM queue WHERE id=?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "DELETE FROM queue WHERE id=?" } );
		$sth->execute($id) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "DELETE FROM queue WHERE id=$id" } );
	};
	die if $@;

	return 1;
}

### Add new AmiRNADesigner job to queue
sub add_job {
	my ( $self, $job ) = @_;
 	$self->cleanup;

	### Check request
	eval {
		### Check number of jobs in queue
		if( $self->get_number_of_jobs_in_queue() >= $self->{db}{MAX_JOB_NUMBER} ) {
			die AmiRNA::Error->new( { errno => "Already too many requests in queue", msg => "Already too many requests in queue", subject => "add_job" } );
		}
		if( $self->get_number_of_jobs_for_ip($job->{ip}) >= $self->{db}{MAX_CALCULATIONS_PER_IP} ) {
			die AmiRNA::Error->new( { errno => "Already too many requests for this IP in queue", msg => "Already too many requests for this IP in queue", subject => "add_job" } );
		}
		if( $self->get_number_of_jobs_for_email($job->{email}) >= $self->{db}{MAX_CALCULATIONS_PER_EMAIL} ) {
			die AmiRNA::Error->new( { errno => "Already too many requests for this email in queue", msg => "Already too many requests for this email in queue", subject => "add_job" } );
		}
	};
	die if $@;

	### Check if musthave is set to reasonable value
	if( $job->{musthave} > $job->{target_count} ) { $job->{musthave} = $job->{target_count}; }

	### Check for correct gene name notation
	if($job->{annotated}) {
		my @genes = split(",", $job->{sequences});
		my @db_genes; # holds gene notation as in sequences table
		
		eval {
			my $sth = $self->{db}{dbh}->prepare(
				"SELECT id FROM sequences WHERE id = ? AND transcript_library = ? LIMIT 1"
			) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Gene name notation check failed (prepare)" } );
		
			foreach my $gene (@genes) {
				$sth->execute($gene, $job->{transcript_library}) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Gene name notation check failed (select)" } );
			
				if( $sth->rows<1 ) {
					die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Gene name not found ($gene)", subject => "Gene name not found ($gene)" } );
				} else {
					while ( my $r = $sth->fetchrow_hashref ) {
						push(@db_genes , $r->{id});
					}
				}
			}
			
			### Overide the gene names
			$job->{sequences} = join(",", @db_genes);
		};
		die if $@;
	}
	
	### Add job to queue
	eval {
		my $sth = $self->{db}{dbh}->prepare(
			"INSERT INTO queue (time, sequences, email, transcript_library, target_count, musthave, ip, started, host, 
			description, trusted, annotated, offtargets) VALUES (?,?,?,?,?,?,?,0,'',?,?,?,?)"
		) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Add job failed" } );

		$sth->execute(
			$job->{now}, $job->{sequences}, $job->{email}, $job->{transcript_library}, $job->{target_count}, $job->{musthave},
			$job->{ip}, $job->{description}, $job->{trusted}, $job->{annotated}, $job->{offtargets}
		) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Add job failed" } );
	};
	die if $@;

	### Finish
	return $self->{db}{dbh}->last_insert_id( undef, undef, 'queue', 'id' );
}


### Delete job from queue if not already started
sub cancel_job {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('DELETE FROM queue WHERE started = 0 AND id=?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Cancel job failed" } );
		$sth->execute( $id ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Cancel job failed" } );
	};
	die if $@;

	return 1;
}


### Flag a job as started
sub start_job {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('UPDATE queue SET started=1, time=?, host=? WHERE id=?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Start job failed" } );
		$sth->execute( time, hostname, $id ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Start job failed" } );
	};
	die if $@;

	return 1;
}


### Get next unstarted job with earliest submission time and set started flag
sub get_next_job {
	my $self = shift;

	my $job = 0;
	eval {
		my $sth  = $self->{db}{dbh}->prepare('SELECT * FROM queue WHERE started = 0 ORDER BY time asc LIMIT 1') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Get job failed" } );
		my $res = $sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Get job failed" } );

		while ( my $r = $sth->fetchrow_hashref ) {
			$job = AmiRNA::DB::Job->new( ( sequences => $r->{sequences}, email => $r->{email}, transcript_library => $r->{transcript_library}, 
				target_count => $r->{target_count}, musthave => $r->{musthave}, ip => $r->{ip}, trusted =>$r->{trusted}, 
				annotated => $r->{annotated}, offtargets => $r->{offtargets}, description => $r->{description} ) );
			$job->{id} = $r->{id};
			$job->{now} = $r->{"time"};
			$job->{host} = hostname;
			return $self->start_job($job->{id});
		}
	};
	die if $@;

	return $job;
}


### Delete all started jobs older than 48 hours
sub cleanup {
	my $self = shift;

	my $secs = time - (48 * 60 * 60);

	eval {
		my $sth  = $self->{db}{dbh}->prepare('DELETE FROM queue WHERE time < ? and started = 1') or 
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete from queue failed (secs=$secs)" });
		$sth->execute($secs) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete from queue failed (secs=$secs)" });
	};
	die if $@;

	return;
}

1;

__END__

=head1 NAME

 AmiRNA::DB::Queue


=head1 SYNOPSIS

 # Create Base object and connect to database
 my $db = AmiRNA::DB::Base->new($configfile);
 $db->connect();

 # Create Queue object
 my $queue = AmiRNA::DB::Queue->new($db);

 # Create new database table for queue
 $queue->create_table();

 # Add new job
 my $job_id = $queue->add_job($job);
 print $queue->get_number_of_jobs_in_queue();

 # Get next pending job
 my $job = $queue->get_next_job();

 # Delete job
 $queue->delete_job($job_id);


=head1 DESCRIPTION

 A module for manipulation of the AmiRNADesigner request queue table.


=head1 METHODS

See L<AmiRNA::DB::Base> for other methods.

=over

=item AmiRNA::DB::Queue-E<gt>new($db)

 Constructor. See L<AmiRNA::DB::Base>

=item AmiRNA::DB::Queue-E<gt>create_table()

 Creates the queue table. Does not overwrite an existing queue table.

=item AmiRNA::DB::Base-E<gt>drop_table()

 Drops the queue table.

=item AmiRNA::DB::Base-E<gt>add_job($job)

 Adds an L<AmiRNA::DB::Job> object to queue. Returns internal id of job in queue.

=item AmiRNA::DB::Base-E<gt>get_next_job()

 Returns an L<AmiRNA::DB::Job> object. It is the next job in queue sorted by submission time.  
 Job is set to started. Returns 0 if no job was found.

=item AmiRNA::DB::Base-E<gt>get_number_of_jobs_in_queue()

 Returns the number of jobs in queue. Considers started and unstarted jobs.

=item AmiRNA::DB::Base-E<gt>get_number_of_jobs_for_ip($ip)

 Returns the number of jobs in queue with the specified ip address. Considers started and unstarted jobs.

=item AmiRNA::DB::Base-E<gt>get_number_of_jobs_for_email($email)

 Returns the number of jobs in queue with the specified email address. Considers started and unstarted jobs.

=item AmiRNA::DB::Base-E<gt>start_job($id)

 Tell queue that AmiRNADesigner started to work on job with internal $id.

=item AmiRNA::DB::Base-E<gt>delete_job($id)>

 Deletes job with internal ijob ID $id from queue. Returns 1 if job is deleted, "0E0"
 if no job was found or 0 if execute statement failed.

=item AmiRNA::DB::Base-E<gt>cancel_job($id, $ip)>

 Deletes job from queue with internal id = $id if it is not yet started (started = 0).
 Returns 0 if execution was not possible. "0EO" if no job was found, 1 if a job
 was successfully deleted from queue.

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::DB::Base> 

=item L<AmiRNA::DB::Job>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>,

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
