package AmiRNA::DB::Results;

use strict;
use warnings;

use AmiRNA::Error;
use XML::Dumper;
use AmiRNA::DB::Base;
use AmiRNA::Designer::MiRNA;

our $VERSION = '3.1';


### Constructor
sub new {
	my ( $self, $db ) = @_;
	$self = {
		db => $db
	};
	bless $self;
	return $self;
}


### Create table 'results' storing artificial miRNAs designed by AmiRNADesigner
sub create_table {
	my ( $self ) = @_;

	my $autoinc = $self->{db}->autoincrement_id();
	my $longtext = 'LONGTEXT';
	if ( $self->{db}{engine} eq 'PostgreSQL' ) { $longtext = 'TEXT'; }

	my $sql = <<SQL
CREATE TABLE IF NOT EXISTS results (
	$autoinc,
	start_time         INTEGER,
	end_time           INTEGER,
	seq_ids            VARCHAR(500),
	email              VARCHAR(100),
	transcript_library VARCHAR(100),
	target_count       INTEGER,
	musthave           INTEGER,
	annotated          INTEGER,
	offtargets         VARCHAR(500),
	ip                 VARCHAR(50),
	result_obj         $longtext,
	description        VARCHAR(250),
	trusted            INTEGER,
	email_sent         INTEGER,
	host               VARCHAR(30)
)
SQL
	;

	eval {
		$self->{db}{dbh}->do($sql) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error",
			subject => "Create table queue for engine " . $self->{db}{engine} . " failed" } );
	};
	die if $@;

	return 1;
}


### Drop table 'results'
sub drop_table {
	my ( $self ) = @_;

	eval {
		$self->{db}{dbh}->do('DROP TABLE results') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Drop table results failed" } );
	};
	die if $@;

	return 1;
}

### Delete result entry
sub delete_result {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('DELETE FROM results WHERE id=?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete result failed" } );
		$sth->execute($id) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete result failed" } );
	};
	die if $@;

	return 1;
}


### Store result object from amiRNADesigner in XML format
sub store {
	my ( $self, $job, $results ) = @_;
	my $dump    = new XML::Dumper;

	eval {
		my $sth = $self->{db}{dbh}->prepare('INSERT INTO results (start_time, end_time, seq_ids, email, transcript_library, target_count, musthave,
			annotated, offtargets, ip, result_obj, description, trusted, email_sent, host) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)' ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Store in results failed" } );

		my @seq_ids = $self->{db}->get_seq_ids_from_sequence($job);
		if (! defined $job->{host} ) { $job->{host} = ''; }

		$sth->execute( $job->{now}, time, join ( ',', sort @seq_ids ), $job->{email}, $job->{transcript_library}, $job->{target_count}, $job->{musthave},
			$job->{annotated}, $job->{offtargets}, $job->{ip}, $dump->pl2xml($results), $job->{description}, $job->{trusted}, 0, $job->{host} ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Store in results failed" } );
	};
	die if $@;

	return $self->{db}{dbh}->last_insert_id( undef, undef, 'results', 'id' );
}


### Fetch result object from amiRNADesigner
sub get {
	my ( $self, $id ) = @_;
	my $dump = new XML::Dumper;
	my $sth;

	eval {
		$sth = $self->{db}{dbh}->prepare('SELECT * FROM results WHERE id = ?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Get results failed" } );
		$sth->execute($id) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Get results failed" } );
	};
	die if $@;

	while ( my $r = $sth->fetchrow_hashref ) {
		my $job = AmiRNA::DB::Job->new( ( sequences => $r->{seq_ids}, email => $r->{email}, transcript_library => $r->{transcript_library}, 
			target_count => $r->{target_count}, musthave => $r->{musthave}, ip => $r->{ip}, trusted => $r->{trusted}, 
			annotated => $r->{annotated}, offtargets => ($r->{offtargets} || 0), description => $r->{description} ) );

		$job->{id}         = $r->{id};
		$job->{now}        = $r->{start_time};
		$job->{email_sent} = $r->{email_sent};
		$job->{host}       = $r->{host};

		my $result_obj = $dump->xml2pl( $r->{result_obj} );
		my @micros = pop(@{ $result_obj });

		### Resolve the serialized xml2perl object
		my @miRNA_objs = ();
		foreach my $n (@micros) {
			foreach my $m (@{ $n }) {
				#TODO here seed hits einfuegen!!!
				my $micro = AmiRNA::Designer::MiRNA->new( $m->{mirna_pattern}, $m->{seed_hits}, $m->{absolute}, $m->{maybe}, $m->{relaxed_filter_32},
					$m->{relaxed_filter_28_32}, $m->{mutations}, $m->{optimal_energy}, $m->{targets}, $m->{target_sequences},
					$m->{flanking_seqs}, $m->{starting_positions}, $m->{type} );
				$micro->{score} = $m->{score};

				push( @miRNA_objs, $micro );
			}
		}

		return ( $job, \@miRNA_objs );
	}

	return 0;
}



### Test if amiRNADesigner results for a job already exist in results table
sub check_if_already_in_db {
	my ( $self, $job ) = @_;
	my $sth;
	my @seq_ids = $self->{db}->get_seq_ids_from_sequence($job);
	eval {
		$sth = $self->{db}{dbh}->prepare('SELECT * from results WHERE seq_ids = ? and musthave = ? and trusted = 1 and transcript_library = ?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Check results failed" } );
		$sth->execute( join ( ',', @seq_ids ), $job->{musthave}, $job->{transcript_library} ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Check results failed" } );
	};
	die if $@;

	while ( my @row = $sth->fetchrow_array ) { return $row[0]; }

	return 0;
}


### Record that email was sent successfully
sub set_email_sent {
	my ( $self, $id ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare('UPDATE results SET email_sent = 1 WHERE id = ?') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to results failed" } );
    		$sth->execute($id) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to results failed" } );
	};
	die if $@;

	return 1;
}


### Get next undelivered job
sub get_undelivered_job {
	my ( $self ) = @_;
	my $sth;

	eval {
		$sth  = $self->{db}{dbh}->prepare('SELECT * FROM results WHERE email_sent = 0 ORDER BY id') or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to results failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to results failed" } );
	};
	die if $@;

	while ( my $r = $sth->fetchrow_hashref ) {
		my $job = AmiRNA::DB::Job->new( ( sequences => $r->{seq_ids}, email => $r->{email}, transcript_library => $r->{transcript_library},
			target_count => $r->{target_count}, musthave => $r->{musthave}, ip => $r->{ip}, trusted => $r->{trusted}, 
			annotated => $r->{annotated}, offtargets => $r->{offtargets}, description => $r->{description} ) );

		$job->{id}         = $r->{id};
		$job->{now}        = $r->{start_time};
		$job->{email_sent} = $r->{email_sent};
		$job->{host}       = $r->{host};
		return $job;
	}

	return 0;
}
1;

__END__

=head1 NAME

 AmiRNA::DB::Results


=head1 SYNOPSIS

 # Create Base object and connect to database
 my $db = AmiRNA::DB::Base->new($configfile);
 $db->connect();

 # Create Result object
 my $results = AmiRNA::DB::Results->new($db);

 # Create new database table for results
 $results->create_table();

 # Store new result in table
 my $id = $results->store($job, $results_xml);

 # Get existing result object from table
 my ( $job, $end_time, $xml_results ) = $results->get($id);


=head1 DESCRIPTION

 A module for communications with the AmiRNADesigner database results table.


=head1 METHODS

See L<AmiRNA::DB::Base> for other methods.

=over

=item AmiRNA::DB::Results-E<gt>new($db)

 Constructor. See L<AmiRNA::DB::Base>


=item AmiRNA::DB::Results-E<gt>create_table()>

 Creates the results table. Does not overwrite an existing results table.


=item AmiRNA::DB::Results-E<gt>drop_table()

 Drops the results table.


=item AmiRNA::DB::Results-E<gt>store($job, $results)

 Stores results in database. See AmiRNA::DB::Job


=item AmiRNA::DB::Results-E<gt>get($result_id)

 Returns results for $result_id.


=item AmiRNA::DB::Results-E<gt>check_if_already_in_db($job)

 Checks if a result for the specified job is already in database. Returns 
 result ID if job was found, 0 otherwise. Searches only for trusted jobs.


=item AmiRNA::DB::Results-E<gt>set_email_sent($result_id)

 Sets flag "email_sent" for the the result with the specified result_id.


=item AmiRNA::DB::Results-E<gt>get_undelivered_job()

 Returns next job where email_sent = 0. 

 while (my $job = $results->get_undelivered_job()) {
     # send email to job->email
     $results->set_email_sent($job->id);
 }
 
=back


=head1 DEPENDENCIES

=over

=item L<XML::Dumper>

=back



=head1 SEE ALSO

=over

=item L<AmiRNA::DB::Base>

=item L<AmiRNA::DB::Job>

=item L<AmiRNA::Designer::MiRNA>

=item L<AmiRNA::Error>

=back



=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
