package AmiRNA::DB::Sequences;

use strict;
use warnings;

use AmiRNA::Error;
use AmiRNA::DB::Base;

our $VERSION = '3.1';

### Constructor
sub new {
	my ( $self, $db ) = @_;
	$self = {
		db => $db,
	};
	bless $self;
	return $self;
}


### Create table Sequences
sub create_table {
	my ( $self ) = @_;
	my $sql =<<SQL
CREATE TABLE IF NOT EXISTS sequences (
	transcript_library VARCHAR(100),
	id VARCHAR(50),
	cdna_length int(7),
	cdna TEXT,
	utr3  TEXT,
	PRIMARY KEY(transcript_library, id)
)
SQL
;
	eval {
		$self->{db}{dbh}->do($sql) or die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error",
			subject => "Create table sequence for engine " . $self->{db}{engine} . " failed" } );
	};
	die if $@;

	return 1;
}

### Drop table Sequences
sub drop_table {
	my ( $self ) = @_;
	eval {
		$self->{db}{dbh}->do('DROP TABLE sequences') or	die AmiRNA::Error->new( { errno => $DBI::errstr, 
			msg => "Database error", subject => "Drop table sequence failed" } );
	};
	die if $@;

	return 1;
}


### Load fasta file into table
sub store_fasta_files {
	my ( $self, $file ) = @_;
	my $fasta_header = "NA";
	my $cdna = "";

	my @folder_structure = split("/", $file);
	my $transcript_library = pop(@folder_structure);

	eval {
		open FASTA, $file or die AmiRNA::Error->new( { errno => "I/O", msg => "I/O error", subject => "Cannot open file $file" } );
		while ( <FASTA> ) {
			chomp;
			if($_ =~ />/) {
				### store gene
				if($fasta_header ne "NA") {
					my $sql_ins;
					if($self->{db}{engine} =~ /^SQLite$/i) {
						$sql_ins = " OR ";
					} elsif($self->{db}{engine} =~ /^MySQL$/i) {
						$sql_ins = "";
					}
					
					my $sth = $self->{db}{dbh}->prepare( "INSERT $sql_ins IGNORE INTO sequences (transcript_library, id, cdna_length, cdna) VALUES (?,?,?,?)" ) or 
						die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Add sequence failed: $sql_ins" } );
					$sth->execute( $transcript_library, $fasta_header, length($cdna), $cdna ) or
						die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Add sequence failed: $sql_ins" } );
				}

				# Get gene id
				$fasta_header = $_;
				$fasta_header =~ s/^>//g;
				$fasta_header =~ s/^\s+//g;
				$fasta_header =~ s/\s.*//g;

				# Reset cDNA sequence
				$cdna = '';
			}
			else {
				$cdna .= $_;
			}
		}
		close FASTA;
	};
	die if $@;

	return 1;
}


### Delete fasta sequence with specified ID from sequences
sub delete_library {
	my ( $self, $transcript_library ) = @_;

	eval {
		my $sth = $self->{db}{dbh}->prepare("DELETE FROM sequences WHERE transcript_library = '$transcript_library'") or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete library failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Delete library failed" } );
	};
	die if $@;

	return 1;
}


sub get_cdna {
	my ( $self, $transcript_library, $id ) = @_;
	my %results = ();

	eval {
		my $sql = "SELECT id, cdna FROM sequences WHERE transcript_library = \"$transcript_library\" AND id like \"%$id%\"";
		
		my $sth  = $self->{db}{dbh}->prepare( $sql ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or 
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			$results{ $r->{id} } = $r->{cdna};
		}
	};
	die if $@;

	return %results;                            
}

sub get_utr3 {
	my ( $self, $transcript_library, $id ) = @_;
	my %results = ();

	eval {
		my $sth  = $self->{db}{dbh}->prepare("SELECT id, utr3 FROM sequences WHERE transcript_library = '$transcript_library' AND id like '%$id%'" ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			$results{ $r->{id} } = $r->{utr3};
		}
	};
	die if $@;

	return %results;
}

sub get_cdna_length{
	my ( $self, $transcript_library, $id ) = @_;
	my %results = ();

	eval {
		my $sth  = $self->{db}{dbh}->prepare("SELECT id, cdna_length FROM sequences WHERE transcript_library = '$transcript_library' AND id like '%$id%'" ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			$results{ $r->{id} } = $r->{cdna_length};
		}
	};
	die if $@;

	return %results;
}

sub get_ids {
	my ( $self, $transcript_library ) = @_;
	my %results = ();

	eval {
		my $sth  = $self->{db}{dbh}->prepare("SELECT id, cdna_length FROM sequences WHERE transcript_library = '$transcript_library'" ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			$results{ $r->{id} } = $r->{cdna_length};
		}
	};
	die if $@;

	return %results;
}

sub get_entry {
	my ( $self, $transcript_library, $id ) = @_;
	my %results = ();

	eval {
		my $sth  = $self->{db}{dbh}->prepare("SELECT id, cdna_length, cdna, utr3 FROM sequences WHERE transcript_library like $transcript_library AND id LIKE %$id%" ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			my @buffer = ( $r->{cdna_length}, $r->{cdna}, $r->{utr3} );
			$results{ $r->{id} } = \@buffer;
		}
	};
	die if $@;

	return %results;
}

sub check_transcript {
	my ( $self, $transcript_library, $id ) = @_;
	my $result = 0;

	eval {
		my $sth  = $self->{db}{dbh}->prepare("SELECT id  FROM sequences WHERE transcript_library = \"$transcript_library\" AND id = \"$id\" ORDER BY id LIMIT 1" ) or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		$sth->execute() or
			die AmiRNA::Error->new( { errno => $DBI::errstr, msg => "Database error", subject => "Request to sequences failed" } );
		while ( my $r = $sth->fetchrow_hashref ) {
			$result = $r->{id};
		}
	};
	die if $@;

	return $result;
}
1;
__END__

=head1 NAME

 AmiRNA::DB::Sequences - module for the sequences table

=head1 SYNOPSIS

 # Create Base object and connect to database
 my $db = AmiRNA::DB::Base->new();
 $db->connect();

 # Create Sequences object
 my $seq_db = AmiRNA::DB::Sequences->new($db);

 # Create new database tabe for sequences
 $seq_db->create_table();

 # Add new library (fasta file, full path required) to sequences table
 $seq_db->store_fasta_files( $transcript_library);

 # Delete complete transcript libray from sequences
 $seq_db->delete_library( $transcript_library );

 # get ids from database (Atg12346 will match internal id ATG12346.7)
 my %results = $seq_db->get_ids( $transcript_library);

 # Get cDNA sequence of gene
 my %results = $seq_db->get_cdna( $transcript_library, $id);

 # Get 3'UTR of gene
 my %results = $seq_db->get_utr3( $transcript_library, $id);

 # Get cdna length
 my %results = $seq_db->get_cdna_length( $transcript_library, $id);


=head1 DESCRIPTION

 A module for the sequence table. 


=head1 METHODS

See L<AmiRNA::DB::Base> for other methods.

=over

=item AmiRNA::DB::Sequences-E<gt>new($db)

 Constructor. See L<AmiRNA::DB::Base>

=item AmiRNA::DB::Sequences-E<gt>create_table()

 Creates the sequences table. Does not overwrite an existing sequences table.

=item AmiRNA::DB::Sequences-E<gt>drop_table()

 Drops the sequences table.

=item AmiRNA::DB::Sequences-E<gt>store_fasta_files($transcript_library)

 Stores the sequences of the transcript library file 
 $transcript_library (in fasta format) in the sequences table.


=item AmiRNA::DB::Sequences-E<gt>delete_library($transcript_library)

 Removes sequences from a transcript library from the sequences table.
 The name of the transcript library is identical to the original fasta 
 file name (without folder structure).


=item AmiRNA::DB::Sequences-E<gt>get_ids($transcript_library)

 Returns all transcript IDs of a transcript library.


=item AmiRNA::DB::Sequences-E<gt>get_cdna($transcript_library, $id)

 Returns cDNA sequence of a specified transcript from a specified
 transcript library.

                             
=item AmiRNA::DB::Sequences-E<gt>get_utr3($transcript_library, $id)

 Returns 3'UTR sequence of a specified transcript from a specified
 transcript library.


=item AmiRNA::DB::Sequences-E<gt>get_cdna_length($transcript_library, $id)

 Returns the length of a specified transcript from a specified
 transcript library.


=item AmiRNA::DB::Sequences-E<gt>get_entry($transcript_library, $id)>

 Returns cDNA, 3'UTR and cDNA length of a specified transcript 
 from a specified transcript library.

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::DB::Base>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <ester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
