package AmiRNA::Designer::AmiRNADesigner;

use strict;
use warnings;

use List::Util;

use AmiRNA::Error;
use AmiRNA::TargetSearch::Vmatch;
use AmiRNA::TargetSearch::GenomeMapper;
use AmiRNA::TargetSearch::ViennaRNA;
use AmiRNA::Designer::AmiRNAPattern;
use AmiRNA::Designer::MiRNA;
use AmiRNA::Config::UserParams;

our $VERSION = '3.1';

####################################################################################################
### Constructor
# user_param:
#	configfile         => Config file location
#	transcript_library => Transcript library (index file) name
#	target_fasta       => Fasta string of the target genes
#	target_genes       => List of target gene names
#	musthave           => Minimum number of target genes to be silenced by one amiRNA. Default: all
#	annotated          => Gene is part of the transcript library?
#	offtargets         => List of acceptable off-target genes
#	temperature        => select folding temperature, 0 - 99 degree C
#	relaxed_rules      => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
#	accelerate_design  => < 0 | 1 | 2 >, 0 = full , 1 = fast, 2 = superfast
#	flanking           => show x bases upstream and downstream of target region

sub new {
	my ($self, %user_param) = @_;


	### Check user parameters
	my @allowed_params  = qw/configfile transcript_library target_fasta target_genes musthave annotated offtargets temperature relaxed_rules accelerate_design flanking/;
	my @required_params = qw/transcript_library musthave annotated temperature relaxed_rules accelerate_design flanking/;

        eval {
                my $param_ckeck = new AmiRNA::Config::UserParams();
                $param_ckeck->check(\@allowed_params, \@required_params, \%user_param);
        };
        die if $@;

	### Turn off calculation of amiRNA flanking region for non annotated genes (currently needed for Vmatch)
	if($user_param{annotated} == 0) {
		$user_param{flanking} = 0;
	}


	### Set up TargetSearch
	my %target_search_params = (
		transcript_library => $user_param{transcript_library},
		mismatches         => 5,
		allow_gaps         => 0,
		strand             => 'rev',
		remove_isoform     => 1,
		one_hit_per_gene   => 1,
		calculate_filter   => 1,
		apply_filter       => 0,
		calculate_folding  => "passed",
		folding_backend    => "RNAcofold",
		temperature        => $user_param{temperature},
		cutoff             => 70,
		relaxed            => 1,
		flanking           => $user_param{flanking},
		showdesc           => 50,
		show_gene_length   => 0,
		sort_mode          => 'ea',
	);


	my $targetSearch;
	my $searchOptions;

	eval {
		$searchOptions = AmiRNA::TargetSearch::SearchOptions->new($user_param{configfile});
		if( $searchOptions->get_search_program() eq "vmatch" ) {
			$targetSearch = AmiRNA::TargetSearch::Vmatch->new($searchOptions, %target_search_params);
		}
		elsif( $searchOptions->get_search_program() eq "genomemapper" ) {
			$targetSearch = AmiRNA::TargetSearch::GenomeMapper->new($searchOptions, %target_search_params);
		}
	};
	die if $@;
		

	$self = {
		targetSearch        => $targetSearch,
		configfile          => $user_param{configfile},
		t_lib               => $user_param{transcript_library},
		target_fasta        => $user_param{target_fasta},
	        target_genes        => $user_param{target_genes},
		musthave            => $user_param{musthave},
		annotated           => $user_param{annotated},
		offtargets          => $user_param{offtargets},
		relaxed_rules       => $user_param{relaxed_rules},
		accelerate_design   => $user_param{accelerate_design},
		flanking            => $user_param{flanking},
		temperature         => $user_param{temperature},
		num_targets         => 0,
		type                => 'single',
		accept_score        => 10,
		too_many_offtargets => 30,
	        genes               => {},
	        candidates          => [],
	        miRNA_objs          => [],
	        tested_patterns     => {},
		offtargets_hash     => {}
	};
	bless $self;
	return $self;
}


####################################################################################################
### Design miRNAs for single targets
sub design {
	my ( $self ) = @_;

	### Create hash of accepted offtargets
	my @tmp_off_targets = ();
	if( $self->{offtargets} ) { @tmp_off_targets = split(/,/, $self->{offtargets}) };

	foreach my $tmp_off_target (@tmp_off_targets) {
		$tmp_off_target =~ s/^\s+//g;
		$tmp_off_target =~ s/\s+.*//g;
		$tmp_off_target =~ s/\..*//g;
		$self->{offtargets_hash}{$tmp_off_target} = 1;
	}

	### Restrict computation time to 24 hours
	my $start_hour = time();
	$start_hour = $start_hour / 3600;

	### Read gene name and sequence from FastA file
	$self->read_genes();

	my @candidate_pattern = ();
	foreach my $seqid ( keys %{ $self->{genes} } ) {
		eval {
			my @tmp_pattern = AmiRNA::Designer::AmiRNAPattern->search_pattern( $self->{genes}{$seqid}, $self->{relaxed_rules} );
			@candidate_pattern = (@candidate_pattern, @tmp_pattern);
		};
		die if $@;
	}

	# Check if at least 'musthave' genes are targeted by the pattern (only used for multi-target amiRNA design)
	if( $self->{num_genes} > 1 ) {
		my $included = 0;
		for (my $i = 0; $i <  @candidate_pattern; $i++) {
			my $included = 0;
			$self->{targetSearch}{user_param}{calculate_folding} = "non";
			$self->{targetSearch}->search($candidate_pattern[$i]);

			foreach my $target ( @{ $self->{targetSearch}{searchResults}{targets} } ) {
				my $target_id = $target->{seq_id};
				$target_id =~ s/\s+.*//g;
				$target_id =~ s/\..*//g;
				if ( exists $self->{genes}{$target_id} ) { 
					$included++; 
				}
			}
			if( $included < $self->{musthave} ) { splice(@candidate_pattern, $i, 1) };
		}
	}

	### Do less mutations/tests if more candidates are available
	if    ( @candidate_pattern >= 120 ) { 
		$self->{too_many_offtargets} = 7;
		$self->{accelerate_design} = 2;

		if($self->{type} eq 'multi') { 
			$self->{accept_score} = 27; 
		}
		else { 
			$self->{accept_score} = 15; 
		}
	}
	elsif ( @candidate_pattern >= 80 )  { 
		$self->{too_many_offtargets} = 8;
		$self->{accelerate_design} = 1;

		if($self->{type} eq 'multi') { 
			$self->{accept_score} = 26;
		}
		else { 
			$self->{accept_score} = 12; 
		}
	}
	elsif ( @candidate_pattern >= 40 )  { 
		$self->{too_many_offtargets} = 12;
		$self->{accelerate_design} = 1;

		if($self->{type} eq 'multi') {
			$self->{accept_score} = 25;
		}
		else {
			$self->{accept_score} = 10; 
		}
	}
	elsif ( @candidate_pattern >= 20 )  {
		$self->{too_many_offtargets} = 18;
		$self->{accelerate_design} = 1;

		if($self->{type} eq 'multi') {
			$self->{accept_score} = 23;
		}
		else {
			$self->{accept_score} = 9;
		}
	}
	else {
		$self->{too_many_offtargets} = 30;
		$self->{accelerate_design} = 0;

		if($self->{type} eq 'multi') {
			$self->{accept_score} = 20;
		}
		else {
			$self->{accept_score} = 7;
		}
	}

	### Evaluate and optimize amiRNA candidates
	my $counter = 0;
	my @shuffled_pattern = List::Util::shuffle(@candidate_pattern);
	foreach my $pattern (@shuffled_pattern) {
		push( @{ $self->{candidates} }, $pattern );

		# Test the miRNA candidate
		eval {
			my ( $occurrence, $included, $score) = $self->test_mirna( $pattern, $pattern, 0);

			if ( ($occurrence < $self->{too_many_offtargets} + $included - 3) && ($included >= $self->{musthave}) ) {
				$counter++;
				$self->optimize($pattern);
			}
		};
		die if $@;

		if($counter > 100) { last; }

		# Check time
		my $current_seq = time();
		my $time_diff   = ( $current_seq / 3600 ) - $start_hour;
		if ( $time_diff > 24 ) { last; }
	}

}



####################################################################################################
### Calculate number of off-targets using miRNA filters, TargetSearch, RNA folding
sub test_mirna {
    	my ( $self, $candidate, $perfect_candidate, $mutations ) = @_;

	if( 	(! exists $self->{tested_patterns}{$candidate} ) &&
		( AmiRNA::Designer::AmiRNAPattern->sequence_composition($candidate) )
	) {

		$self->{tested_patterns}{$candidate} = 1;
		my $count = 0;
		my $included = 0;
		my $success = 1;
		my %targets_hash = ();
		my $targets = '';
        	my %target_seqs_hash = ();
		my $target_seqs = '';
		my %flanking_seqs_hash = ();
		my $flanking_seqs = '';
		my %target_beg_hash = ();
		my %target_end_hash = ();
		my %isoform = ();
		my $starting_positions = '';
		my $optimal_energy = 0;
		my $final_score = 9999;


		### Test if pattern is worth of further customization: estimate number of hits not considering hybridization
		$self->{targetSearch}{user_param}{calculate_folding} = "non";
		eval {
			$self->{targetSearch}->search($candidate);
		};
		die if $@;
		my $absolute = $self->{targetSearch}{searchResults}{passed_targets};


		### Too many off-targets, do not consider this amiRNA at all
		if( $absolute > ($self->{too_many_offtargets} + $self->{num_genes}) ) { return(9999, 0, 9999); }


		### Not too many off-targets estimated, calculate exact number of off-targets considering hybridization energy
		# Perform full TargetSearch and binding energy calculation
		$self->{targetSearch}{user_param}{calculate_folding} = "passed";
		eval {
			$self->{targetSearch}->search($candidate);
		};
		die if $@;
		$optimal_energy = $self->{targetSearch}{searchResults}{pdG};

            	# Loop through targets
            	foreach my $target ( @{ $self->{targetSearch}{searchResults}{targets} } ) {
			if( $target->{message} eq "passed" ) {
                		$count++;
				my $target_id = $target->{seq_id};
				$target_id =~ s/\s+.*//g;
                		$target_id =~ s/\..*//g;

                		if ( exists $self->{genes}{$target_id} ) {
                   			$included++;
					$flanking_seqs_hash{$target_id} = $target->{sequence};
					$targets_hash{$target_id} = $target->{dG};
					$target_seqs_hash{$target_id} = $target->{target_string};
					$target_beg_hash{$target_id} = $target->{begin};
					$target_end_hash{$target_id} = $target->{end};
					$isoform{$target_id} = $target->{fasta_header};
					$isoform{$target_id} =~ s/\s.*//g;
                		}
                		else {
					if(exists $self->{offtargets_hash}{$target_id} ) {
						# Nothing to do
					}
					else { $success = 0; }
                		}
			}
            	}


		### If miRNA pattern is proofed, create new miRNA object
		if ( ( $success == 1 ) && ( $included >= $self->{musthave} || $self->{annotated} == 0) ) {

			# Calculate binding energy if gene is not annotated
			if($self->{annotated} == 0) {
				my $target_seq = reverse($perfect_candidate);
				$target_seq =~ tr/ACGTUacgtu/TGCAATGCAA/;

				my $mirna_energy;
				eval{
					my $folding = AmiRNA::TargetSearch::ViennaRNA->new($self->{configfile}, $self->{temperature});
					$mirna_energy = $folding->RNAcofold( $target_seq, $candidate);
				};
				die if $@;

				foreach( keys %{$self->{genes}} ) {
					$targets_hash{$_} = $mirna_energy;
					$target_seqs_hash{$_} = $target_seq;
					$flanking_seqs_hash{$_} = $target_seq;
					$target_beg_hash{$_} = 0;
					$target_end_hash{$_} = 0;
					$isoform{$_} = 1;
				}
			}

        	
			### Write targets to string including upstream and downstream flanking sequence (if requested)
			foreach my $target_id ( keys %targets_hash ) {
				$targets .= "$target_id\t" . $targets_hash{$target_id} . "\t";
				$target_seqs .= $target_seqs_hash{$target_id} . "\t";
				$starting_positions .= $target_beg_hash{$target_id} . "\t";
				$flanking_seqs .= $flanking_seqs_hash{$target_id} . "\t";
			}
			chop($target_seqs); chop($targets); chop($flanking_seqs); chop($starting_positions);


            		### Calculate number of targets with relaxed rules and dG between -28 and -32 or dG < -32
            		$self->{targetSearch}{user_param}{calculate_folding} = "all";
			$self->{targetSearch}{user_param}{cutoff} = 55;
			$self->{targetSearch}{user_param}{relaxed} = 1;

			eval {
				$self->{targetSearch}->search($candidate);
			};
			die if $@;

            		my $relaxed_filter_32 = 0;
            		my $relaxed_filter_28_32 = 0;
			my $MM3_candidate = 0;
            		
			foreach my $target ( @{ $self->{targetSearch}{searchResults}{targets} } ) {
				my $target_id = $target->{seq_id};
				$target_id =~ s/\..*//g;
                		if( ! exists $self->{genes}{$target_id} ) {
					if($target->{message} eq "Possible candidate") {
						$MM3_candidate++;	
					}

                		    	if($target->{dG} < -32) { $relaxed_filter_32++; }
                		    	elsif ($target->{dG} < -28) { $relaxed_filter_28_32++; }
                		}
            		}

	    		### Add new amiRNA
	    		eval {
	    			my $micro = AmiRNA::Designer::MiRNA->new( $candidate, 0, $absolute, $MM3_candidate, $relaxed_filter_32,
					$relaxed_filter_28_32, $mutations, $optimal_energy, $targets, $target_seqs, $flanking_seqs, 
					$starting_positions, $self->{type} );
				push( @{ $self->{miRNA_objs} }, $micro );
				$final_score = $micro->{score};
			};
			die if $@;
		}

        	return ($count, $included, $final_score);
    	}
    	else { 
		return (9999, 0, 9999); 
	}
}



####################################################################################################
# Mutate promising miRNA targets until no unwanted hits occure
sub optimize {

	### Process and initialize variable ############################################################
	my ( $self, $cand_seq )  = @_;
	my @consensus        = ();
	my %test_candidates  = ();
	my @bases = ( 'A', 'C', 'G', 'T' );


	### Fetch all occurrencies of the pattern and calculate consensus sequence #####################
	$self->{targetSearch}{user_param}{calculate_folding} = "non";
	$self->{targetSearch}->search($cand_seq);

	foreach my $target ( @{ $self->{targetSearch}{searchResults}{targets} } ) {
		if ( $target->{filter_passed} == 1 ) {
			push( @consensus, $target->{target_string} );
		}
	}
	my %consensus_pos = ();
	eval {
		%consensus_pos = AmiRNA::Designer::AmiRNAPattern->get_consensus( 13, 20, @consensus );
	};
	die if $@;


	### Mutate miRNA candidate in position 13-21 creating a single mismatch ########################
	my $min_mutation_pos = 12;
	if($self->{accelerate_design} == 1) { $min_mutation_pos = 13; }
	if($self->{accelerate_design} > 1) { $min_mutation_pos = 14; }

	for ( my $i = $min_mutation_pos; $i < 21; $i++ ) {
		if( $i == 18 ) { next; } # Pos 19 has to be 'C', do not mutate

		# If all targets have similar base at position
		if(exists $consensus_pos{$i} ) {
			my $new_cand_seq = $cand_seq;
			my $nucleotide = substr( $new_cand_seq, $i, 1 );

			# No Purin-Purin mismatch at position 16
			if( $i == 15 ) {
				if( $nucleotide ne 'C' ) {
					$new_cand_seq = $cand_seq;
					substr( $new_cand_seq, $i, 1, 'C' );
					$test_candidates{$new_cand_seq} = $i;
				}
				elsif($nucleotide ne 'T' ) {
					$new_cand_seq = $cand_seq;
					substr( $new_cand_seq, $i, 1, 'T' );
					$test_candidates{$new_cand_seq} = $i;
				}
			}

			# G or C Prefered for positions 15,17,18 (five prime instability)
			elsif($i >= 14 && $i < 18) { 
				if ( $nucleotide ne 'G' ) {
					$new_cand_seq = $cand_seq;
					substr( $new_cand_seq, $i, 1, 'G' );
					$test_candidates{$new_cand_seq} = $i;
				}
				if ( $nucleotide ne 'C' ) {
					$new_cand_seq = $cand_seq;
					substr( $new_cand_seq, $i, 1, 'C' );
					$test_candidates{$new_cand_seq} = $i;
				}
			}

			# Position 13, 14, 20, 21
			else {
				# test all base changes (slow)
				if( $self->{accelerate_design} < 2 ) {
					if( $nucleotide ne 'A' ) {
						$new_cand_seq = $cand_seq;
						substr( $new_cand_seq, $i, 1, 'A' );
						$test_candidates{$new_cand_seq} = $i; 
					}
					if( $nucleotide ne 'C' ) {
						$new_cand_seq = $cand_seq;
						substr( $new_cand_seq, $i, 1, 'C' );
						$test_candidates{$new_cand_seq} = $i;
					}
					if( $nucleotide ne 'G' ) {
						$new_cand_seq = $cand_seq;
						substr( $new_cand_seq, $i, 1, 'G' );
						$test_candidates{$new_cand_seq} = $i;
					}
					if( $nucleotide ne 'T' ) {
						$new_cand_seq = $cand_seq;
						substr( $new_cand_seq, $i, 1, 'T' );
						$test_candidates{$new_cand_seq} = $i;
					}
				}
				# choose random base change (fast)
				else {
					my $base = $nucleotide;
					while( $base eq $nucleotide) {
						my $base_index = int(rand(4));
						$base = $bases[$base_index];
					}
					substr( $new_cand_seq, $i, 1, $base );
				}
			}
		}
		
		# Not all targets have similar base at position
		else {
			if( $i == 15 ) { next; } # No mismatch at position 16 prefered

			my $new_cand_seq = $cand_seq;
			my $nucleotide = substr( $new_cand_seq, $i, 1 );

			# G or C Prefered for positions 15,17,18 (five prime instability)
			if($i >= 14 && $i < 18) {
				if    ( $nucleotide eq 'C' || $nucleotide eq 'A' ) { substr( $new_cand_seq, $i, 1, 'G' ); }
				elsif ( $nucleotide eq 'G' || $nucleotide eq 'T' ) { substr( $new_cand_seq, $i, 1, 'C' ); }
			}

			# Position 13, 14, 20, 21, choose random base change
			else {
				my $base = $nucleotide;
				while( $base eq $nucleotide) {
					my $base_index = int(rand(4));
					$base = $bases[$base_index];
				}
				substr( $new_cand_seq, $i, 1, $base );
			}

			$test_candidates{$new_cand_seq} = $i;
		}
	}


	### Mutate miRNA candidate in positions 14-21 creating a second mismatch ###################
	foreach my $key_string ( keys %test_candidates ) {
		my $single_mutation_pos = $test_candidates{$key_string};

		# If consensus position was found between 13 and 21
		for ( my $i = 13; $i < 21; $i++ ) {
			if( 	($i == $single_mutation_pos) || 
				($i == $single_mutation_pos - 1) ||
				($i == $single_mutation_pos + 1) || 
				($i == 18) ) { next; }

			# If all targets have similar base at position
			if(exists $consensus_pos{$i} ) {
				my $new_cand_seq = $key_string;
				my $nucleotide = substr( $new_cand_seq, $i, 1 );

				# No Purin-Purin mismatches at posiiton 16
				if( $i == 15 ) {
					if( $nucleotide ne 'C' ) {
						$new_cand_seq = $key_string;
						substr( $new_cand_seq, $i, 1, 'C' );
						$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
					}
					elsif($nucleotide ne 'T' ) {
						$new_cand_seq = $key_string;
						substr( $new_cand_seq, $i, 1, 'T' );
						$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
					}
				}

				# G or C Prefered for positions 15, 17, 18 (five prime instability)
				elsif($i >= 14 && $i < 18) {
					if ( $nucleotide ne 'G' ) {
						$new_cand_seq = $key_string;
						substr( $new_cand_seq, $i, 1, 'G' );
						$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
					}
					if ( $nucleotide ne 'C' ) {
						$new_cand_seq = $key_string;
						substr( $new_cand_seq, $i, 1, 'C' );
						$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
					}
				}

	                        # Position 13, 14, 20, 21
	                        else {
	                                # test all base changes (slow)
	                                if( $self->{accelerate_design} < 2 ) {
	                                        if( $nucleotide ne 'A' ) {
	                                                $new_cand_seq = $key_string;
	                                                substr( $new_cand_seq, $i, 1, 'A' );
	                                                $test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
	                                        }
	                                        if( $nucleotide ne 'C' ) {
	                                                $new_cand_seq = $key_string;
	                                                substr( $new_cand_seq, $i, 1, 'C' );
	                                                $test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
	                                        }
	                                        if( $nucleotide ne 'G' ) {
	                                                $new_cand_seq = $key_string;
	                                                substr( $new_cand_seq, $i, 1, 'G' );
	                                                $test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";;
	                                        }
	                                        if( $nucleotide ne 'T' ) {
	                                                $new_cand_seq = $key_string;
	                                                substr( $new_cand_seq, $i, 1, 'T' );
	                                                $test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
	                                        }
	                                }
					# choose random base change (fast)
					else {
						my $base = $nucleotide;
						while( $base eq $nucleotide) {
							my $base_index = int(rand(4));
							$base = $bases[$base_index];
						}
						substr( $new_cand_seq, $i, 1, $base );
						$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
	                                }
	                        }
			}

			### In depth search (accelerate = 0), all secondary positions are mutated
			elsif( $self->{accelerate_design} == 0 ) {
				if( $i == 15 ) { next; } # No mismatch at position 16

				my $new_cand_seq = $key_string;
				my $nucleotide = substr( $new_cand_seq, $i, 1 );

				# Positions 13-21 except pos 16, 18
				my $base = $nucleotide;
				while( $base eq $nucleotide) {
					my $base_index = int(rand(4));
					$base = $bases[$base_index];
				}
				substr( $new_cand_seq, $i, 1, $base );
				
				$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
			}

			### Fast search, randomly mutate secondary positions (1 out of 4)
			else {
				if( $i == 15 ) { next; } # No mismatch at position 16

				my $dice_sides = 4;
				if($i > 18) { $dice_sides = 3; }
				my $dice = int(rand($dice_sides));
				if($dice != 1) { next; } 

				my $new_cand_seq = $key_string;
				my $nucleotide = substr( $new_cand_seq, $i, 1 );

				my $base = $nucleotide;
				while( $base eq $nucleotide ) {
					my $base_index = int(rand(4));
					$base = $bases[$base_index];
				}
				substr( $new_cand_seq, $i, 1, $base );
				$test_candidates{$new_cand_seq} = "$single_mutation_pos,$i";
			}
		}


		### Mutate random second and third position
		if( $self->{accelerate_design} < 2 ) {
			my $rand_pos = -1;
			while( 1 ) {
				$rand_pos = int(rand(9)) + 12;
				if(     ($rand_pos != 18) && ($rand_pos != 15) &&
					($rand_pos != $single_mutation_pos) &&
					($rand_pos != $single_mutation_pos - 1) &&
					($rand_pos != $single_mutation_pos + 1)
				){ last; }
			}
			my $rand_pos_2 = -1;
			while( 1 ) {
				$rand_pos_2 = int(rand(9)) + 12;
				if( 	($rand_pos_2 != 18) && ($rand_pos != 15) &&
					($rand_pos_2 != $rand_pos) && 
					($rand_pos_2 != $single_mutation_pos) &&
					($rand_pos_2 != 18) && ($rand_pos_2 != 15)
				) { last; }
			}

			my $new_cand_seq = $key_string;
			my $nucleotide = substr( $new_cand_seq, $rand_pos, 1 );
			my $base = $nucleotide;
			while( $base eq $nucleotide ) {
				my $base_index = int(rand(4));
				$base = $bases[$base_index];
			}
			substr( $new_cand_seq, $rand_pos, 1, $base );

			$nucleotide = substr( $new_cand_seq, $rand_pos_2, 1 );
			$base = $nucleotide;
			while( $base eq $nucleotide ) {
				my $base_index = int(rand(4));
				$base = $bases[$base_index];
			}
			substr( $new_cand_seq, $rand_pos_2, 1, $base );

			my $dice = int(rand(6));
			if( (! exists $test_candidates{$new_cand_seq}) && ($dice < 3) ) {
				$test_candidates{$new_cand_seq} = "$single_mutation_pos,$rand_pos,$rand_pos_2";
			}
		}
	}


	### Test all mutated miRNAs for uniquness ######################################################
	my $optimal_amiRNA = 0;
	foreach my $mutated_candidate ( sort { length($test_candidates{$a}) cmp length($test_candidates{$b}) } keys %test_candidates ) {
		my @elem = split(/,/, $test_candidates{$mutated_candidate});
		my $mismatch_num = @elem;

		eval {
			my ( $occurrence, $included, $score) = $self->test_mirna( $mutated_candidate, $cand_seq, $mismatch_num );
			if( $score <= $self->{accept_score} ) { $optimal_amiRNA++; }
		};
		die if $@;
		if($optimal_amiRNA == 2) { last; }
	}
}


####################################################################################################
### Read all genes of a species from a FastA-file and store in hash
sub read_genes {
    	my ( $self ) = @_;

	### No fasta provided, read sequence for gene names from index
	if($self->{target_fasta} eq '') {

		# Check consistency
		if( $self->{target_genes} eq '' ) { die $!; }
		if( $self->{annotated} == 0 ) { die $!; }

		# Get sequence
		my @seqids = split (/[,\n]/, $self->{target_genes});
		foreach my $seqid ( @seqids ) {
			$seqid =~ s/^>//g;
			$seqid =~ s/^\s+//g;
			$seqid =~ s/\s.*//g;
			eval {
				my ($fetched_id, $sequence) = $self->{targetSearch}->get_sequence($seqid);
				$self->{target_fasta} .= ">$fetched_id\n$sequence\n";
			};
			die if $@;
		}
	}
	
	### Parse fasta
	my $current_gene = '';
	my @fasta_lines = split(/\n/, $self->{target_fasta});
	for( my $i = 0; $i < @fasta_lines; $i++) {
		chomp($fasta_lines[$i]);

		if($fasta_lines[$i] =~ />/) {
			$current_gene = $fasta_lines[$i];
			$current_gene =~ s/^>//g;
			$current_gene =~ s/^\s+//g;
			$current_gene =~ s/\s.*//g; # remove everthing but the name
			$current_gene =~ s/\..*//g; # remove isoform
		}
		else {
			$self->{genes}{$current_gene} .= $fasta_lines[$i];
		}
	}
	$self->{num_genes} = scalar(keys %{$self->{genes}});
	if( $self->{num_genes} != 1 ) { 
		$self->{type} = 'multi';
		$self->{accept_score} = 25;
	}
}

1;

__END__

=head1 NAME

 AmiRNA::Designer::AmiRNADesigner


=head1 DESCRIPTION

 Designs amiRNAs to specifically silence a single or multiple genes.


=head1 SYNOPSIS

 # User param hash
 my %user_param = (
        configfile         => "/etc/AmiRNA/AmiRNA.xml",
        transcript_library => "A_thaliana_TAIR8.fa",
        target_fasta       => '',
        target_genes       => 'AT1G01090',
        musthave           => 1,
        annotated          => 1,
        offtargets         => '',
        temperature        => 23,
        relaxed_rules      => 1,
        accelerate_design  => 1,
        flanking           => 3,
 );

 # Create new designer
 my $amiRNAdesigner = AmiRNA::Designer::AmiRNADesigner->new(%user_param);

 # Create miRNAs for genes in Fasta-file
 $amiRNAdesigner->design();

 # Fetch candidates and proofed miRNAs as arrays
 foreach my $amiRNA ( @{ $amiRNAdesigner->{miRNA_objs} } ) {
    print   $amiRNA->{mirna_pattern} . "\t" . $amiRNA->{absolute} . "\t" . $amiRNA->{maybe} . "\t" .
            $amiRNA->{relaxed_filter_32} . "\t" . $amiRNA->{relaxed_filter_28_32} . "\t" .
            $amiRNA->{mutations} . "\t" . $amiRNA->{optimal_energy} . "\t" . $amiRNA->{targets} . "\t" .
            $amiRNA->{type} . "\t" . $amiRNA->{score} . "\n";
 }		


=head1 METHODS

=over

=item AmiRNA::Designer::AmiRNADesigner-E<gt>new(%user_param)

 Constructs a AmiRNADesigner object.

 User parameter hash:
       configfile         => Config file location
       transcript_library => Transcript library (index file) name
       target_fasta       => Fasta string of the target genes
       target_genes       => List of target gene names
       musthave           => Minimum number of target genes to be silenced by one amiRNA. Default: all
       annotated          => Gene is part of the transcript library?
       offtargets         => List of acceptable off-target genes
       temperature        => select folding temperature, 0 - 99 degree C
       relaxed_rules      => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
       accelerate_design  => < 0 | 1 | 2 >, 0 = full , 1 = fast, 2 = superfast
       flanking           => show x bases upstream and downstream of target region


=item AmiRNA::Designer::AmiRNADesigner-E<gt>design()

 Designs amiRNAs heuristically.

 The following attributes will be populated by calling design:

     candidates      =>  Array of unfiltered amiRNA candidates

     tested_patterns => Hash of all sequences that were already tested using L<AmiRNA::TargetSearch>

     miRNA_objs      =>  Array of L<AmiRNA::Designer::MiRNA> objects containing filterd (high quality) amiRNAs


=item AmiRNA::Designer::AmiRNADesigner-E<gt>test_mirna($candidate, $perfect_candidate, $mutations)

 Find all targets of an amiRNA candidate using the L<AmiRNA::TargetSearch> modules to distinguish 
 specific and unspecific amiRNAs (relating to the specified target genes)

=item AmiRNA::Designer::AmiRNADesigner-E<gt>optimize( $candidate_sequence )

 Mutates a candidate sequence to make it more specific to the target genes (heuristic).

=item AmiRNA::Designer::AmiRNADesigner-E<gt>optimize()

 Reads the sequence of the target genes either from provided fasta file or from a 
 database using L<AmiRNA::TargetSearch::Vmatch> or L<AmiRNA::TargetSearch::GenomeMapper>


=back

=head1 DEPENDENCIES

=over

=item L<List::Util>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::TargetSearch::Vmatch>

=item L<AmiRNA::TargetSearch::GenomeMapper>

=item L<AmiRNA::TargetSearch::ViennaRNA>

=item L<AmiRNA::Designer::AmiRNAPattern>

=item L<AmiRNA::Designer::MiRNA>

=item L<AmiRNA::Config::UserParams>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut
# vim: ft=perl sw=4 ts=4 expandtab
