package AmiRNA::Designer::AmiRNAPattern;

use strict;
use warnings;

use Data::Types qw(:all);
use AmiRNA::Error;

our $VERSION = '3.1';


########################################################################################
### Get positions with equal base in multiple sequences
sub get_consensus
{
	my ($self, $start, $end, @in_seq) = @_;

	eval {
		if( (! is_int($start)) || (! is_int($end)) || (@in_seq == 0) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "bad parameters" } );
		}
	};
	die if $@;
	
	my %cons_positions = ();
	my $opt = $in_seq[0];

	for(my $i = $start; $i <= $end; $i++) {
		my $mismatch = 0;
		foreach (@in_seq) {
			if( substr($opt, $i, 1) ne  substr($_, $i, 1)) {
				$mismatch++;
			}
		}
		if( $mismatch == 0 ) { $cons_positions{$i} = 1; }
	}
	return (%cons_positions)
}


########################################################################################
### Search for good miRNA target patterns in a gene sequence
sub search_pattern
{
	my ($self, $gene_seq, $relaxed) = @_;

	eval {
		if( (! is_string($gene_seq)) || (! is_int($relaxed)) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "bad parameters" } );
		}
	};
	die if $@;

    my $start = 0;
    my $end = 21;
    my $position=21;
    my @predicted_patterns = ();

    while($position <= length($gene_seq)) {
        my $word = substr($gene_seq, $start, $end);
        $word =~ tr/uUacgt/TTACGT/;

        if($relaxed) {
            if($word =~ /[ATCG]{2}[G]{1}[ATCG]{8}[ACGT]{1}[ATCG]{9}/ig) {
                my $rev_word = reverse($word);
                $rev_word =~ tr/ACGT/TGCA/;
                substr($rev_word, 0, 1, 'T');
                if( $self->sequence_composition($rev_word) ){
                    push(@predicted_patterns, $rev_word);
                }
            }
        }
        else {
            if($word =~ /[ACG]{1}[ATCG]{1}[G]{1}[ATCG]{8}[T]{1}[ATCG]{9}/ig) {
                my $rev_word = reverse($word);
                $rev_word =~ tr/ACGT/TGCA/;
                substr($rev_word, 0, 1, 'T');
                if( $self->sequence_composition($rev_word) ){
                    push(@predicted_patterns, $rev_word);
                }
            }
        }
        $start++; $position++;
    }
    return(@predicted_patterns);
}

########################################################################################
### Wrapper for gc_content and five_prime_instability functions
sub sequence_composition
{
    my ($self, $seq) = @_;
    my $success = 1;
    if( ! $self->gc_content($seq) ) {$success = 0;}
    if( ! $self->five_prime_instability($seq) ) {$success = 0;}
    return($success);
}


########################################################################################
#### Returns true if GC content of miRNA is average and miRNA is no homo/di/tri-polymere
sub gc_content {
    my ($self, $test_pattern) = @_;
    my @dna = split('', $test_pattern);
    my $a=0;
    my $t=0;
    my $c=0;
    my $g=0;

    foreach my $base (@dna) {
        if( $base eq 'A') { $a++;}
        if( $base eq 'T') { $t++;}
        if( $base eq 'C') { $c++;}
        if( $base eq 'G') { $g++;}
    }
    my $g_c = $c + $g;

    if( ($g_c>5) && ($g_c<14) && ($a>0) && ($t>0) && ($c>0) && ($g>0) ) { return 1; }
    else { return 0; }
}


#############################################################################
### Returns true if five prime end of miRNA has less C/G than three prime end
sub five_prime_instability {
    my ($self, $test_pattern) = @_;
    my $FivePrime = substr($test_pattern,0,5);
    my $ThreePrime = substr($test_pattern,length($test_pattern)-7,5);

    my $NgcFP = 0;
    my $NgcTP = 0;

    for(my $i =0 ; $i<5 ; $i++){
        my $char = substr($FivePrime,$i,1);
        if($char=~/[GC]/){
            $NgcFP++;
        }
    }
    for(my $i =0 ; $i<5 ; $i++){
        my $char = substr($ThreePrime,$i,1);
        if($char=~/[GC]/){
            $NgcTP++;
        }
    }
    if($NgcFP < $NgcTP){
        return 1;
    }
    else {
        return 0;
    }
}

1;

__END__

=head1 NAME

 AmiRNA::Designer::AmiRNAPattern

=head1 SYNOPSIS

 # Find positions without mismatches in a multiple alignment of short sequences
 my %consensus_positions = AmiRNA::Designer::AmiRNAPattern->get_consensus( 13, 20, @consensus );

 # Detect typical miRNA patterns
 my @tmp_pattern = AmiRNA::Designer::AmiRNAPattern->search_pattern( $gene_id, $relaxed_rules );

 # Check five prime instability (deprecated?)
 my $five_prime_instable = AmiRNA::Designer::AmiRNAPattern->five_prime_instability($seq)

=head1 DESCRIPTION

 Detects typical miRNA patterns or consensus positions in short sequences. 
 Helper module for AmiRNA::Designer::AmiRNADesigner.

=head1 METHODS

=over

=item AmiRNA::Designer::AmiRNAPattern-E<gt>get_consensus( $start, $end, @consensus)

 Finds position without mismatches in the alignment of all sequences stored in @consensus.
 Considers only positions between $start and $end.


=item AmiRNA::Designer::AmiRNAPattern-E<gt>search_pattern( $start, $end, @consensus)

 Searches for sequences of 21bp that have specific nucleotides in specific positions (duh).
 As an example plant miRNA have a high overrepresentation of 'U' in position 1 and 'G' in
 position 19 resulting in a favourable incorporation into the AGO1 protein.


=item AmiRNA::Designer::AmiRNAPattern-E<gt>five_prime_instability( $sequence )

 Tests if a sequence is five prime instable. For details see

 Functional siRNAs and miRNAs exhibit strand bias.
 Khvorova A, Reynolds A, Jayasena SD.
 Cell. 2003 Oct 17;115(2):209-16. Erratum in: Cell. 2003 Nov 14;115(4):505.

=back


=head1 DEPENDENCIES

=over

=item L<Data::Types>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Designer::AmiRNADesigner>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
