package AmiRNA::Designer::OligoDesign;

use strict;
use warnings;

use AmiRNA::Error;

our $VERSION = '3.1';

sub new {
	my $object = shift;
	my $self = {};
	bless($self, $object);
	return $self;
}

# Design oligos for vector RS300 from A. thaliana using endogenous miRNA miR319.
# Vector was designed by Rebecca Schwab and published in Schwab et al. Plant Cell 2005.
sub RS300
{
	my ($self, $mirna_sen) = @_;
	
	# preprocess
	$mirna_sen =~ tr/uacgt/UACGT/;
	$mirna_sen =~ tr/U/T/;
	$mirna_sen =~ s/[\s\n\r]//g;
	
	eval { 
		if ($mirna_sen!~/^[ACTG]+$/) {
			die AmiRNA::Error->new( { errno => "mirna sequence error ($mirna_sen)", msg => "The miRNA sequence contains characters other than the allowed A,G,C,T or U", 
			subject => "RS300" });
		}
	};
	die if ($@);
	
	# miRNA*sense = revcom miRNAsense 
	my $star_sen = reverse($mirna_sen);
	$star_sen =~ tr/ACGT/TGCA/;
	
	# change bases at positions 3, 14, 21 in miRNA*sense
	my $nucleotide = substr($star_sen, 2, 1);
	$nucleotide =~ tr/CTAG/ACCA/;
	substr($star_sen, 2, 1, $nucleotide);
	
	$nucleotide = substr( $star_sen, 13, 1);
	$nucleotide =~ tr/CTAG/GATC/;
	substr($star_sen, 13, 1, $nucleotide);

	$nucleotide = substr($star_sen, 20, 1);
	$nucleotide =~ tr/A/T/;
	substr($star_sen, 20, 1, $nucleotide);

	# Calculate second strand (revcom) of both miRNAs
	my $mirna_rev = reverse($mirna_sen);
	$mirna_rev =~ tr/ACGT/TGCA/;
	my $star_rev  = reverse($star_sen);
	$star_rev  =~ tr/ACGT/TGCA/;
		
	$mirna_sen = "ga" . $mirna_sen . "tctctcttttgtattcc"; 
	$mirna_rev = "ga" . $mirna_rev . "tcaaagagaatcaatga";
	$star_sen = "ga"  . $star_sen  . "tcacaggtcgtgatatg";
	$star_rev = "ga"  . $star_rev  . "tctacatatatattcct";
	
	return($mirna_sen, $mirna_rev, $star_sen, $star_rev); # TODO: This should be a hash
}


# Design oligos for vector RS300 from A. thaliana using endogenous miRNA miR319.
# Vector was designed by Rebecca Schwab and published in Schwab et al. Plant Cell 2005.
sub RS300_21mer
{
        my ($self, $mirna_sen) = @_;

        # preprocess
        $mirna_sen =~ tr/uacgt/UACGT/;
        $mirna_sen =~ tr/U/T/;
        $mirna_sen =~ s/[\s\n\r]//g;

        eval {
                if ($mirna_sen!~/^[ACTG]+$/) {
                        die AmiRNA::Error->new( { errno => "mirna sequence error ($mirna_sen)", msg => "The miRNA sequence contains characters other than the allowed A,G,C,T or U",
                        subject => "RS300" });
                }
        };
        die if ($@);

        # miRNA*sense = revcom miRNAsense 
        my $star_sen = reverse($mirna_sen);
        $star_sen =~ tr/ACGT/TGCA/;

        # change bases at positions 3, 14, 21 in miRNA*sense
        my $nucleotide = substr($star_sen, 2, 1);
        $nucleotide =~ tr/CTAG/ACCA/;
        substr($star_sen, 2, 1, $nucleotide);

        $nucleotide = substr( $star_sen, 13, 1);
        $nucleotide =~ tr/CTAG/GATC/;
        substr($star_sen, 13, 1, $nucleotide);

        $nucleotide = substr($star_sen, 20, 1);
        $nucleotide =~ tr/A/T/;
        substr($star_sen, 20, 1, $nucleotide);

        # Calculate second strand (revcom) of both miRNAs
        my $mirna_rev = reverse($mirna_sen);
        $mirna_rev =~ tr/ACGT/TGCA/;
        my $star_rev  = reverse($star_sen);
        $star_rev  =~ tr/ACGT/TGCA/;

        $mirna_sen = "ga" . $mirna_sen . "ctctcttttgtattcca";
        $mirna_rev = "ag" . $mirna_rev . "tcaaagagaatcaatga";
        $star_sen  = "ag" . $star_sen  . "tcacaggtcgtgatatg";
        $star_rev  = "ga" . $star_rev  . "ctacatatatattccta";

        return($mirna_sen, $mirna_rev, $star_sen, $star_rev); # TODO: This should be a hash
}


# Designs oligos for vector pChlamyRNA2 from Chlamydomonas reinhardtii
# Code was provided by Frank Schwach. Vector was designed by Atilla Molnar.
sub pChlamyRNA2
{
    my ($self, $mirna) = @_;

    # A few predefined bits of sequence for the amirna hairpin
    my $hp_fwd_5end='ctagt';
    my $hp_rev_5end='ctagc';
    my $hp_middle = 'tctcgctgatcggcaccatgggggtggtggtgatcagcgcta' ;
    my $hp_fwd_3end='g';
    my $hp_rev_3end='a';

    $mirna = uc($mirna);
    $mirna =~ s/[\s\n\r]//g;
    
    eval { 
		if ($mirna!~/^[ACTGU]+$/) {
			die AmiRNA::Error->new( { errno => "mirna sequence error ($mirna)", msg => "The miRNA sequence contains characters other than the allowed A,G,C,T or U ($mirna)", 
			subject => "RS300" });
		}
	};
	die if ($@);
   
    # construct oligos as follows
    # Cr_amiFor 5â ctagtRRRRRRRRRRRRRRRRRRRRRtctcgctgatcggcaccatgggggtggtggtgatcagcgctaMMMMMMMMMMMMMMMMMMMMMg 3â
    # Cr_amiRev 5â CTAGC - r e v e r s e  c o m p l e m e n t  o f  u n d e r l i n e d  s e q u e n c e  - A 3â
    # M - miRNA sequence, R â reverse complement of miRNA sequence (miRNA*)
    $mirna =~s/U/T/g;
    my $mirna_star = reverse($mirna);
    $mirna_star =~tr/ACTG/TGAC/;
    
    my $fwd_oligo_middle  = $mirna_star. $hp_middle . $mirna ;
    
    # reverse complement the middle section
    my $rev_oligo_middle = reverse($fwd_oligo_middle);
    $rev_oligo_middle =~tr/ACTGactg/TGACtgac/;
    
    # put the oligos together
    my $fwd_oligo = $hp_fwd_5end . $fwd_oligo_middle . $hp_fwd_3end ;
    my $rev_oligo = $hp_rev_5end . $rev_oligo_middle . $hp_rev_3end ;
    
    # Introduce mismatches in miRNA star at positions corresponding to miRNA positions 5/6:
    # If possible, use two G:U mismatches, which were also present in the "template" for the
    # amirna (can only do this if miRNA pos 5/6 are GG,GT,TG or TT). Otherwise use a full
    # mismatch at pos 5 by changing the corresponding base on the miRNA star to the same as 
    # the miRNA.
    my $use_2_gu = 0 ;
    if (substr($mirna,4,2) eq 'GG') {
      substr($mirna_star,-6,2) = 'TT' ;
      $use_2_gu = 1 ;
    } elsif (substr($mirna,4,2) eq 'TT') {
      substr($mirna_star,-6,2) = 'GG' ;
      $use_2_gu = 1 ;
    } elsif (substr($mirna,4,2) eq 'GT') {
      substr($mirna_star,-6,2) = 'GT' ;
      $use_2_gu = 1 ;
    } elsif (substr($mirna,4,2) eq 'TG') {
      substr($mirna_star,-6,2) = 'TG' ;
      $use_2_gu = 1 ;
    } else {
      substr($mirna_star,-5,1) = substr($mirna,4,1) ;
    }


    # put the mismatched oligos together
    my $fwd_oligo_middle_mm  = $mirna_star. $hp_middle . $mirna ;

    # reverse complement the middle section
    my $rev_oligo_middle_mm = reverse($fwd_oligo_middle_mm);
    $rev_oligo_middle_mm =~tr/ACTGactg/TGACtgac/;

    my $fwd_oligo_mm = $hp_fwd_5end . $fwd_oligo_middle_mm . $hp_fwd_3end ;
    my $rev_oligo_mm = $hp_rev_5end . $rev_oligo_middle_mm . $hp_rev_3end ;

    # Tell user what type of mismatch we used
    my $mismatch_msg = $use_2_gu ? '(two G:U mismatches at miRNA pos 5 and 6)' : '(one same-base mismatch at miRNA pos 5)' ;

	return($mirna, $mismatch_msg, $fwd_oligo_mm, $rev_oligo_mm); # TODO: This should be a hash
}

# Design oligos for vector NW55 from Oryza sativa.
# Vector was designed by Norman Warthmann and published in Warthmann et al. PLoS One 2008.
sub NW55
{
    my ($self, $mirna_sen) = @_;

    # preprocess
    $mirna_sen =~ tr/uacgt/UACGT/;
    $mirna_sen =~ tr/U/T/;
    $mirna_sen =~ s/[\s\n\r]//g;
    
	eval { 
		if ($mirna_sen !~ /^[ACTG]+$/) {
			die AmiRNA::Error->new( { errno => "mirna sequence error", msg => "The miRNA sequence contains characters other than the allowed A,G,C,T or U", subject => "NW55" });
		}
	};
	die if ($@);
    

    # miRNA*sense = revcom miRNAsense 
    my $star_sen = reverse($mirna_sen);
    $star_sen =~ tr/ACGT/TGCA/;

    # change bases at positions 6 and 10 in miRNA*sense
    my $nucleotide = substr( $star_sen, 5, 1);
    $nucleotide =~ tr/CTAG/GATC/;
    substr($star_sen, 5, 1, $nucleotide);

    $nucleotide = substr( $star_sen, 9, 1);
    $nucleotide =~ tr/CTAG/GATC/;
    substr($star_sen, 9, 1, $nucleotide);

    # calculate second strand (revcom) of both miRNAs
    my $mirna_rev = reverse($mirna_sen);
    $mirna_rev =~ tr/ACGT/TGCA/;
    my $star_rev  = reverse($star_sen);
    $star_rev  =~ tr/ACGT/TGCA/;

    $mirna_sen = "ag" . $mirna_sen . "caggagattcagtttga";
    $mirna_rev = "tg" . $mirna_rev . "ctgctgctgctacagcc";
    $star_sen = "ct"  . $star_sen  . "ttcctgctgctaggctg";
    $star_rev = "aa"  . $star_rev  . "agagaggcaaaagtgaa";

    return($mirna_sen, $mirna_rev, $star_sen, $star_rev); # TODO: This should be a hash
}

1;

__END__

=head1 NAME

 AmiRNA::Designer::OligoDesign


=head1 DESCRIPTION

 Oligo design for amiRNA cloning protocol.


=head1 METHODS

=over 

=item AmiRNA::Designer::OligoDesign-E<gt>RS300($amirna)

 Returns oligos for the A. thaliana vector RS300 published in:

 Highly specific gene silencing by artificial microRNAs in Arabidopsis.
 Schwab R, Ossowski S, Riester M, Warthmann N, Weigel D.
 Plant Cell. 2006 May;18(5):1121-33. Epub 2006 Mar 10.


=item AmiRNA::Designer::OligoDesign-E<gt>RS300_21mer($amirna)

 Update for RS300 using 21mers instead of 20mers as basis. Might have better
 silencing capacity but is currently untested.


=item AmiRNA::Designer::OligoDesign-E<gt>pChlamyRNA2($amirna)

 Returns oligos for the Chlamydomonas reinhardtii  vector pChlamyRNA2 published in:

 Highly specific gene silencing by artificial microRNAs in the unicellular alga Chlamydomonas reinhardtii.
 Molnar A, Bassett A, Thuenemann E, Schwach F, Karkare S, Ossowski S, Weigel D, Baulcombe D.
 Plant J. 2009 Jan 17


=item AmiRNA::Designer::OligoDesign-E<gt>NW55($amirna)

 Returns oligos for the rice vector NW55 published in:

 Highly specific gene silencing by artificial miRNAs in rice.
 Warthmann N, Chen H, Ossowski S, Weigel D, Hervé P.
 PLoS ONE. 2008 Mar 19;3(3):e1829.

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Designer::AmiRNADesigner>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
