package AmiRNA::TargetSearch::GenomeMapper;

use strict;
use warnings;
use Error;
use Data::Types qw(:all);
use File::Temp;
use AmiRNA::TargetSearch::SearchOptions;
use AmiRNA::TargetSearch::SearchResult;
use AmiRNA::TargetSearch::MIRNAFilter;
use AmiRNA::Config::ConfigParser;
use AmiRNA::DB::Base;
use AmiRNA::DB::Sequences;
use AmiRNA::Config::UserParams;

our $VERSION = '3.1';

### Constructor
# user_param:
#      transcript_library => transcript library name
#      mismatches         => number of allowed mismatches 
#      allow_gaps         => allow gaps in alignment, < 0 | 1 >
#      strand             => < fwd | rev | both >
#      remove_isoform     => Count gene only once and neglect additional isoforms, < 0 | 1 >
#      one_hit_per_gene   => report only first hit of a gene, < 0 | 1 >
#      calculate_filter   => calculate results of MiRNA filter < 0 | 1 >
#      apply_filter       => remove filtered hits from SearchResults
#      calculate_folding  => use free hybridization energy for filtering < all | passed | non >
#      folding_backend    => select RNA folding tool< RNAup | RNAcofold >
#      temperature        => select folding temperature, 0 - 99 degree C
#      cutoff             => cutoff for imperfect  to perfect energy, 0 to 99
#      relaxed            => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
#      flanking           => show x bases upstream and downstream of target region,
#      showdesc           => show number of characters rfom fasta header, 0 - 200, 0 = show all
#      show_gene_length   => store length of target gene for visualization. If not set gene length will be set to 0.
#      sort_mode          => select sort modde, < ea | ed | ga | gd | la | ld >

sub new {
	my ($self, $searchOptions, %user_param) = @_;
	my $miRNAfilter;
	my $seq_db;	

	
	### Check user parameters
 	my @allowed_params  = qw/allow_gaps calculate_folding cutoff calculate_filter apply_filter folding_backend mismatches one_hit_per_gene relaxed remove_isoform show_gene_length showdesc sort_mode strand temperature transcript_library flanking/;
	my @required_params  = qw/allow_gaps calculate_folding cutoff calculate_filter apply_filter folding_backend mismatches one_hit_per_gene relaxed remove_isoform show_gene_length showdesc sort_mode strand temperature transcript_library flanking/;

	eval {
		my $param_ckeck = new AmiRNA::Config::UserParams();
		$param_ckeck->check(\@allowed_params, \@required_params, \%user_param);
	};
	die if $@;


	### Set up MiRNA filter and database connection
	eval {
		if( ! defined $searchOptions ) {
			die AmiRNA::Error->new( { errno => "GenomeMapper error", msg => "Wrong parameters for TargetSearch.", subject => "" } );
		}

		### Filter object
		$miRNAfilter  = AmiRNA::TargetSearch::MIRNAFilter->new( $searchOptions->{configfile}, $user_param{folding_backend}, 
			$user_param{temperature}, $user_param{cutoff}, $user_param{relaxed}, $user_param{apply_filter} );

		### Sequence database table object
		my $db = AmiRNA::DB::Base->new( $searchOptions->{configfile} );
		$db->connect();
		$seq_db = AmiRNA::DB::Sequences->new($db);
	};
	die if $@;

	my %available_sort_modes = (
		ga  => 'ascending order of dG',
		gd  => 'descending order of dG',
		ea  => 'ascending order of Score',
		ed  => 'descending order of Score',
	);

	$self = {
		searchOptions        => $searchOptions,
		user_param           => \%user_param,
		available_sort_modes => \%available_sort_modes,
		miRNAfilter          => $miRNAfilter,
		seq_db               => $seq_db,
		searchResults        => ''
	};

	bless($self);
	return $self;
}

sub search {
	my ( $self, $query ) = @_;

	### Check input
	$self->check_user_param();

	### Generate command string
	# Edit distance
	my $edit_distance = '';
	if( $self->{user_param}{mismatches} > 0 ) {
		if( $self->{user_param}{allow_gaps} == 0 ) {
			$edit_distance = " -E " . $self->{user_param}{mismatches} . " -M " . $self->{user_param}{mismatches} . " -G 0";
		}
		else {
			$edit_distance = " -E " . $self->{user_param}{mismatches} . " -M " . $self->{user_param}{mismatches} . " -G " . $self->{user_param}{mismatches};
		}
	}

	# Prepare query sequence, align fwd, rev or both
	$query =~ tr/uU/tT/;
	$query = uc($query);
	my $gm_query = $query;
	my $fwdrev = " -r ";
	if( $self->{user_param}{strand} eq "rev" ) {
		$gm_query = reverse($query);
		$gm_query =~ tr/ACGTUacgtu/TGCAATGCAA/;
	}

	### Print temporary reads flat file and map list file
	my $tmp_query = new File::Temp(DIR => $self->{searchOptions}{general_param}{tmppath});
	my $tmp_out = "$tmp_query.maplist";
	print $tmp_query "1\t$gm_query\t0\tNA\tNA\tNA";

	### Upstream and downstream sequence
	my $flanking = "";
	if( $self->{user_param}{flanking} != 0 ) {
		$flanking = " -flanking " . $self->{user_param}{flanking} . " ";
	}

	# Concatenate command
	my $cmd = $self->{searchOptions}{general_param}{execpath} . '/genomemapper' .
		' -i ' . $self->{searchOptions}{general_param}{indexpath} . "/" .  $self->{user_param}{transcript_library} .
		' -q ' . $tmp_query .
		' -o ' . $tmp_out . 
		$edit_distance . $fwdrev . $flanking . ' -e -a ';

	### Search
	system($cmd);

	### Parse and store results 
	$self->parse_results($query, $tmp_out);

	### Run filter if selected
	if( $self->{user_param}{calculate_filter} == 1 ) {
	$self->{miRNAfilter}->filter($self->{searchResults}, $self->{user_param}{calculate_folding});
	}

	### Sort bt dG if selected
	if( substr( $self->{user_param}{sort_mode}, 0, 1 ) eq 'g' ) {
		$self->{searchResults}->sort_by_dg();
	}
	elsif( substr( $self->{user_param}{sort_mode}, 0, 1 ) eq 'e' ) {
		$self->{searchResults}->sort_by_score();
	}
    
	### Delete temporary output file
	unlink $tmp_out if -e $tmp_out;

	# Finish
	return 1;
}


### Parse and store results
sub parse_results {
	my ( $self, $query, $tmp_out ) = @_;

	my %id_exist_check = ();
	$self->{searchResults}  = AmiRNA::TargetSearch::SearchResult->new($query);

	open VMOUT, $tmp_out or die "Cannot open vmatch result files\n";

	### Read all hits
	my %hits_hash = ();
	while ( <VMOUT> ) {
		chomp;
		my @elem = split(/\s+/, $_);
		$hits_hash{ $elem[1] + 1 } = $_;
	}

	### Store sorted array of all hits
	my @hits_array = ();
	foreach ( sort {$a <=> $b} keys %hits_hash ) {
		my @tmp = ( $_, $hits_hash{$_});
		push( @hits_array, \@tmp );
	}

	### Parse fasta headers from translation file
	my $trans_file = $self->{searchOptions}{general_param}{indexpath} . "/" .  $self->{user_param}{transcript_library} . ".trans";
	my $TRANS;

	eval {
		open $TRANS, $trans_file or
			die AmiRNA::Error->new( { errno => "file I/O error", msg => "TargetSearch error", subject => "cannot open file $trans_file" });
	};
	die if $@;

	while( <$TRANS> ) {
		chomp;
		if( @hits_array == 0 ) { last; }

		my @trans_elem = split("\t", $_);

		### Check consistency
		eval {
			if( $hits_array[0][0] < $trans_elem[0]) {
print "$hits_array[0][0]\t$trans_elem[0]\n";
				die AmiRNA::Error->new( { errno => "translation error", msg => "TargetSearch error", 
					subject => "Translation file and hits out of sync. Trans file might not be sorted." });
			}
		};
		die if $@;

		### Check if next hit is in gene range
		while( ($hits_array[0][0] >= $trans_elem[0]) && ($hits_array[0][0] <= $trans_elem[1]) ) {

			### GenomeMapper 'Maplist' format: 
			### fasta_header, begin, alignment, query_id, orientation, mismatches, hits, 
			### target_length, offset, pe_flag, 3 x quality, target_gene_length, flanking_sequence
			my @elem = split(/\s+/, $hits_array[0][1]);
			my $fasta_header  = $trans_elem[2];
			my $begin         = $hits_array[0][0] - $trans_elem[0];
			my $alignment     = $elem[2];
			my $mismatches    = $elem[5];
			my $target_length = $elem[7];
			my $gene_length   = $trans_elem[1] - $trans_elem[0] + 1;
			my $end           = $begin + $target_length - 1;

			### Check if hit is fully contained in gene
			if( ($hits_array[0][0] + $target_length - 1) > $trans_elem[1]) {
				if( @hits_array >= 2 ) {
					shift(@hits_array);
					next;
				}
				else {
					shift(@hits_array);
					last;
				}
			}

			### Extract query and target from alignment
			my $query_aln = "";
			my $target_aln = "";
			for(my $i = 0; $i < length($alignment); $i++) {
				my $base = substr($alignment, $i, 1);
				if( $base ne "[" && $base ne "]" ) {
					$target_aln .= $base;
					$query_aln  .= $base;
				}
				elsif( $base eq "[" ) {
					$i++;
					$target_aln .= substr($alignment, $i, 1);
					$i++;
					$query_aln  .= substr($alignment, $i, 1);
					$i++;
				}
			}
			my $target_string = $target_aln;
			$target_string =~ s/-//g;
			my $query_string = $query_aln;
			$target_string =~ s/-//g;
		
			### Extract id from fasta header
			my $seq_id = $fasta_header;
			$seq_id =~ s/^>//g;
			$seq_id =~ s/^\s+//g;
			$seq_id =~ s/\s.*//g;


			### Fetch upstream and downstream flanking sequence of target if requested
			my $sequence = $target_string;
			if( $self->{user_param}{flanking} != 0 ) {
				$sequence = $elem[14];
			}


			### Remove isoform identifier (if requested)
			if( $self->{user_param}{remove_isoform} != 0 ) { $seq_id =~ s/\..*//g; }
	

			### Add begin of alignment to ID if multiple hits per target should be recorded
			my $unique_id = $seq_id;
			if($self->{user_param}{one_hit_per_gene} == 0) { $unique_id .= $begin; }


			### Insert result if target is not yet recorded
			if(! exists $id_exist_check{$unique_id} ) {
				$self->{searchResults}->set( $sequence, $query_string, $target_string, $query_aln, 
					$target_aln, $seq_id, $fasta_header, $begin, $end, $mismatches, $mismatches, $gene_length );
			}
			$id_exist_check{$unique_id} = 1;

			if( @hits_array >= 2 ) {
				shift(@hits_array);
			}
			else {
				shift(@hits_array); 
				last;
			}
		}
	}

	return 0;
}


### Get sequence of cDNA transcript
### Use only if complete gene sequence is needed. Flanking regions of a target 
### locus can be aquired using -flanking option of GenomeMapper.
sub get_sequence {
	my ($self, $id) = @_;

	my %results = ();
	eval {
		%results = $self->{seq_db}->get_cdna( $self->{user_param}{transcript_library}, $id);
	};
	die if $@;

	foreach (sort {$a cmp $b} keys %results) {
		return ($_, $results{$_});
	}

	eval {
		die AmiRNA::Error->new( { errno => "Database error", msg => "Gene not found in index: $id.", subject => "Gene:$id" } );
	};
	die if $@;
}

### Get length of cDNA transcript.
### DEPRECATED, GenomeMapper returns target gene length using parameter "-target_info 1" !!!
### Use only if gene length is needed independent of a target search, 
### because this function uses a slow database request!!!
sub get_gene_length {
	my ($self, $id) = @_;

	my %results = ();
	eval {
		%results = $self->{seq_db}->get_cdna_length( $self->{user_param}{transcript_library}, $id );
	};
	die if $@;

	foreach (sort {$a cmp $b} keys %results) {
		return ($_, $results{$_});
	}

	eval {
		die AmiRNA::Error->new( { errno => "Database error", msg => "Gene not found in index: $id.", subject => "Gene:$id" } );
	};
	die if $@;
}

### Check if user input is consistent
sub check_user_param {
        my ($self) = @_;

        eval {
                # Check integer values
                if(     (! is_int($self->{user_param}{mismatches}))   || (! is_int($self->{user_param}{showdesc})) ||
                        (! is_int($self->{user_param}{allow_gaps}))   || (! is_int($self->{user_param}{calculate_filter}))     ||
                        (! is_int($self->{user_param}{temperature}))  || (! is_int($self->{user_param}{cutoff}))     ||
                        (! is_int($self->{user_param}{relaxed}))      || (! is_int($self->{user_param}{flanking})) ||
			(! is_int($self->{user_param}{apply_filter}))
                ) {
                        die AmiRNA::Error->new( { errno => $!, msg => "Input variable is not Integer. Please check your parameters.", subject => "bad int" } );
                }

                # Check user input, string values
                if(     (! is_string($self->{user_param}{transcript_library})) || (! is_string($self->{user_param}{strand})) ||
                        (! is_string($self->{user_param}{calculate_folding}))  || (! is_string($self->{user_param}{folding_backend})) ||
                        (! is_string($self->{user_param}{sort_mode}))
                ) {
                        die AmiRNA::Error->new( { errno => $!, msg => "Input variable is not String. Please check your parameters.", subject => "bad string" } );
                }

                # Check if sort mode is valid
                if(! exists $self->{available_sort_modes}{$self->{user_param}{sort_mode}}) {
                        die AmiRNA::Error->new( { errno => $!, msg => "Sort mode is not supported." }, subject => "bad sort mode" );
                }

        };
        die if $@;

        return;
}
1;

__END__

=head1 NAME

 AmiRNA::TargetSearch::GenomeMapper

=head1 SYNOPSIS

 use AmiRNA::TargetSearch::GenomeMapper;

 my %user_param = (
        configfile         => "/etc/AmiRNA/AmiRNA.xml,
        transcript_library => "A_thaliana_TAIR8.fa",
        mismatches         => 5,
        allow_gaps         => 0,
        strand             => "rev",
        remove_isoform     => 1,
        one_hit_per_gene   => 1,
        calculate_filter   => 1,
        apply_filter       => 1,
        calculate_folding  => "passed",
        folding_backend    => "RNAup",
        temperature        => 23,
        cutoff             => 70,
        relaxed            => 1,
        flanking           => 6,
        showdesc           => 50,
        show_gene_length   => 1,
        sort_mode          => 'ea',
 );

 my $targetSearch = AmiRNA::TargetSearch::GenomeMapper->new(%user_param);

 my $query = 'ugacagaagagagugagcac';
 $targetSearch->search($query);

 foreach my $target ( @{ $$targetSearch{searchResults}{targets} } ) {
        print "seq_id:\t\t" . $target->{seq_id} . "\n";
        print "query_string:\t" . $target->{query_string} . "\n";
        print "target_string:\t" . $target->{target_string} . "\n";
        print "sequence:\t" . $target->{sequence} . "\n";
        print "query_aln:\t" . $target->{query_aln} . "\n";
        print "target_aln:\t" . $target->{target_aln} . "\n";
        print "gene_length:\t" . $target->{gene_length} . "\n";
        print "begin:\t\t" . $target->{begin} . "\n";
        print "end:\t\t" . $target->{end} . "\n";
        print "score:\t\t" . $target->{score} . "\n";
        print "dG:\t\t" . $target->{dG} . "\n";
        print "dG_ratio:\t" . $target->{dG_ratio} . "\n";
        print "filter_passed:\t" . $target->{filter_passed} . "\n";
        print "message:\t" . $target->{message} . "\n";
        print "---\n";
 }


=head1 DESCRIPTION

 Searches for a query in a hash index using GenomeMapper available
 from http://1001genomes.org/software


=head1 METHODS

=over

=item AmiRNA::TargetSearch::GenomeMapper-E<gt>new($searchOptions, %user_param)

 Constructs a GenomeMapper TargetSearch object.

 For $searchOptions see L<AmiRNA::TargetSearch::SearchOptions>

 The %user_param hash requires the following entries:

       transcript_library => transcript library name
       mismatches         => number of allowed mismatches 
       allow_gaps         => allow gaps in alignment, < 0 | 1 >
       strand             => < fwd | rev | both >
       remove_isoform     => count gene only once and neglect additional isoforms, < 0 | 1 >
       one_hit_per_gene   => report only first hit of a gene, < 0 | 1 >
       calculate_filter   => calculate results of MiRNA filter < 0 | 1 >
       apply_filter       => remove filtered hits from SearchResults
       calculate_folding  => use free hybridization energy for filtering < all | passed | non >
       folding_backend    => select RNA folding tool< RNAup | RNAcofold >
       temperature        => select folding temperature, 0 - 99 degree C
       cutoff             => cutoff for imperfect  to perfect energy, 0 to 99
       relaxed            => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
       flanking           => show x bases upstream and downstream of target region,
       showdesc           => show number of characters rfom fasta header, 0 - 200, 0 = show all
       show_gene_length   => store length of target gene for visualization. If not set gene length will be set to 0.
       sort_mode          => select sort modde, < ea | ed | ga | gd | la | ld >



=item AmiRNA::TargetSearch::GenomeMapper-E<gt>search($query)

 Search query string (nucleotides) in a GenomeMapper index according to the parameters specified in user_param.


=item AmiRNA::TargetSearch::GenomeMapper-E<gt>parse_results($query, $result_file)

 Parse GenomeMapper result_file and stores results in L<AmiRNA::TargetSearch::SearchOptions>


=item AmiRNA::TargetSearch::GenomeMapper-E<gt>get_sequence($transcript_id)

 Returns sequence of specified transcript ID $transcript_id.


=item AmiRNA::TargetSearch::Vmatch-E<gt>check_user_param()

 Test if user parameters are of the correct type.


=item AmiRNA::TargetSearch::Vmatch-E<gt>get_gene_length($transcript_id)

 Returns length specified transcript ID $transcript_id.
 DEPRECATED: GenomeMapper returns target gene length using parameter "-target_info 1"
 which is a lot faster than this method!

=back


=head1 DEPENDENCIES

=over

=item GenomeMapper from L<http://1001genomes.org>

=item L<Data::Types>

=item L<File::Temp>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::TargetSearch::SearchOptions>

=item L<AmiRNA::TargetSearch::SearchResult>

=item L<AmiRNA::TargetSearch::MIRNAFilter>

=item L<AmiRNA::Config::ConfigParser>

=item L<AmiRNA::DB::Base>

=item L<AmiRNA::DB::Sequences>

=item L<AmiRNA::Config::UserParams>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
