package AmiRNA::TargetSearch::MIRNAFilter;

use strict;
use warnings;
use Data::Types qw(:all);
use AmiRNA::Error;
use AmiRNA::TargetSearch::ViennaRNA;

our $VERSION = '3.1';

### Constructor
sub new {
	my ( $self, $configfile, $folding_backend, $temperature, $cutoff, $relaxed, $apply_filter) = @_;

	my $folding;
	eval {
		if( not -e $configfile ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "configfile '$configfile' does not exist" } );
		}
		if( ! is_string($folding_backend) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "folding_backend '$folding_backend' is invalid" } );
		}
		if( ! is_int($temperature ) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "temperature '$temperature' is invalid (must be integer)" } );
		}
		if( ! is_int($cutoff) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "cutoff '$cutoff' is invalid (must be integer)" } );
		}
		if( ! is_int($relaxed) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "relaxed '$relaxed' is invalid (must be integer)" } );
		}
		$folding = AmiRNA::TargetSearch::ViennaRNA->new($configfile, $temperature);
	};
	die if $@;

	$self = {
		folding_backend => $folding_backend,
		temperature     => $temperature,
		folding         => $folding,
		cutoff          => $cutoff,
		relaxed         => $relaxed,
		apply_filter    => $apply_filter,
	};
	bless $self;
	return $self;
}


### Tests if two sequences are a putative mirna/target pair
sub filter {
	my ( $self, $searchResults, $perform_dG ) = @_;
	eval {
		if( (! defined $searchResults) || (! defined $perform_dG) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "bad parameters" } );
		}
	};
	die if $@;

	### Stores RNA folding results to upspeed calculation for similar target sequences
	my %folding_hash = ();

	eval {
		### Calculate perfect match hybridization energy
		if( $perform_dG eq "all" || $perform_dG eq "passed") {
			$searchResults->{pdG} = $self->{folding}->fold( $self->{folding_backend}, $searchResults->{query}, $self->revcomp($searchResults->{query}) );
		}

		foreach my $searchResult ( @{$searchResults->{targets}} ) {

			### Get query / target sequences as reverse strings
			my $q    = reverse( lc( $searchResult->{query_aln} ) );
			my $t    = reverse( lc( $searchResult->{target_aln} ) );
    
			### Perform free hybridization energy calculation if selected
			if( $perform_dG eq "all") {
				if(  ($self->{folding_backend} eq "RNAup") && (exists $folding_hash{$searchResult->{sequence}}) ) {
					$searchResult->{dG} = $folding_hash{$searchResult->{sequence}};
				}
				elsif( exists $folding_hash{$searchResult->{target_string}} ) {
					$searchResult->{dG} = $folding_hash{$searchResult->{target_string}};
				}
				else {
					if($self->{folding_backend} eq "RNAup") {
						$searchResult->{dG} = $self->{folding}->fold( $self->{folding_backend}, $searchResults->{query}, $searchResult->{sequence});
						$folding_hash{$searchResult->{sequence}} = $searchResult->{dG};
					}
					else {
						$searchResult->{dG} = $self->{folding}->fold( $self->{folding_backend}, $searchResults->{query}, $searchResult->{target_string});
						$folding_hash{$searchResult->{target_string}} = $searchResult->{dG};
					}
				}
				$searchResult->{dG_ratio} = $searchResult->{dG} / $searchResults->{pdG} * 100;
			}
    
			### If $self->relaxed is defined, accept all alignments with up to 3 mismatches even if seed region has multiple mismatches
			my $MM3_candidate_flag = 0;
			if ( $self->{relaxed} != 0 && $self->get_mismatch( $q, $t ) <= 3 ) {
				$MM3_candidate_flag = 1;
			}

			### Test: mismatch at the cleavage site positions 10 or 11
			if ( ($self->is_mismatch( $q, $t, 9 )) || ($self->is_mismatch( $q, $t, 10 )) ) {
				$searchResult->{filter_passed} = 0;
				$searchResult->{message} = "Mismatch at position 10 and/or 11";
				if( $MM3_candidate_flag == 0 ) { next; }
			}

			### Test: number of mismatches at positions 2 to 12
			my $seed_mm = 0;
			for my $i ( 1 .. 11 ) { 
				$seed_mm++ if $self->is_mismatch( $q, $t, $i );
			}
			if ( $seed_mm > 1 ) {
				$searchResult->{filter_passed} = 0;
				if($searchResult->{message} eq "NA") {
					$searchResult->{message} = "More than one mismatch at positions 2-12";
				}
				if( $MM3_candidate_flag == 0) { next; }
			}
	
			### Test: Mismatch cluster > 2MM
			my $cluster = 0;
			my $last_char = 0;
			$last_char = -1 if $self->is_mismatch( $q, $t, 0 );
			my $cluster_flag = 1;

			for my $i ( 0 .. ( length($q) ) ) {
				# cluster at the beginning are ok
				next if ( $last_char == -1 && $self->is_mismatch( $q, $t, $i ) );
		
				if ( $self->is_mismatch( $q, $t, $i ) ) { 
					$cluster++; 
				}
				else {
					if ( $cluster > 2 ) {
						$searchResult->{filter_passed} = 0;
						if($searchResult->{message} eq "NA") {
							$searchResult->{message} =  "Mismatch cluster from " . ($i - $cluster) . " to " . ( $i - 1 );
						}
						$cluster_flag = 0;
					}
					$last_char = 1;
					$cluster = 0;
				}
			}
			if( ($cluster_flag == 0) & ($MM3_candidate_flag == 0) ) { next; }


			### Calculate free hybridization energy for passed results and possible candidates only
			if( $perform_dG eq "passed") {
				if(  ($self->{folding_backend} eq "RNAup") && (exists $folding_hash{$searchResult->{sequence}}) ) {
					$searchResult->{dG} = $folding_hash{$searchResult->{sequence}};
				}
				elsif( exists $folding_hash{$searchResult->{target_string}} ) {
					$searchResult->{dG} = $folding_hash{$searchResult->{target_string}};
				}
				else {
					if($self->{folding_backend} eq "RNAup") {
						$searchResult->{dG} = $self->{folding}->fold( $self->{folding_backend}, $searchResults->{query}, $searchResult->{sequence});
						$folding_hash{$searchResult->{sequence}} = $searchResult->{dG};
					}
					else {
						$searchResult->{dG} = $self->{folding}->fold( $self->{folding_backend}, $searchResults->{query}, $searchResult->{target_string});
						$folding_hash{$searchResult->{target_string}} = $searchResult->{dG};
					}
				}
				$searchResult->{dG_ratio} = $searchResult->{dG} / $searchResults->{pdG} * 100;
			}


			### Filter using free hybridization energy
			if( ($perform_dG ne "passed") && ($perform_dG ne "all") && ($searchResult->{message} eq "NA") ) {
				$searchResult->{filter_passed} = 1;
				$searchResult->{message} = "passed";
				$searchResults->{passed_targets}++;
			}
			elsif ( $self->{cutoff} > $searchResult->{dG_ratio} ) {
				$searchResult->{filter_passed} = 0;
				if($searchResult->{message} eq "NA") {
					$searchResult->{message} = "Low binding affinity: Only " . sprintf( "%.2f", $searchResult->{dG_ratio} ) . "% of optimal dG";
				}
			}
			elsif ( $searchResult->{dG} > -27 ) {
				$searchResult->{filter_passed} = 0;
				if($searchResult->{message} eq "NA") {
					$searchResult->{message} = 'Low binding affinity: dG = ' . sprintf( "%.2f", $searchResult->{dG} );
				}
			}
			elsif ($searchResult->{message} eq "NA") {
				$searchResult->{filter_passed} = 1;
				$searchResult->{message} = "passed";
				$searchResults->{passed_targets}++;
			}

			# Check if all filter message is correctly set (should not be "NA" anymore)
			if ($searchResult->{message} eq "NA") {
				$searchResult->{message} = "Unclassified";
			}

			if( ($searchResult->{filter_passed} == 0) && ($MM3_candidate_flag == 1) ) {
				$searchResult->{message} = "Possible candidate. " . $searchResult->{message};
			}
		}
	};
	die if $@;

	return 1;
}


### Returns number of mismatches between query and target
sub get_mismatch {
	my ( $self, $string1, $string2 ) = @_;

	eval {
		if( (! is_string($string1)) || (! is_string($string2)) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "bad parameters" } );
		}
	};
	die if $@;

	my $mismatch = 0;
	for ( my $i = 0; $i < length($string1); $i++ ) {
		my $char1 = substr( $string1, $i, 1 );
		my $char2 = substr( $string2, $i, 1 );
		if ( $char1 ne $char2 ) {
			$mismatch++;
		}
	}
	return $mismatch;
}


### Check if position is a mismatch
sub is_mismatch {
	my ( $self, $string1, $string2, $position ) = @_;

	eval {
		if( (! is_string($string1)) || (! is_string($string2)) || (! is_int($position)) ||
			(length($string1) < $position) || (length($string2) < $position) )
		{
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "bad parameters" } );
		}
	};
	die if $@;

	my $char1 = substr( $string1, $position, 1 );
	my $char2 = substr( $string2, $position, 1 );
	if ( lc($char1) ne lc($char2) ) {
		return 1;
	}
	else {
		return 0;
	}
}


### Reverse complement of sequence
sub revcomp {
	my ($self, $seq) = @_;

	eval {
		if( ! is_string($seq) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "TargetSearch error", subject => "bad parameters" } );
		}
	};
	die if $@;
			
	$seq =~ tr/uUacgt/TTACGT/;
	my $revcomp_seq = reverse($seq);
	$revcomp_seq =~ tr/ACGT/TGCA/;
	return $revcomp_seq;
}

1;
 
__END__

=head1 NAME

 AmiRNA::Filter::MIRNAFilter

=head1 SYNOPSIS

 # Create MIRNAFilter object
 my $filter = AmiRNA::TargetSearch::MIRNAFilter->new($configfile, $folding_backend, $temperature, $cutoff, $relaxed, $apply_filter);

 # Filter results from a target search
 $filter->filter($searchResults, $perform_dG);


=head1 DESCRIPTION

 Filters search results with empirical rules for plant MiRNA target detection 
 according to the following publications:

 Specific Effects of MicroRNAs on the Plant Transcriptome. 
 Schwab R, Palatnik JF, Riester M, Schommer C, Schmid M, Weigel D. 
 Dev Cell. 2005 Apr;8(4):517-527.)

 and

 Highly specific gene silencing by artificial microRNAs in Arabidopsis.
 Schwab R, Ossowski S, Riester M, Warthmann N, Weigel D.
 Plant Cell. 2006 May;18(5):1121-33. Epub 2006 Mar 10.


=head1 METHODS

=over

=item AmiRNA::TargetSearch::MIRNAFilter-E<gt>new($configfile, $folding_backend, $temperature, $cutoff, $relaxed, $apply_filter)

 Constructs a MIRNAFilter object.

 Parameters:
     configfile       => e.g. /etc/AmiRNA/AmiRNA.xml,
     folding_backend  => folding tool (RNAcofold or RNAup),
     temperature      => folding temperature,
     cutoff           => minimum ratio of observed to optimal folding energy dG,
     relaxed          => if relaxed = 1 all hits with up to 3 mismatches are returned regardless of targeting rules
     apply_filter     => Filtered result are not discarded (deprecated, not used anymore)


=item AmiRNA::TargetSearch::MIRNAFilter-E<gt>filter( $searchResults, $perform_dG )

 Filter target search results in $searchResults according to empirical rules.
 For $searchResults see L<AmiRNA::TargetSearch::SearchResult>.
 Folding energy filter can be turned of using $perform_dG = 0 (drastically increases speed).


=item AmiRNA::TargetSearch::MIRNAFilter-E<gt>get_mismatch($string1, $string2)

 Returns number of mismatches between $string1 and $string2


=item AmiRNA::TargetSearch::MIRNAFilter-E<gt>is_mismatch($string1, $string2, $pos)

 Returns 1 if the alignment between $string1 and $string2 has a mismatch in position $pos


=item AmiRNA::TargetSearch::MIRNAFilter-E<gt>revcomp($seq)

 Returns the reverse complement of $seq.

=back


=head1 DEPENDENCIES

=over

=item L<Data::Types>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::TargetSearch::Vmatch>

=item L<AmiRNA::TargetSearch::GenomeMapper>

=item L<AmiRNA::TargetSearch::ViennaRNA>

=item L<AmiRNA::TargetSearch::SearchResult>

=item L<use AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=item Alexis Maizel

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
