package AmiRNA::TargetSearch::SearchOptions;

use strict;
use warnings;
use Data::Types qw(:all);

use AmiRNA::Error;
use AmiRNA::Config::ConfigParser;

our $VERSION = '3.1';

### Constructor
sub new {
	my ($self, $configfile) = @_;
	my $configParser;
	eval { 
		$configParser =  AmiRNA::Config::ConfigParser->new($configfile); 
	};
	die if $@;

	my %general_param = (
		tmppath   => $configParser->get_tmp_path(),
		execpath  => $configParser->get_executable_path("mapping"),
		execname  => $configParser->get_executable_name("mapping"),
		indexpath => $configParser->get_index_path(),
	);

	$self = {
		configfile    => $configParser->{configfile},
		general_param => \%general_param,
	};

	bless($self);

	eval {
		$self->check_general_param();
	};
	die if $@;

	return $self;
}


### Check if configurations from conf.xml are consistent
sub check_general_param {
	my ( $self ) = @_;

	eval {
		unless ( stat $self->{general_param}{tmppath} ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Tmp path does not exist.", subject => $self->{general_param}{indexpath} } );
		}
		unless ( stat $self->{general_param}{execpath} ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Exec path does not exist.", subject => $self->{general_param}{indexpath} } );
		}
		unless ( stat $self->{general_param}{indexpath} ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Index path does not exist.", subject => $self->{general_param}{indexpath} } );
		}
		unless ( -e $self->{general_param}{execpath} . "/" . $self->{general_param}{execname} ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Mapping executable does not exist.", subject => $self->{general_param}{execname} } );
		}
	};
	die if $@;

	return;
}


### Return name of mapping tool
sub get_search_program {
	my ( $self ) = @_;
	return( $self->{general_param}{execname} );
}


### Get alias (description) of transcript libraries
sub get_transcript_libraries {
	my ( $self ) = @_;
	my %results = ();
	my $indexpath = $self->{general_param}{indexpath};

	### Get all .nfo files from index folder
	eval {
		opendir( DIR, $indexpath ) or 
			die AmiRNA::Error->new( { errno => $!, msg => "Configuration error", subject => $indexpath });
	};
	die if $@;
	
	my @files = grep( /\.nfo/, readdir(DIR) );
	closedir DIR;

	if(!@files) {
		die AmiRNA::Error->new( { errno => "No .nfo file found", msg => "No transcript library description found", subject => "$indexpath" });
	}

	### Read description from .nfo files
	foreach my $file (@files) {
		my $desc = '';
		my $prefix = '';
		($prefix = $file) =~ s/\.nfo//;
		
		eval {
			open NFO, $indexpath."/".$file or
				die AmiRNA::Error->new( { errno => $!, msg => "No transcript library found", subject => "$indexpath/$file" });
		};
		die if $@;
		
		while ( <NFO> ) {
			chomp;
			$desc .= $_;
		}
		close NFO;

		$results{$prefix} = $desc;		
	}
	

	### Return filenames and descriptions
	return %results;
}

1;

__END__

=head1 NAME

 AmiRNA::TargetSearch::SearchOptions 

=head1 SYNOPSIS

 # Create SearchOptions object
 my $searchOptions = AmiRNA::TargetSearch::SearchOptions->new($configfile);
	
 # Read some parameters from SearchOptions
 my %transcript_libraries = $searchOptions->get_transcript_libraries();


=head1 DESCRIPTION

 Container for TargetSearch options.


=head1 METHODS

=over 2

=item AmiRNA::TargetSearch::SearchOptions-E<gt>new($configfile)

 Constructs a SearchOptions object. Configfile name is optional, default /etc/AmiRNA/AmiRNA.xml.


=item AmiRNA::TargetSearch::SearchOptions-E<gt>check_general_param()

 Check if all parameters could be fetched from config file


=item AmiRNA::TargetSearch::SearchOptions-E<gt>get_search_program()

 Returns executable name of mapping tool.


=item AmiRNA::TargetSearch::SearchOptions-E<gt>get_transcript_libraries()

 Returns name and description (or alias) of all available transcript libraries.

=back


=head1 DEPENDENCIES

=over

=item L<Data::Types>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Config::ConfigParser>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
