package AmiRNA::TargetSearch::SearchResult;
use strict;
use warnings;
use Data::Types qw(:all);
use AmiRNA::Error;


our $VERSION = '3.1';

sub new {
	my ( $self, $query ) = @_;

	$self = {
		query          => $query,
		pdG            => 0,
		total_targets  => 0,
		passed_targets => 0,
		targets        => [],
	};
	bless $self;
	return $self;
}


#############################################################################################
# set usage:
#	sequence      => target sequence including upstream and downstream seq (if requested)
#	query_string  => query sequence without gaps ("-")
#	target_string => target sequence without gaps
#	query_aln     => query sequence as aligned by vmatch (including gaps)
#	target_aln    => target sequence as aligned by vmatch (including gaps)
#	seq_id        => ID of the gene or transcript targeted by an amiRNA
#	fasta_header  => complete fasta header of the target gene
#	gene_length   => length of the target gene
#	begin         => begin of the hit within the target gene
#	end           => end of the hit within the target gene
#	score         => evalue of the alignment as defined by vmatch
#       mismathces    => number of mismathces in the alignment
#
# Variables which can only be set directly:
#	dG            => free hybridization energy between query and target calculated with ViennaRNA
#	dG_ratio      => ratio of perfect hybridization (no mismatches) to observed hybridization with mismatches
#	filter_passed => sequence and hybridization based filters flag
#	message       => filter result message ("passed" if all criteria are met).

sub set {
	my ($self, $sequence, $query_string, $target_string, $query_aln, $target_aln, $seq_id, $fasta_header, $begin, $end, $score, $mismatches, $gene_length) = @_;
	
	### Check parameter consistency
	eval {
		if(	(! is_string($sequence)) || (! is_string($query_string)) || (! is_string($target_string)) ||
			(! is_string($query_aln)) || (! is_string($target_aln))
		) {
			die AmiRNA::Error->new( { errno => $!, msg => "Variable has to be of type string.",
				subject => "Variables:$sequence,$query_string,$target_string,$query_aln,$target_aln" } );
		}
		if( (! is_int($begin)) || (! is_int($end)) || (! is_int($mismatches)) || (! is_int($gene_length)) ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Variable has to be of type int.",
				subject => "Variables:$begin,$end,$mismatches,$gene_length" } );
		}
		if( (! defined $seq_id) || (! defined $fasta_header) || (! defined $score) ) {
			die AmiRNA::Error->new( { errno => $!, msg => "Variable not defined.",
				subject => "Variables:$seq_id,$fasta_header,$score" });
		}
	};
	die if $@;

	my %target = (
		sequence      => $sequence,
		query_string  => $query_string,
		target_string => $target_string,
		query_aln     => $query_aln,
		target_aln    => $target_aln,
		seq_id        => $seq_id,
		fasta_header  => $fasta_header,
		gene_length   => $gene_length,
		begin         => $begin,
		end           => $end,
		score         => $score,
		mismatches    => $mismatches,
		dG            => 0,
		dG_ratio      => 0,
		filter_passed => 0,
		message       => "NA"
	);

	push( @{$self->{targets}}, \%target );
	$self->{total_targets}++;
	return;
}


### Sort targets by free hybridization energy dG
sub sort_by_dg {
	my ( $self ) = @_;

	### Check if dG was calculated for all results
	my $dG_flag = 1;
	foreach my $target ( @{$self->{targets}} ) {
		if(! defined $target->{dG}) { $dG_flag = 0; }
	}

	### Sort
	if($dG_flag == 1) {
		my @tmp = @{ $self->{targets} };
		$self->{targets} = ();
		foreach my $target ( sort { $a->{dG} <=> $b->{dG} } @tmp ) {
			push( @{$self->{targets}}, $target );
		}
	}
}

sub sort_by_score {
	my ( $self ) = @_;

	my @tmp = @{ $self->{targets} };
	$self->{targets} = ();
	foreach my $target ( sort { $a->{score} <=> $b->{score} } @tmp ) {
		push( @{$self->{targets}}, $target );
	}
}

1;

__END__

=head1 NAME

 AmiRNA::TargetSearch::SearchResult

=head1 SYNOPSIS
  
 # Create new SearchResult object
 my $searchResults = AmiRNA::TargetSearch::SearchResult->new($query);
  
 # Set new target
 $searchResults->set($sequence, $query_string, $target_string, $query_aln, $target_aln, $seq_id,
                      $fasta_header, $begin, $end, $score, $mismatches, $gene_length);

 # Print all targets
 foreach my $target ( keys %{ $searchResults{targets} } ) {
      print "seq_id:\t\t" . uc( $target->{seq_id} ) . "\n";
      print "query_string:\t" . $target->{query_string} . "\n";
      print "target_string:\t" . $target->{target_string} . "\n";
      print "sequence:\t" . $target->{sequence} . "\n";
      print "query_aln:\t" . $target->{query_aln} . "\n";
      print "target_aln:\t" . $target->{target_aln} . "\n";
      print "gene_length:\t" . $target->{gene_length} . "\n";
      print "begin:\t\t" . $target->{begin} . "\n";
      print "end:\t\t" . $target->{end} . "\n";
      print "score:\t\t" . $target->{score} . "\n";
      print "dG:\t\t" . $target->{dG} . "\n";
      print "dG_ratio:\t" . $target->{dG_ratio} . "\n";
      print "filter_passed:\t" . $target->{filter_passed} . "\n";
      print "message:\t" . $target->{message} . "\n";
 }

  
=head1 DESCRIPTION

 Datastructure recording all hits of a query sequence in an indexed genome or transcriptome.


=head1 METHODS

=over 

=item AmiRNA::TargetSearch::SearchResult-E<gt>new($query)

 Constructs a SearchResult object. Takes as parameter the search 
 string (nucleotides) as an identifier for the result.


=item AmiRNA::TargetSearch::SearchResult-E<gt><set($sequence, $query_string, $target_string, $query_aln, $target_aln, $seq_id, $fasta_header, $begin, $end, $score, $gene_length)>

 Add new target of the query sequence.

 Parameters:

       sequence      => target sequence including upstream and downstream seq (if requested)
       query_string  => query sequence without gaps ("-")
       target_string => target sequence without gaps
       query_aln     => query sequence as aligned by vmatch (including gaps)
       target_aln    => target sequence as aligned by vmatch (including gaps)
       seq_id        => ID of the gene or transcript targeted by an amiRNA
       fasta_header  => complete fasta header of the target gene
       begin         => begin of the hit within the target gene
       end           => end of the hit within the target gene
       score         => evalue of the alignment as defined by vmatch
       gene_length   => length of the target gene


=item AmiRNA::TargetSearch::SearchResult-E<gt>sort_by_dg()

 Sort targets by free hybridization energy dG.


=item AmiRNA::TargetSearch::SearchResult-E<gt>sort_by_score()

 Sort targets by alignment score.

=back


=head1 DEPENDENCIES

=over

=item L<Data::Types>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
