package AmiRNA::Designer::shRNADesigner;

use strict;
use warnings;
use Data::Types qw(:all);
use AmiRNA::Error;
use AmiRNA::TargetSearch::Vmatch;
use AmiRNA::TargetSearch::GenomeMapper;
use AmiRNA::Config::UserParams;

our $VERSION = '3.1';

####################################################################################################
### Constructor
# user_param:
#      transcript_library => transcript library name
#      strand             => < fwd | rev | both >
#      remove_isoform     => count gene only once and neglect additional isoforms, < 0 | 1 >
#      one_hit_per_gene   => report only first hit of a gene, < 0 | 1 >
sub new {
	my ($self, $searchOptions, %user_param) = @_;

        ### Check user parameters
        my @allowed_params  = qw/configfile transcript_library target_fasta target_genes musthave annotated offtargets temperature relaxed_rules accelerate_design flanking/;
        my @required_params = qw/transcript_library musthave annotated temperature relaxed_rules accelerate_design flanking/;

        eval {
                my $param_ckeck = new AmiRNA::Config::UserParams();
                $param_ckeck->check(\@allowed_params, \@required_params, \%user_param);
        };
        die if $@;

	my %seed_search_params = (
		transcript_library => $user_param{transcript_library} . ".utr3",
		mismatches         => 0,
		allow_gaps         => 0,
		strand             => 'rev',
		remove_isoform     => 0,
		one_hit_per_gene   => 0,
		calculate_filter   => 0,
		apply_filter       => 0,
		calculate_folding  => 'non',
		folding_backend    => 'RNAup',
		temperature        => 0,
		cutoff             => 0,
		relaxed            => 0,
		flanking           => 0,
		showdesc           => 200,
		show_gene_length   => 0,
		sort_mode          => 'ea',
	);

	my $seedSearch;

	eval {
		if( $searchOptions->get_search_program() eq "vmatch" ) {
			$seedSearch = AmiRNA::TargetSearch::Vmatch->new($searchOptions, %seed_search_params);
		}
		elsif( $searchOptions->get_search_program() eq "genomemapper" ) {
			$seedSearch = AmiRNA::TargetSearch::GenomeMapper->new($searchOptions, %seed_search_params);
		}
	};
	die if $@;
		

	$self = {
		seedSearch    => $seedSearch,
	        utr3_hits7    => {},
		utr3_hits8    => {},
		counter       => (71 => 0, 72 => 0, 73 => 0, 81 => 0, 82 => 0, 83 => 0),
	};
	bless $self;
	return $self;
}

####################################################################################################
### Design miRNAs for single targets
sub search {
	my ( $self, $pattern ) = @_;

	### Extract seed sequence
	my $seed7_pattern = substr($pattern, 1, 7);
	my $seed8_pattern = substr($pattern, 1, 8);

	### Search for 3'UTR hits of length 7
	eval {
		$self->{seedSearch}->search($seed7_pattern);
	};
	die if $@;

	### Check each 7mer offtarget
	foreach my $target ( @{ $self->{seedSearch}{searchResults}{targets} } ) {
		my $fasta_header = $target->{seq_id};
		my ($target_id, $begin) = split("-", $fasta_header);
		if( ! exists $self->{genes}{$target_id} ) {
			$self->{utr3_hits7}{$target_id}++;
		}
	}

	### Search for 3'UTR hits of length 8
	eval {
		$self->{seedSearch}->search($seed8_pattern);
	};
	die if $@;

	### Check each 8mer offtarget
	foreach my $target ( @{ $self->{seedSearch}{searchResults}{targets} } ) {
		my $fasta_header = $target->{seq_id};
		my ($target_id, $begin) = split("-", $fasta_header);
		if( ! exists $self->{genes}{$target_id} ) {
			$self->{utr3_hits8}{$target_id}++;
		}
	}

	# Count 7mer hits
	foreach my $gene ( keys %{ $self->{utr3_hits7} } ) {
		if($self->{utr3_hits7}{$gene} == 1) { $self->{counter}{71}++; }
		if($self->{utr3_hits7}{$gene} == 2) { $self->{counter}{72}++; }
		if($self->{utr3_hits7}{$gene} >= 3) { $self->{counter}{73}++; }
	}

	# Count 8mer hits
	foreach my $gene ( keys %{ $self->{utr3_hits7} } ) {
		if($self->{utr3_hits8}{$gene} == 1) { $self->{counter}{81}++; }
		if($self->{utr3_hits8}{$gene} == 2) { $self->{counter}{82}++; }
		if($self->{utr3_hits8}{$gene} >= 3) { $self->{counter}{83}++; }
	}

}
1;

__END__

=head1 NAME

AmiRNA::TargetSearch::SeedSearch

=head1 DESCRIPTION

Serch seed matches (7 to 8 bp) in 3'UTRs

=head1 SYNOPSIS

use AmiRNA::TargetSearch::Vmatch;
use AmiRNA::TargetSearch::SeedSearch;

=head1 METHODS

=over 2

=item new()

B<Description:> This function constructs a single_miRNA_designer object

B<Parameters:> none

B<Returns:> $single_miRNA_designer

The following attributes will be populated by calling design_miRNAs:

=back

=head1 AUTHOR

Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>
Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>

=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005 by Max Planck Institute for Developmental Biology,
Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut
# vim: ft=perl sw=4 ts=4 expandtab
