package AmiRNA::TargetSearch::ViennaRNA;

use strict;
use warnings;

use Data::Types qw(:all);
use File::Temp;
use Cwd;

use AmiRNA::Error;
use AmiRNA::Config::ConfigParser;

our $VERSION = '3.1';

###############################################################################
# Usage      : AmiRNA::Folding::RNAcofold->new($temperature);
# Purpose    : constructor
# Returns    : RNAcofold object
sub new {
	my ($self, $configfile, $temp) = @_;
	my $configParser;
	eval {
		$configParser = AmiRNA::Config::ConfigParser->new($configfile);
	};
	die if $@;

	$self = {
		temp => $temp,
		tmppath => $configParser->get_tmp_path(),
		execpath => $configParser->get_executable_path("folding"),
		DataRNAcofold => {},
		DataRNAup => {},
	};

	bless $self;
	return $self;
}

### General folding request
sub fold {
	my ( $self, $backend, $seq_a, $seq_b ) = @_;

	eval {
		if( (! is_string($backend)) || (! is_string($seq_a)) || (! is_string($seq_b)) ) {
			die AmiRNA::Error->new( { errno => "variable type error", msg => "Variable is not of type string",
				subject => "Variables:$backend,$seq_a,$seq_b" });
		}
		if( ($backend ne "RNAcofold") && ($backend ne "RNAup") ) {
			die AmiRNA::Error->new( { errno => "variable type error", msg => "Unknown folding backend", subject => "Backend: $backend" });
		}
	};
	die if $@;

	my $dg;
	if($backend eq "RNAcofold") {
		eval {
			$dg = $self->RNAcofold($seq_a, $seq_b);
		} or die $@;
	}
	elsif($backend eq "RNAup") {
		eval {
			$dg = $self->RNAup($seq_a, $seq_b);
		}
	}

	return $dg
}

###############################################################################
# Usage      : $folding->RNAcofold()
# Purpose    : calculate dG of seq_a/seq_b duplex
# Returns    : free energy dG
sub RNAcofold {
	my ( $self, $seq_a, $seq_b ) = @_;

	# Create input 'linked' sequence    
	my $linked_seq = $seq_a . "&" . $seq_b;
	$linked_seq =~ tr/tT/uU/;
	my $dg;

	eval {
		# Write tmp file with linked sequence
		my $path = cwd;
		chdir( $self->{tmppath} );

		my $tmp = new File::Temp(DIR => $self->{tmppath}) or
			die AmiRNA::Error->new( { errno => "File error", msg => "Cannot create tmp file", subject => $self->{tmppath} });
		my @folder_structure = split("/", $tmp);
		my $filename = pop(@folder_structure);
		print $tmp ">$filename\n$linked_seq" or
			die AmiRNA::Error->new( { errno => "File error", msg => "Cannot write to tmp file", subject => $tmp });

		# Hybridize
		my $command = $self->{execpath} . "/RNAcofold -noCloseGU -noLP -T " . $self->{temp} . " < " . $tmp;
		my $res = `$command` or die AmiRNA::Error->new( { errno => "ViennaRNA error", msg => "RNAcofold failed", subject => "" });

		( $dg ) = $res =~ /\(\s*([0-9\-\.]*?)\)$/;

		my $ps_file = $tmp . "_ss.ps";
		unlink $ps_file if -e $ps_file;

		chdir($path);
	};
	die if $@;

	# Store and return folding results
	if( ! is_float($dg) ) { $dg = 0; }
	$self->{DataRNAcofold}{$linked_seq} = $dg;
	return $dg;
}

###############################################################################
# Usage      : $folding->RNAup()
# Purpose    : calculate dG of seq_a/seq_b duplex
# Returns    : free energy dG
sub RNAup {
	my ( $self, $seq_a, $seq_b ) = @_;

	# Create input sequence    
	my $linked_seq = $seq_a . "\n" . $seq_b;
	$linked_seq =~ tr/tT/uU/;
	my ($dg, $sum);

	eval {
		# Write tmp file with two sequences
		my $path = cwd;
		chdir( $self->{tmppath} );

		my $tmp = new File::Temp(DIR => $self->{tmppath}) or
			die AmiRNA::Error->new( { errno => "File error", msg => "Cannot create tmp file", subject => $self->{tmppath} });
		my @folder_structure = split("/", $tmp);
		my $filename = pop(@folder_structure);
		print $tmp ">$filename\n$linked_seq" or 
			die AmiRNA::Error->new( { errno => "File error", msg => "Cannot write to tmp file", subject => $tmp });

		# Hybridize (add -b to use folding energy of miRNA in addition to folding energy of mRNA 
		my $command = $self->{execpath} . "/RNAup -Xp -noCloseGU -noLP -T " . $self->{temp} . " < " . $tmp;
		my $res = `$command` or die AmiRNA::Error->new( { errno => "ViennaRNA error", msg => "RNAcofold failed", subject => "" });

		# Parse dG information
		my @result_lines = split("\n", $res);
		my ($energy_formula) = $result_lines[1] =~ /\((\-.*)\)/;
		($dg, $sum) = split(/=/, $energy_formula);
		my ($dg_int, $dg_mRNA, $dg_sRNA) = split(/\+/, $sum);
		$dg =~ s/ //g; $dg_int =~ s/ //g; $dg_mRNA =~ s/ //g;

		# delete temporary files
		my $out_file = $tmp . "_w25_u4_up.out";
		unlink $out_file if -e $out_file;
		chdir($path);
	};
	die if $@;

	# Store folding results
	$self->{DataRNAup}{$linked_seq} = $dg;
	return $dg;
}


###############################################################################
sub cite_string {
    return <<EOT
I.L. Hofacker, W. Fontana, P.F. Stadler, S. Bonhoeffer, M. Tacker, P.
Schuster (1994) Fast Folding and Comparison  of  RNA  Secondary
Structures.  Monatshefte f. Chemie 125: 167-188
EOT
;
}

1;

__END__

=head1 NAME

 AmiRNA::TargetSearch::ViennaRNA

=head1 SYNOPSIS
  
 use AmiRNA::TargetSearch::ViennaRNA;

 my $seq_a = "ugacagaagagagugagcac";
 my $seq_b = "gugcucacucucuucuguca";

 ### Create new amiRNA object
 my $folding = AmiRNA::TargetSearch::ViennaRNA->new($temperature);

 ### Use RNAcofold
 my $dG = $folding->RNAcofold($seq_a, $seq_b);
 print "\n\nRNAcofold: $dG\n\n";

 ### Use RNAup
 $dG = $folding->RNAup($seq_a, $seq_b);
 print "\n\nRNAup: $dG\n\n";

=head1 DESCRIPTION

 ViennaRNA Backend. Calculates folding energy of two RNAs 
 using either RNAcofold or RNAup.


=head1 METHODS

=over 

=item C<AmiRNA::TargetSearch::ViennaRNA-E<gt>new($configfile, $temperature)>

 Constructs a ViennaRNA folding object. Parameter: configfile name, folding temperature.


=item C<AmiRNA::TargetSearch::ViennaRNA-E<gt>fold($backend, $seq_a, $seq_b)>

 Hybridize two RNAs ($seq_a, $seq_b) using $backend.


=item C<AmiRNA::TargetSearch::ViennaRNA-E<gt>RNAcofold($seq_a, $seq_b)>

 Hybridize two RNAs ($seq_a, $seq_b) using RNAcofold.


=item C<AmiRNA::TargetSearch::ViennaRNA-E<gt>RNAup($seq_a, $seq_b)>

 Hybridize two RNAs ($seq_a, $seq_b) using RNAup.


=item C<AmiRNA::TargetSearch::cite-E<gt>RNAup()>

 Returns citation for the ViennaRNA package.

=back


=head1 FILES

=over

=item ViennaRNA package

=item L<Data::Types>

=item L<File::Temp>

=item L<Cwd>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::TargetSearch::MIRNAFilter>

=item L<AmiRNA::Config::ConfigParser>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over
=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
