package AmiRNA::TargetSearch::Vmatch;

use strict;
use warnings;
use Data::Types qw(:all);
use File::Temp;
use AmiRNA::Error;
use AmiRNA::TargetSearch::SearchOptions;
use AmiRNA::TargetSearch::SearchResult;
use AmiRNA::TargetSearch::MIRNAFilter;
use AmiRNA::Config::ConfigParser;

our $VERSION = '3.1';

### Constructor
# user_param:
#      transcript_library => transcript library name
#      mismatches         => number of allowed mismatches 
#      allow_gaps         => allow gaps in alignment, < 0 | 1 >
#      strand             => < fwd | rev | both >
#      remove_isoform     => count gene only once and neglect additional isoforms, < 0 | 1 >
#      one_hit_per_gene   => report only first hit of a gene, < 0 | 1 >
#      calculate_filter   => calculate results of MiRNA filter < 0 | 1 >
#      apply_filter       => remove filtered hits from SearchResults
#      calculate_folding  => use free hybridization energy for filtering < all | passed | non >
#      folding_backend    => select RNA folding tool< RNAup | RNAcofold >
#      temperature        => select folding temperature, 0 - 99 degree C
#      cutoff             => cutoff for imperfect  to perfect energy, 0 to 99
#      relaxed            => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
#      flanking           => show x bases upstream and downstream of target region,
#      showdesc           => show number of characters rfom fasta header, 0 - 200, 0 = show all
#      show_gene_length   => store length of target gene for visualization. If not set gene length will be set to 0.
#      sort_mode          => select sort modde, < ea | ed | ga | gd | la | ld >

sub new {
	my ($self, $searchOptions, %user_param) = @_;
	my $miRNAfilter;


	### Check user parameters
	my @allowed_params  = qw/allow_gaps calculate_folding cutoff calculate_filter apply_filter folding_backend mismatches one_hit_per_gene relaxed remove_isoform show_gene_length showdesc sort_mode strand temperature transcript_library flanking/;
	my @required_params  = qw/allow_gaps calculate_folding cutoff calculate_filter apply_filter folding_backend mismatches one_hit_per_gene relaxed remove_isoform show_gene_length showdesc sort_mode strand temperature transcript_library flanking/;

	eval {
		my $param_ckeck = new AmiRNA::Config::UserParams();
		$param_ckeck->check(\@allowed_params, \@required_params, \%user_param);
	};
	die if $@;


	### Set up MiRNA filter	
	eval {
		if( ! defined $searchOptions) {
			die AmiRNA::Error->new( { errno => "Vmatch error", msg => "Wrong parameters for TargetSearch.", subject => "" } );
		}

		$miRNAfilter  = AmiRNA::TargetSearch::MIRNAFilter->new( $searchOptions->{configfile}, $user_param{folding_backend},
			$user_param{temperature}, $user_param{cutoff}, $user_param{relaxed}, $user_param{apply_filter} );
	};
	die if $@;

	my %available_sort_modes = (
		ga  => 'ascending order of dG',
		gd  => 'descending order of dG',
		ea  => 'ascending order of Evalue',
		ed  => 'descending order of Evalue',
	);

	$self = {
		searchOptions        => $searchOptions,
		user_param           => \%user_param,
		available_sort_modes => \%available_sort_modes,
		miRNAfilter          => $miRNAfilter,
		searchResults        => ''
	};

	bless($self);
	return $self;
}


sub search {
	my ( $self, $query ) = @_;

	### Check input
	eval { 
		$self->check_user_param(); 
	};
	die if $@;

	### Prepare query sequence
	$query =~ tr/uU/tT/;
	$query = uc($query);

	### Generate command string
	# Edit distance and min seed length
	my $edit_distance = '';
	my $seedlength = '';
	if( $self->{user_param}{mismatches} > 0 ) {
		$seedlength = '-seedlength 5';
		if( $self->{user_param}{allow_gaps} == 0 ) {
			$edit_distance = ' -h ' . $self->{user_param}{mismatches};
		}
		else {
			$edit_distance = ' -e ' . $self->{user_param}{mismatches};
		}
	}

	# Minimum length of alignment
	my $length = ' -l ' . length($query);

	# Align fwd, rev or both
	my $fwdrev = " -d -p ";
	if( $self->{user_param}{strand} eq "fwd" ) { $fwdrev = " -d "; }
	elsif( $self->{user_param}{strand} eq "rev" ) { $fwdrev = " -p "; }

	# Sort results
	my $sort   = '';
	if ( substr( $self->{user_param}{sort_mode}, 0, 1 ) ne 'g' ) {
		$sort = ' -sort ' . $self->{user_param}{sort_mode} . ' ';
	}

	# Set max hits to speed up alignment
	my $maxhits = ' -best 75000';

	# Show description
	my $showdesc = '';
	if( $self->{user_param}{showdesc} > 0 ) { $showdesc = ' -showdesc ' . $self->{user_param}{showdesc}; }

	eval {
		### Print temporary fasta file used as vmatch input
		my $tmp_query = new File::Temp(DIR => $self->{searchOptions}{general_param}{tmppath});
		my $tmp_out = "$tmp_query.vm";
		print $tmp_query ">1\n$query";

		# Concatenate command
		my $cmd = $self->{searchOptions}{general_param}{execpath} . "/vmatch -q $tmp_query $fwdrev "
			. "$edit_distance $length $maxhits $sort $showdesc -s $seedlength -noscore -noidentity "
			. $self->{searchOptions}{general_param}{indexpath} . "/" .  $self->{user_param}{transcript_library}
			. " | grep '.'  > $tmp_out";

		### Search
		system($cmd); # or die AmiRNA::Error->new( { errno => "Vmatch failed", msg => "TargetSearch error.", subject => "Command:$cmd" } );

		### Parse and store results 
		$self->parse_results($query, $tmp_out);

		### Run filter if selected
		if( $self->{user_param}{calculate_filter} == 1 ) {
			$self->{miRNAfilter}->filter($self->{searchResults}, $self->{user_param}{calculate_folding});
		}

		### Sort bt dG if selected
		if( substr( $self->{user_param}{sort_mode}, 0, 1 ) eq 'g' ) {
			$self->{searchResults}->sort_by_dg();
		}

		### Delete temporary output file
		unlink $tmp_out if -e $tmp_out;
	};
	die if $@;

	# Finish
	return 1;
}


### Parse and store results
sub parse_results {
	my ( $self, $query, $tmp_out ) = @_;
	my %id_exist_check = ();

	eval {
		$self->{searchResults} = AmiRNA::TargetSearch::SearchResult->new($query);

		open VMOUT, $tmp_out or die AmiRNA::Error->new( { errno => "Cannot open file", msg => "TargetSearch error", subject => $tmp_out } );
		my $junk = <VMOUT>;

		while (my $line = <VMOUT>) {
			chomp($line);
			$line =~ s/^\s+//;

			my @elem = split(/\s+/, $line);
			my $target_length = $elem[0];
			my $fasta_header  = $elem[1];
			my $begin         = $elem[2] + 1;
			my $orientation   = $elem[3];
			my $query_length  = $elem[4];
			my $query_id      = $elem[5];
			my $offset        = $elem[6];
			my $mismatches    = $elem[7];
			my $evalue        = $elem[8];

			$mismatches    = abs($mismatches);
			my $end = $begin + $target_length - 1;
			$fasta_header =~ s/_/ /g;

			### Read Alignment
			my $alignment = '';

			### Read next 2 or 3 lines
			my $target_line = <VMOUT>;
			my $query_line = <VMOUT>;
			if( $query_line !~ /Query/ ) { $query_line = <VMOUT>; }

			### Get query and target alignment
			my ($junk1, $target_aln, $junk2) = split(/\s/, $target_line);
			my ($junk3, $query_aln, $junk4) = split(/\s/, $query_line);
			my $target_string = $target_aln;
			$target_string =~ s/-//g;
			my $query_string = $query_aln;
			$target_string =~ s/-//g;

			### Extract id from fasta header
			my $seq_id = $fasta_header;
			$seq_id =~ s/^>//g;
			$seq_id =~ s/^\s+//g;
			$seq_id =~ s/\s.*//g;
			#$seq_id = uc($seq_id);

			### Fetch upstream and downstream flanking sequence of target if requested
			my $sequence = $target_string;
			if( $self->{user_param}{flanking} != 0 ) {
				my $upstream_start = $begin - $self->{user_param}{flanking} - 1;
				my $downstream_end = $end + $self->{user_param}{flanking} - 1;
				$sequence = $self->get_sequence_range($seq_id, $upstream_start, $downstream_end);
			}

			### Get gene length if requested
			my $gene_length = 0;
			if( $self->{user_param}{show_gene_length} == 1) {
				$gene_length = $self->get_sequence_range($seq_id, 0, 0);
			}

			### Remove isoform identifier (if requested)
			if( $self->{user_param}{remove_isoform} != 0 ) { $seq_id =~ s/\..*//g; }
	
			### Add begin of alignment to ID if multiple hits per target should be recorded
			my $unique_id = $seq_id;
			if($self->{user_param}{one_hit_per_gene} == 0) { $unique_id .= "-$begin"; }

			### Insert result if target is not yet recorded
			if(! exists $id_exist_check{$unique_id} ) {
				$self->{searchResults}->set( $sequence, $query_string, $target_string, $query_aln, $target_aln,
					$seq_id, $fasta_header, $begin, $end, $evalue, $mismatches, $gene_length );
			}
			$id_exist_check{$unique_id} = 1;
		}

		close(VMOUT);
	};
	die if $@;

	return 0;
}


### Check if user input is consistent
sub check_user_param {
        my ($self) = @_;

        eval {
                # Check integer values
                if(     (! is_int($self->{user_param}{allow_gaps}))   || (! is_int($self->{user_param}{calculate_filter}))     ||
                        (! is_int($self->{user_param}{temperature}))  || (! is_int($self->{user_param}{cutoff}))     ||
                        (! is_int($self->{user_param}{relaxed}))      || (! is_int($self->{user_param}{flanking}))   ||
                        (! is_int($self->{user_param}{showdesc}))     || (! is_int($self->{user_param}{mismatches})) ||
			(! is_int($self->{user_param}{apply_filter}))
                ) {
                        die AmiRNA::Error->new( { errno => "wrong parameters", msg => "TargetSearch error", subject => "bad integer" } );
                }

                # Check user input, string values
                if(     (! is_string($self->{user_param}{transcript_library})) || (! is_string($self->{user_param}{strand})) ||
                        (! is_string($self->{user_param}{calculate_folding}))  || (! is_string($self->{user_param}{folding_backend})) ||
                        (! is_string($self->{user_param}{sort_mode}))
                ) {
                        die AmiRNA::Error->new( { errno => "wrong parameters", msg => "TargetSearch error", subject => "bad string" } );
                }

                # Check if sort mode is valid
                if(! exists $self->{available_sort_modes}{$self->{user_param}{sort_mode}}) {
                        die AmiRNA::Error->new( { errno => "unsupported sort mode", msg => "Sort mode is not supported." }, subject => "unsupported sort mode" );
                }

        };
        die if $@;

        return;

}


### Fetch fasta entry from index file
sub get_sequence {
	my ( $self, $seqid )  = @_;
	my %seqs = ();

	### Check parameter consistency
	eval {
		if( ! is_string($seqid) ) {
			die AmiRNA::Error->new( { errno => "wrong parameters", msg => "TargetSearch error", subject => "bad string" } );
		}
	};
	die if $@;

	my $cmd = $self->{searchOptions}{general_param}{execpath} . "/vseqselect -matchdesc '$seqid' "
		. $self->{searchOptions}{general_param}{indexpath} . "/" .  $self->{user_param}{transcript_library}
		. " 2> /dev/null";

	my $fasta = `$cmd` or die AmiRNA::Error->new( { errno => "vseqselect error", msg => "TargetSearch error", subject => "Command:$cmd" } );

	### No results
	eval {
		if($fasta !~ />/) {
			die AmiRNA::Error->new( { errno => "vseqselect error", msg => "Gene no found in index: $seqid.", subject => "Gene:$seqid" } );
		}
	};
	die if $@;

	### Results, parse fasta
	my $current_gene = '';
	my @fasta_lines = split(/\n/, $fasta);
	for( my $i = 0; $i < @fasta_lines; $i++) {
		chomp($fasta_lines[$i]);
		if($fasta_lines[$i] =~ />/) {
			$current_gene = $fasta_lines[$i];
			$current_gene =~ s/^>//g;
			$current_gene =~ s/^\s+//g;
			$current_gene =~ s/\s.*//g;
			#$current_gene = uc($current_gene);
		}
		else {
			$seqs{$current_gene} .= $fasta_lines[$i];
		}
	}

	### Return gene with lowest isoform number
	foreach ( sort {$a cmp $b} keys %seqs ) {
		#return( ">$first_id\n" . $seqs{$first_id});
		return( $_, $seqs{$_});
	}

	eval {
		die AmiRNA::Error->new( { errno => "vseqselect error", msg => "Gene not found in index: $seqid.", subject => "Gene:$seqid" } );
	};
	die if $@;
}

### Fetch fasta sequence from index file and return subsequence between start and end
sub get_sequence_range {
	my ( $self, $seqid, $start, $end)  = @_;
	my %seqs = ();

	### Check parameter consistency
	eval {
		if( (! is_string($seqid)) || (! is_int($start)) || (! is_int($end)) ) {
			die AmiRNA::Error->new( { errno => "wrong parameters", msg => "TargetSearch error", subject => "wrong parameter" } );
		}
	};
	die if $@;

	my $cmd = $self->{searchOptions}{general_param}{execpath} . "/vseqselect -matchdesc '$seqid' "
		. $self->{searchOptions}{general_param}{indexpath} . "/" .  $self->{user_param}{transcript_library}
		. " 2> /dev/null";
	my $fasta = `$cmd` or die AmiRNA::Error->new( { errno => "vseqselect error", msg => "TargetSearch error", subject => "Command:$cmd" } );

	### No results
	eval {
		if($fasta !~ />/) {
			die AmiRNA::Error->new( { errno => "vseqselect error", msg => "Gene no found in index: $seqid.", subject => "Gene:$seqid" } );
		}
	};
	die if $@;

	### Results, parse fasta
	my $current_gene = '';
	my @fasta_lines = split(/\n/, $fasta);
	for( my $i = 0; $i < @fasta_lines; $i++) {
		chomp($fasta_lines[$i]);

		if($fasta_lines[$i] =~ />/) {
			$current_gene = $fasta_lines[$i];
			$current_gene =~ s/^>//g;
			$current_gene =~ s/^\s+//g;
			$current_gene =~ s/\s.*//g;
			#$current_gene = uc($current_gene);
		}
		else {
			$seqs{$current_gene} .= $fasta_lines[$i];
		}
	}

	foreach my $first_id ( sort {$a cmp $b} keys %seqs ) {
		# Check if correct gene was fetched
		if($first_id !~ /$seqid/) { next;}

		# No range selected (0, 0) -> return gene length
		if( $start == 0 && $end == 0) { return length($seqs{$first_id}); }

		# Check if range is within the gene
		if($start < 0) { $start = 0; }
		if($end >= length($seqs{$first_id}) - 1) { $end = length($seqs{$first_id}) - 1; }		
		my $length = $end - $start + 1;

		return ( substr($seqs{$first_id}, $start, $length) );
	}

	eval {
		die AmiRNA::Error->new( { errno => "vseqselect error", msg => "Gene not found in index: $seqid.", subject => "Gene:$seqid" } );
	};
	die if $@;
}

sub cite_string {
    return <<EOT
http://www.vmatch.de
M.I. Abouelhoda, S. Kurtz, and E. Ohlebusch.
Replacing Suffix Trees with Enhanced Suffix Arrays.
Journal of Discrete Algorithms, 2:53-86, 2004.
EOT
;
}


1;

__END__

=head1 NAME

 AmiRNA::TargetSearch::Vmatch


=head1 SYNOPSIS

 use AmiRNA::TargetSearch::Vmatch;

 my %user_param = (
        configfile         => "/etc/AmiRNA/AmiRNA.xml,
        transcript_library => "A_thaliana_TAIR8.fa",
        mismatches         => 5,
        allow_gaps         => 0,
        strand             => "rev",
        remove_isoform     => 1,
        one_hit_per_gene   => 1,
        calculate_filter   => 1,
	apply_filter       => 1,
        calculate_folding  => "passed",
        folding_backend    => "RNAup",
        temperature        => 23,
        cutoff             => 70,
        relaxed            => 1,
        flanking           => 6,
	showdesc           => 50,
        show_gene_length   => 1,
        sort_mode          => 'ea',
 );

 my $targetSearch = AmiRNA::TargetSearch::Vmatch->new($searchOptions, %user_param);

 my $query = 'ugacagaagagagugagcac';
 $targetSearch->search($query);

 foreach my $target ( @{ $$targetSearch{searchResults}{targets} } ) {
        print "seq_id:\t\t" . $target->{seq_id} . "\n";
        print "query_string:\t" . $target->{query_string} . "\n";
        print "target_string:\t" . $target->{target_string} . "\n";
        print "sequence:\t" . $target->{sequence} . "\n";
        print "query_aln:\t" . $target->{query_aln} . "\n";
        print "target_aln:\t" . $target->{target_aln} . "\n";
        print "gene_length:\t" . $target->{gene_length} . "\n";
        print "begin:\t\t" . $target->{begin} . "\n";
        print "end:\t\t" . $target->{end} . "\n";
        print "score:\t\t" . $target->{score} . "\n";
        print "dG:\t\t" . $target->{dG} . "\n";
        print "dG_ratio:\t" . $target->{dG_ratio} . "\n";
        print "filter_passed:\t" . $target->{filter_passed} . "\n";
        print "message:\t" . $target->{message} . "\n";
        print "---\n";
 }


=head1 DESCRIPTION

 Vmatch Backend. Searches a query string in an enhanced suffix array
 using vmatch available from  http://www.vmatch.de.


=head1 METHODS

=over

=item AmiRNA::TargetSearch::Vmatch-E<gt>new($searchOptions, %user_param)

 Constructs a Vmatch TargetSearch object.

 For $searchOptions see L<AmiRNA::TargetSearch::SearchOptions>

 The %user_param hash requires the following entries:

       transcript_library => transcript library name
       mismatches         => number of allowed mismatches 
       allow_gaps         => allow gaps in alignment, < 0 | 1 >
       strand             => < fwd | rev | both >
       remove_isoform     => count gene only once and neglect additional isoforms, < 0 | 1 >
       one_hit_per_gene   => report only first hit of a gene, < 0 | 1 >
       calculate_filter   => calculate results of MiRNA filter < 0 | 1 >
       apply_filter       => remove filtered hits from SearchResults
       calculate_folding  => use free hybridization energy for filtering < all | passed | non >
       folding_backend    => select RNA folding tool< RNAup | RNAcofold >
       temperature        => select folding temperature, 0 - 99 degree C
       cutoff             => cutoff for imperfect  to perfect energy, 0 to 99
       relaxed            => relaxe MiRNA filter rules, taking all matches with up to 3MM into account,  < 0 | 1 >
       flanking           => show x bases upstream and downstream of target region,
       showdesc           => show number of characters rfom fasta header, 0 - 200, 0 = show all
       show_gene_length   => store length of target gene for visualization. If not set gene length will be set to 0.
       sort_mode          => select sort modde, < ea | ed | ga | gd | la | ld >



=item AmiRNA::TargetSearch::Vmatch-E<gt>search($query)

 Search query string (nucleotides) in a Vmatch index according to the parameters specified in user_param.



=item AmiRNA::TargetSearch::Vmatch-E<gt>parse_results($query, $result_file)

 Parse Vmatch result_file and stores results in L<AmiRNA::TargetSearch::SearchOptions>



=item AmiRNA::TargetSearch::Vmatch-E<gt>check_user_param()

 Test if user parameters are of the correct type.



=item AmiRNA::TargetSearch::Vmatch-E<gt>get_sequence($transcript_id)

 Returns sequence of specified transcript ID $transcript_id.



=item AmiRNA::TargetSearch::Vmatch-E<gt>get_sequence_range($transcript_id, $start, $end)

 Returns sequence in range from $start to $end within the transcript $transcript_id.



=item AmiRNA::TargetSearch::Vmatch-E<gt>cite_string()

 Returns the citation for vmatch.

=back


=head1 DEPENDENCIES

=over

=item Vmatch L<http://www.vmatch.de>

=item L<Data::Types>

=item L<File::Temp>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::TargetSearch::SearchOptions>

=item L<AmiRNA::TargetSearch::SearchResult>

=item L<AmiRNA::TargetSearch::MIRNAFilter>

=item L<AmiRNA::Config::ConfigParser>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski, <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
