#!/usr/bin/perl
#
#    WMD - A tool for miRNA target search and amiRNA design
#
#    Test program for WMD3 classes
#    Version 3.1
#
#    Authors: Stephan Ossowski, Joffrey Fitz, Markus Riester
#    Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.
##########################################################################################

use strict;
use warnings;
use Getopt::Std;
use AmiRNA::Designer::AmiRNADesigner;

# cmd line arguments:
#  -c  configfile
#  -g  transcript_library
#  -f  gene list file

our( $opt_c, $opt_g, $opt_f );
getopts('c:g:f:');

unless ( defined ($opt_g) && (defined $opt_f)) {
	print usage();
	exit;
}

my %jobs = ();
my $job_id = 0;
open GENE, $opt_f or die "Cannot open gene list file\n";
while( <GENE> ) {
	chomp;
	if($_ =~ "###") {
		chop($jobs{$job_id});
		print STDERR "Design job found for gene IDs: " . $jobs{$job_id} . "\n";
		$job_id++;
	}
	else {
		$jobs{$job_id} .= "$_,";
	}
}
chop($jobs{$job_id});
print STDERR "Design job found for gene IDs: " . $jobs{$job_id} . "\n";


foreach my $job_id ( sort {$a <=> $b} keys %jobs ) {
	my $seq_ids = $jobs{$job_id};
	my @genes = split(",", $seq_ids);
	my $musthave = @genes;

	print STDERR "\nNow starting job with gene IDs: $seq_ids\tmusthave: $musthave\n\n";

	my %user_param = (
		configfile         => $opt_c,
		transcript_library => $opt_g,
	        target_fasta       => '',
		target_genes       => $seq_ids,
		musthave           => $musthave,
		annotated          => 1,
		offtargets         => '',
		temperature        => 23,
		relaxed_rules      => 1,
		accelerate_design  => 1,
		flanking           => 3,
	);


	eval {
		my $amiRNAdesigner = AmiRNA::Designer::AmiRNADesigner->new(%user_param);
		$amiRNAdesigner->design();

		foreach my $amiRNA ( @{ $amiRNAdesigner->{miRNA_objs} } ) {
			print	$amiRNA->{mirna_pattern} . "\t" . $amiRNA->{absolute} . "\t" . $amiRNA->{maybe} . "\t" .
				$amiRNA->{relaxed_filter_32} . "\t" . $amiRNA->{relaxed_filter_28_32} . "\t" .
				$amiRNA->{mutations} . "\t" . $amiRNA->{optimal_energy} . "\t" . $amiRNA->{targets} . "\t" .
				$amiRNA->{type} . "\t" . $amiRNA->{score} . "\n";
		}
	};
	if($@) {
	        print "Error: " . $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n";
	        foreach my $x (@{ $@->{'chain'} } ) {
	                foreach my $y ( @{ $x } ) { print "$y\n"; }
	        }
	}
}

exit(0);


sub usage {
	return <<EOT
$0 Version 3.1, Stephan Ossowski, Joffrey Fitz, Markus Riester 2009

Usage: $0 -c <configfile> -g <transcript_library> -f <gene_list_file>

Available Options:

-c      Configfile 
-g      Transcript library
-f      Text file with a list of target gene names. See documentation regarding format.
--help	This help.

Example: $0 -c /etc/AmiRNA/AmiRNA.xml -g TAIR8_cdna_20080412 -f myGeneFile.txt

EOT
;
}
