#!/usr/bin/perl
#
#    WMD - A tool for miRNA target search and amiRNA design
#
#    Test program for WMD3 classes
#    Version 3.1
#
#    Authors: Stephan Ossowski, Joffrey Fitz, Markus Riester
#    Copyright (C) 2009 by Max Planck Institute for Developmental Biology, Tuebingen.
#####################################################################################

use strict;
use warnings;

use Getopt::Std;
use Data::Dumper;
use AmiRNA::Config::ConfigParser;
use AmiRNA::DB::Base;
use AmiRNA::DB::Job;
use AmiRNA::DB::Queue;
use AmiRNA::DB::Results;


# Process command line arguments
our( $opt_f, $opt_c, $opt_t );
getopts('f:c:t:i:');
unless ( defined($opt_f) && defined($opt_t) && defined($opt_t)) {
	print usage();
	exit;
}

our $email = 'you@yourmail.com';
our $ip    = '127.0.0.1';
 
print "\nThis script modifies the database specified in $opt_c\nDo you want to continue ? (yes/no) : "; 
my $line = <STDIN>;
exit(1) unless $line =~ m/^y$|^yes$/i;

print "\n";

### TEST CODE ###########################################################################################################

eval {
	### Connect to database
	my $db = AmiRNA::DB::Base->new($opt_c);
	$db->connect();

	my $queue = AmiRNA::DB::Queue->new($db);
	my $results = AmiRNA::DB::Results->new($db);

	### Create tables Queue, Results
 	if($opt_t != 0) {
 		print "Creating tables\n";
		print "---------------\n";
 		$queue->create_table();
		print "C1: table queue created\n";
 		$results->create_table();
		print "C2: table results created\n\n";
 	}

	### Test job
	my $job1 = AmiRNA::DB::Job->new( 
		( 	sequences 			=> "AT1G74280.1,AT1G74290.1,AT1G74300.1",
			email 				=> $email, 
			transcript_library 	=> "TAIR7_cdna_20070425", 
			target_count 		=> 3,
			musthave 			=> 2,
			ip					=> $ip,
			trusted 			=> 1,
			annotated 			=> 1,
			offtargets 			=> 0,
			description			=> "Multi Target: AT1G74280.1,AT1G74290.1,AT1G74300.1" ) 
	);
	my $job2 = AmiRNA::DB::Job->new(
		( 	sequences 			=> "AT1G01090.1",
			email 				=> $email,
			transcript_library	=> "TAIR7_cdna_20070425",
			target_count 		=> 1,
			musthave 			=> 1,
			ip 					=> $ip,
			trusted				=> 1,
			annotated 			=> 1,
			offtargets 			=> 0,
			description 		=> "Single Target: AT1G01090.1" ) 
	);

	### Test Queue table
	print "Testing queue table\n";
	print "-------------------\n";
	$queue->add_job($job1);
	$queue->add_job($job2);

	print "Q1: Number of jobs in queue: " . $queue->get_number_of_jobs_in_queue() . "\n";
	print "Q2: Number of jobs for ip $ip: " . $queue->get_number_of_jobs_for_ip($ip) . "\n";
	print "Q3: Number of jobs for $email: " . $queue->get_number_of_jobs_for_email($email) . "\n";

	# Delete first job
	print "\nDeleting first job ... ";
	my $next_job = $queue->get_next_job();
	$queue->delete_job( $next_job->{id} );
	print "OK\n\n";
	print "Q4: Number of jobs in queue: " . $queue->get_number_of_jobs_in_queue() . "\n";

	# Delete second job
	print "\nDeleting second job ... ";
	$next_job = $queue->get_next_job();
	$queue->delete_job( $next_job->{id} );
	print "OK\n\n";
	print "Q5: Number of jobs in queue: " . $queue->get_number_of_jobs_in_queue() . "\n";

	### Test Results table
	print "\nTesting results table\n";
	print "-----------------------\n";
	my $result_id_1 = $results->store( $job1, "HELLOWORLD" );
	print "R1: result id 1:                " . $result_id_1 . "\n";
	my $result_id_2 = $results->store( $job2, "BYEWORLD" );
	print "R2: result id 1:                " . $result_id_2 . "\n";

	print "R3: is job 1 already in db? " . $results->check_if_already_in_db($job1) . "\n";

# 	print "R4: get result 10: " . $results->get(10) . "\n";

	print "R5: set email sent for result id 1: " . $results->set_email_sent($result_id_1) . "\n";
	print "R6: get undelivered job: " . $results->get_undelivered_job() . "\n";
	print "R7: set email sent for result id 2: " . $results->set_email_sent($result_id_2) . "\n";

	# Delete result
	print "\nDeleting results ... ";
	$results->delete_result($result_id_1);
	$results->delete_result($result_id_2);
	print "OK\n";
	
	### Drop tables
 	if($opt_f != 0) {
 		print "\nDroping tables ... ";
 		$queue->drop_table();
 		$results->drop_table();
 		print "OK\n";
 	}

	$db->disconnect();

};
if($@) {
	print "Start error message.\n" . $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n";
	foreach my $x (@{ $@->{'chain'} } ) {
		foreach my $y ( @{ $x } ) { print "$y\n"; }
	}
	exit 1;
}

exit(0);

sub usage {
	return <<EOT
$0 Version 3.1, Stephan Ossowski, Joffrey Fitz, Markus Riester 2008

Usage: $0 -f {0|1} -t {0|1} -c configfile

Available Options:

-t    create tables? {0|1}
-f    delete tables when finished? {0|1}
-c    configfile
--help	This help.	

Example: $0 -t 0 -f 0 -c /etc/AmiRNA/AmiRNA.xml

EOT
;
}	
