#!/usr/bin/perl

###############################################################################
#
# designerd.pl - A daemon for the AmiRNA designer
#
# Version: 3.1
#
# Authors: 
#	Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#	Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>
#
# Copyright (c) 2009 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use strict;
use warnings;

use POSIX;
use Log::Log4perl;
#use HTTP::Daemon;
use Net::SMTP;
use XML::Smart;
use AmiRNA::Designer::AmiRNADesigner;
use AmiRNA::DB::Base;
use AmiRNA::DB::Queue;
use AmiRNA::DB::Results;
use AmiRNA::DB::Job;

### Set correct process name
$0 = "designerd.pl";

### Read config file
my @config_locs = ("./designerd.xml", "/etc/AmiRNA/designerd.xml");
my $config_loc = '';
foreach my $l (@config_locs) {
	if(stat $l) {
		$config_loc = $l;
		last;
	}
}
die "Can't locate configuration file (Searched for " . join(" or ", @config_locs) . "\n" unless $config_loc ne "";

my $conf = XML::Smart->new($config_loc) or die $!;
$conf = $conf->cut_root;

### Number of child processes to keep going
my $total_children = $conf->{max_childs};

### If there are no jobs, sleep some seconds before checking job database
my $no_jobs_sleep_time = $conf->{no_jobs_sleep_time};

### PID file
# my $pid_file = '/var/run/designerd.pid';
my $pid_file = $conf->{pid_file};

### AmiRNA configuration
my $AmiRNA_config_file = $conf->{amirna_config};

### Designerd log file
my $log_file = $conf->{log_file};

### Email stuff
my $mailhost = $conf->{mailhost};
my $from_email = $conf->{from_email};
my $wmd3_webapp_url = $conf->{wmd3_webapp_url};


### Initialize the logger
my %log_conf = (
		"log4perl.category"       => "INFO, Logfile",
		"log4perl.appender.Logfile" => "Log::Log4perl::Appender::File",
		"log4perl.appender.Logfile.filename"	=>	"$log_file",
		"log4perl.appender.Logfile.mode"	=>	"append",
		"log4perl.appender.Logfile.layout"	=>	"Log::Log4perl::Layout::PatternLayout",
		"log4perl.appender.Logfile.layout.ConversionPattern"	=>	"%d %p> %m%n",
);
Log::Log4perl->init( \%log_conf );
my $logger = Log::Log4perl::get_logger();

$logger->info("AmiRNA_config_file=$AmiRNA_config_file\n");

# my $d = HTTP::Daemon->new( LocalPort => 2012, LocalAddr => '127.0.0.1', Reuse => 1, Timeout => 180 ) || die "Cannot create socket: $!\n";

my %children;
my $children_count = 0;

### And now become a daemon
&vaporize;
#&confess;
&sentinel;


###############################################################################

sub vaporize {
	my $pid = fork;
	defined ($pid) or die "Cannot start daemon: $!";

	# If we're the shell-called process,
	# let the user know the daemon is now running.
	$logger->info("Parent daemon running, PID=$pid\n") if $pid;
    
	if ($pid) # parent pid
	{
		open PIDFILE, ">$pid_file" or die "can't open $pid_file: $!\n";
		print PIDFILE $pid;
		close PIDFILE;
		
		# If we're the shell-called process, exit back.
		exit;
  	}

	# If we're the shell-called process, exit back.
	# exit if $pid;

	# Detach from shell and close open handles
	POSIX::setsid();
	close(STDOUT);
	close(STDIN);
#	close(STDERR);

	# Set up signals we want to catch: warnings, fatal errors, hangups and dying children
	$SIG{__WARN__} = sub {
		$logger->error("WARN! " . join(" ", @_));
	};

	$SIG{__DIE__} = sub {
		my $sig = shift;
		kill 'INT' => keys %children;
		if($@) {
			$logger->error("FATAL! " . $@->{'msg'} . ": " . $@->{'errno'} . ": " . $@->{'subject'} . "\n" );
		}
		exit;
	};

	$SIG{HUP} = $SIG{INT} = $SIG{TERM} = sub {
		# Any sort of death trigger results in death of all
		my $sig = shift;
		&reset_killed_jobs;
		$SIG{$sig} = 'IGNORE';
		kill 'INT' => keys %children;
		$logger->info("Killed by $sig");;
		exit;
	};

	# We handle our dieing childs in a separate subroutine
	$SIG{CHLD} = \&gravedigger;
}

# Start a web server to allow daemon monitoring
sub confess {
	my $pid = fork;
	defined ($pid) or die "Cannot start child for HTTP server: $!";
	if($pid) {
		$logger->info("started process for HTTP server $pid\n");
		return;
	}
	
	my $d = HTTP::Daemon->new( LocalPort => 2012, LocalAddr => '127.0.0.1', Reuse => 1, Timeout => 180 ) || die "Cannot create socket: $!\n";
	while (1) {
    
		# Accept a connection from HTTP::Daemon
		my $c = $d->accept or last;
		$c->autoflush(1);
		$logger->info("HTTP connect:". $c->peerhost . "\n");

		# Get the request
		my $r = $c->get_request(1) or last;

		# Process the request..
		# you can do whatever you like here..
		# we blindly respond to anything for now..
		my $url = $r->url;
		my $response = HTTP::Response->new(200);
		#     logMessage ($c->peerhost . " " . $d->url . $url . "\n");
		my $content = keys %children;
		$response->content("I'm up and running</br>$content");
		$response->header("Content-Type" => "text/html");
		$c->send_response($response);
		$c->close;
	}
}

# If children sudden die, we have to reset the queue table's started flag to 0
sub reset_killed_jobs {
	my $db = AmiRNA::DB::Base->new($AmiRNA_config_file);
	$db->connect();
	my $queue = AmiRNA::DB::Queue->new($db);
	
	foreach my $pid (keys %children) {
		$logger->info("Resetting job id $children{$pid}");
		$queue->reset_job($children{$pid});
	}
	
	$db->disconnect();
}

# Wait for deceased childs and clean up
sub gravedigger {
	my $cadaver; # The child's pid
	while (($cadaver = waitpid(-1, &WNOHANG)) > 0) { # Is there a child deceased?
		$logger->info("child $cadaver terminated with status $?");
		$children_count--;
		delete $children{$cadaver};
	}
  
	$SIG{CHLD} = \&gravedigger;
}


# Watch periodically for jobs to compute.
# If there are jobs to compute and <$total_childeren processes, start a new one.
sub sentinel {
	while ( 1 ) {

		eval {
			### Connect to database and create Queue object
			my $db = AmiRNA::DB::Base->new($AmiRNA_config_file);
			$db->connect();
			my $queue = AmiRNA::DB::Queue->new($db);

			for (my $i = $children_count; $i < $total_children; $i++ ) {

				### Get next job from queue
				my $pending_jobs = $queue->get_number_of_pending_jobs();
				if( $pending_jobs > 0 ) {			# we got a new job
					&new_child();
					sleep(5);
				}
				else {
# 					$logger->info("Nothing to do - there are no jobs. Sleeping for $no_jobs_sleep_time seconds.\n");
					last;
				}
			}
		};
		if($@) {
			$logger->info( "Error:\n" . $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n" );
			foreach my $x (@{ $@->{'chain'} } ) {
				foreach my $y ( @{ $x } ) { $logger->info( "$y\n" ); }
			}
		}

		sleep($no_jobs_sleep_time);
	}
}


sub new_child {
	### Database handling
	my $db = AmiRNA::DB::Base->new($AmiRNA_config_file);
	$db->connect();
	my $queue = AmiRNA::DB::Queue->new($db);
	my $job = $queue->get_next_job();
	$db->disconnect();
	
	### Fork
	my $pid;
	my $sigset = POSIX::SigSet->new(SIGINT);
	sigprocmask(SIG_BLOCK, $sigset) or die "Can't block SIGINT for fork: $!";
	die "Cannot fork child: $!\n" unless defined ($pid = fork);
	
	if ($pid) {
		$children{$pid} = $job->{id};
		$children_count++;
		$logger->info("Forked new child $pid for next job in queue (job id = $job->{id}). Currently running $children_count children in total.");
		return;
	}

	eval {
		### Configure amiRNADesigner
		my %user_param = (
			configfile         => $AmiRNA_config_file,
			transcript_library => $job->{transcript_library},
			target_fasta       => '',
			target_genes       => '',
			musthave           => $job->{musthave},
			annotated          => $job->{annotated},
			offtargets         => ($job->{offtargets} || 0),
			temperature        => 23,
			accelerate_design  => 1,
			relaxed_rules      => 1,
			flanking           => 0,
		);

		if( $job->{sequences} =~ /^>/) { $user_param{target_fasta} = $job->{sequences}; }
		elsif( $job->{sequences} !~ />/ ) { $user_param{target_genes} = $job->{sequences}; }
		else { $logger->error("Neither gene name nor fasta sequence submitted\n"); exit; }


		## Design amiRNAs
		my $amiRNAdesigner = AmiRNA::Designer::AmiRNADesigner->new(%user_param);
		$logger->info("Start design of job_id " . $job->{id} . " for " . $job->{email} . "\n");
		$amiRNAdesigner->design();


		## Fetch candidates and proofed miRNAs as arrays
		my @candidates = @{ $amiRNAdesigner->{candidates} };
		my @micros     = @{ $amiRNAdesigner->{miRNA_objs} };

		$logger->info("job_id " . $job->{id} . " has " . scalar(@candidates) . " candidates\n");
		$logger->info("job_id " . $job->{id} . " has " . scalar(@micros) . " micros\n");

		### Store results in database and remove job from queue
		$db->connect();
		my $results = AmiRNA::DB::Results->new($db);
		my $result_id = $results->store( $job, [ \@candidates, \@micros ] );
		$queue->delete_job( $job->{id} );
		$db->disconnect();
		$logger->info("Finished job with result_id $result_id for " . $job->{email} . "\n");


		### Send result email to user
		my $EMAILTEXT = "\nLink:\n$wmd3_webapp_url?page=Designer&project=stdwmd&rm=show_results&id=$result_id\nDescription: " . $job->{description} . "\n";
		my $subject = "WMD3: " . $job->{description} . " finished";

		my $smtp = Net::SMTP->new(
			Host    => $mailhost,
			Hello   => $from_email,
			Timeout => 30,
			Debug   => 0,
		) || $logger->error( "Can not create SMTP obj: $! (mailhost => $mailhost)\n");

		$smtp->mail($from_email);
		$smtp->to($job->{email});
		$smtp->data();
		$smtp->datasend("Subject: $subject\n");
		$smtp->datasend("To: " . $job->{email} . "\n");
		$smtp->datasend("From: $from_email\n");
		$smtp->datasend($EMAILTEXT);
		$smtp->dataend();
		$smtp->quit;

		$logger->info("Send job $result_id to user " . $job->{email} . " via $mailhost\n");
	};
	if($@) {
		$logger->info( "Error:\n" . $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n" );
		foreach my $x (@{ $@->{'chain'} } ) {
			foreach my $y ( @{ $x } ) { $logger->info( "$y\n" ); }
		}
	}

	exit;
}
