#!/usr/bin/perl
#
#    WMD - A tool for miRNA target search and amiRNA design
#
#    Index generation for Vmatch and GenomeMapper
#    Version 3.1
#
#    Authors: Stephan Ossowski, Joffrey Fitz, Markus Riester
#    Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology, Tuebingen.
##########################################################################################

use strict;
use warnings;
use Getopt::Std;
use File::Copy;
use AmiRNA::Error;
use AmiRNA::Config::ConfigParser;
use AmiRNA::DB::Base;
use AmiRNA::DB::Sequences;

our( $opt_f , $opt_d, $opt_c, $opt_u );
getopts('f:d:c:u:');

unless ( defined($opt_f) && defined($opt_d) ) {
	print usage();
	exit;
}

my $utr_library = "";
if(defined $opt_u) { $utr_library = $opt_u; }

### Get and check parameters
my $configParser;
my $executable_name;
my $executable_path;
my $index_path;

eval {
	$configParser =  AmiRNA::Config::ConfigParser->new($opt_c);
	$index_path = $configParser->get_index_path();
	$executable_name = $configParser->get_executable_name("mapping");
	$executable_path = $configParser->get_executable_path("mapping");

	if( ($executable_name ne "vmatch") && ($executable_name ne "genomemapper") ) {
		die AmiRNA::Error->new( { errno => "Unknown executable", msg => "Mapping executable is not supported", subject => $executable_name } );
	}
	if( not -e "$executable_path/$executable_name" ) {
		die AmiRNA::Error->new( { errno => $!, msg => "Mapping executable does not exist.", subject => "$executable_path/$executable_name" } );
	}
	if( not -e $opt_f ) {
		die AmiRNA::Error->new( { errno => $!, msg => "Transcript library file does not exist.", subject => $opt_f } );
	}
	if( ($utr_library ne "") && (not -e $utr_library) ) {
		die AmiRNA::Error->new( { errno => $!, msg => "3'UTR library file does not exist.", subject => $opt_u } );
	}
	unless ( stat $index_path ) {
		die AmiRNA::Error->new( { errno => $!, msg => "Index path does not exist.", subject => $index_path } );
	}
};
if($@) {
	print $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n";
	foreach my $x (@{ $@->{'chain'} } ) {
		foreach my $y ( @{ $x } ) { print "$y\n"; }
	}
	exit 127;
}


### Load transcript library into database table 'sequences'
eval {
	### Open database connection
	my $db = AmiRNA::DB::Base->new($opt_c);
	$db->connect();
	my $seq_db = AmiRNA::DB::Sequences->new($db);

	### Create sequences table if not exist
	$seq_db->create_table();

	### Store transcript library
	$seq_db->store_fasta_files( $opt_f );
};
if($@) {
	print "Some errors occured.\n";
	print "Start error message.\n" . $@->{'msg'} . "\n" . $@->{'errno'} . "\n" . $@->{'subject'} . "\n";
	foreach my $x (@{ $@->{'chain'} } ) {
		foreach my $y ( @{ $x } ) { print "$y\n"; }
	}
	print "End error message.\n";
}


### Create transcript library index for mapping tool specified in config file
if($executable_name eq "vmatch") {
	&vmatch_index($executable_path, $opt_f, $index_path, $opt_d, $utr_library);
}
elsif($executable_name eq "genomemapper") {
	&gm_index($executable_path, $opt_f, $index_path, $opt_d, $utr_library);
}

exit(0);


##########################################################################################################
### Generate Vmatch index out of fasta file of transcript library
sub vmatch_index {
	my ($executable_path, $file, $index_folder, $description, $utr_library) = @_;

	### Copy original transcript library file
        my @folder_structure = split("/", $file);
        my $filename = pop(@folder_structure);
        my $newfile = $index_folder . "/" . $filename;
        copy( $file, $newfile );

	if($utr_library ne "") {
		copy( $utr_library, "$newfile.utr3" );
	}

	### Replace all "_" by "-". This is needed because Vmatch is using "_" for blanks in the fasta header!!!
	system("sed -i -e 's/_/-/g' $newfile");
	if($utr_library ne "") {
		system("sed -i -e 's/_/-/g' $newfile.utr3");
	}

	### Write info file
        open NFOFILE, '>', $newfile . '.nfo';
        print NFOFILE $description;
        close NFOFILE;

	### Create Vmatch index
	my $command = "cd $index_folder ; $executable_path/mkvtree -db $newfile -dna -pl 5 -allout -v ";
	system($command);

	if($utr_library ne "") {
		$command = "cd $index_folder ; $executable_path/mkvtree -db $newfile.utr3 -dna -pl 5 -allout -v ";
		system($command);
	}

	return;
}


##########################################################################################################
### Generate GenomeMapper index out of fasta file of transcript library
sub gm_index {
	my ($executable_path, $file, $index_folder, $description, $utr_library) = @_;

	### Extract filename
	my @folder_structure = split("/", $file);
	my $filename = pop(@folder_structure);

	### Init variables
	my $fasta_header = "NA";
	my $gene_seq = "";
	my $start_pos = 1;

	### Open output files
	open TRANS, ">$index_folder/$filename.trans" or die "Cannot open file $index_folder/$filename.trans";
	open NFO,  ">$index_folder/$filename.nfo" or die "Cannot open file $index_folder/$filename.nfo";
	open SEQ,  ">$index_folder/$filename" or die "Cannot open file $index_folder/$filename";
	print NFO $description; close NFO;
	print SEQ ">NULL\n";

	### Parse fasta file
	open FASTA, $file or die "Cannot open fasta file\n";
	while( <FASTA> ) {
		chomp;
		if($_ =~ />/) {
			### print gene
			if($fasta_header ne "NA") {
				$gene_seq =~ tr/uUacgt/TTACGT/;
				print SEQ $gene_seq;
				my $end_pos = $start_pos + length($gene_seq) - 1;
				print TRANS "$start_pos\t$end_pos\t$fasta_header\n";
				$start_pos += length($gene_seq);
				$gene_seq = "";
			}

			### Store fasta header
			$fasta_header = $_;
			$fasta_header =~ s/>//g;
		}
		else {
			### Store gene seq
			$gene_seq .= $_;
		}
	}

	close TRANS; close SEQ; close FASTA;

	### Create GenomeMapper index
	my $cmd = "cd $index_folder ; $executable_path/gmindex -i $index_folder/$filename -s 5 -r ";
	system($cmd);
}


##########################################################################################################
### Usage
sub usage {
	return <<EOT
$0 Version 3.1, Stephan Ossowski, Joffrey Fitz, Markus Riester 2008

Usage: $0 [Options] -f <fastafile> -d <name> -c <configfile>

Available Options:
################################################################################

-f		The filename of the fasta file
-d		A short description of the fasta file
-c              Config file. default: '/etc/AmiRNA/AmiRNA.xml'
-u              3'UTR file
--help	This help.

Example: $0 -f TAIR8_cdna_20080412 -d 'Arabidospis thalaian TAIR8 Release' -c /etc/AmiRNA/AmiRNA.xml

EOT
;
}
