package Designer;

###############################################################################
#
# Designer.pm
#
# Version 3.1
#
# Author: Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#
# Copyright (c) 2008 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use base 'WMD3::WebApp::Base';

use strict;

use WMD3::WebApp::Widgets;
use WMD3::WebApp::Helper;
use WMD3::WebApp::Error;
use WMD3::Plugins::Designer::Backend;
use AmiRNA::DB::Results;

use constant ID          => 2;
use constant PLUGIN_NAME => 'Designer';


use Data::Dumper;
###############################################################################



sub setup {
	my $self = shift;
	$self->start_mode('showform');
	$self->run_modes(
			'showform' => 'showform',
			'compute' => 'compute',
			'show_results' => 'show_results',
			'show_results_xls' => 'show_results_xls',
	);
}

# Execute the following before we execute the requested run mode
sub cgiapp_prerun {
	my $self = shift;

	# Get CGI query object
	my $q = $self->query();
	# Load configuration for plugin and store it
	$self->load_plugin_conf(PLUGIN_NAME);
	# Set current project
	$self->set_project( $q->param('project') || $self->{conf}{web_app}{standard_project} );
	# Setup title, menubar, etc.
	$self->init_view();
	
	my $db = AmiRNA::DB::Base->new( $self->{conf}{amirna}{configfile} );
	$db->connect();
	$self->{db} = $db;
}


sub teardown {
	my $self = shift;
	
	$self->{db}->disconnect();
}

sub showform {
	my $self = shift;
	
	# Get CGI query object
	my $q = $self->query();
	my $full_url = $q->url(-relative => 1) . "?page=" . ID;
	
	# SearchOptions object
	my $searchOptions;
	my %local_dbs_description;
	eval {
		$searchOptions = AmiRNA::TargetSearch::SearchOptions->new( $self->{conf}{amirna}{configfile} );
		%local_dbs_description = $searchOptions->get_transcript_libraries();
	};
	if($@) {	
		$self->stash->{params} = process_error();
		$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');
		return;
	}

	my @local_dbs             = sort { $local_dbs_description{$a} cmp $local_dbs_description{$b} } keys %local_dbs_description;
	$self->stash->{db_popup_menu} = $q->popup_menu(
            -name   => 'transcript_library',
            -id     => 'transcript_library',
            -values => \@local_dbs,
            -labels => \%local_dbs_description,
            -default=> $self->{conf}{amirna}{default_genome},
            -class  => 'form_widget',
        );
    
    	# Collect parameters from ENV and fill out form
	# This allows sticky fields in case of an error
	foreach my $param ($q->param) {
		$self->stash->{$param} = $q->param($param);
	}
	
	# Set the form
	#
	# Add a POST form and hidden parameters
	$self->stash->{start_form} = $q->start_multipart_form;
	$self->stash->{hidden_fields} = $q->hidden('page', $self->{_plugin_name} )
		. $q->hidden('project', $self->get_current_project )
		. $q->hidden('rm', "compute" );
	$self->stash->{end_form} = $q->endform;

	$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/view.tmpl');

	return;
}


sub compute {
	my $self = shift;
	
	# Check for user input errors
	if(not $self->check_input_data) {
		# Redirect to form
		$self->showform();
	
		return;
	}
	
	# Get CGI query object
	my $q = $self->query();
	my $full_url = $q->url(-relative => 1) . "?page=" . ID;

	# User parameters skeleton with default values, 
	# keys will be overidden later
	my %user_param = (
		transcript_library => "",
		email         	   => "",
		musthave           => 0,
		offtargets         => 0,
		sequences               => '',
		ip				   => $q->remote_host,
		trusted			   => 1
	);
	
	# collect parameters from ENV
	foreach my $param ($q->param) {
		$user_param{$param} = $q->param($param);
	}
	
	# remove webapp specific parameters
	delete @user_param{ "rm", "project", "page" };

	# Set specific parameters
	if( $user_param{'offtargets'} > 0) {
		# We do not trust if offtargets were accepted
		$user_param{'trusted'} = 0;
	}
	
	

	my $html;
# 	foreach my $key (sort keys %user_param) {
# 		if($key eq "sequences") {
# 			$html .= "<b>$key</b> => <pre>" . $user_param{$key} . "</pre>";
# 		} else {
# 			$html .= "<b>$key</b> => " . $user_param{$key} . "<br><br>";
# 		}
# 	}

	# Is job already computed?
	my $id = 0;
	$id = check_job($self, %user_param);
	if( $id > 0) { 
		$self->query->param(-name => 'id', -value => $id);
		$self->show_results();
		return;
	}
	else {
		# Submit job
		$html .= "<br>" . submit_job($self, %user_param);

		$self->stash->{params} = $html;
		$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');

		return;
	}
	
	return;
}


sub check_input_data {
	my $self = shift;
	my $q    = $self->query;

	my $err  = 1;

	# Email should be valid
	if(! $q->param('email') or check_email( $q->param('email') ) ) {
		$self->stash->{error_msg} =  $self->print_error_div("Please specify a valid email adress.");
		$err = 0;
	}
	# Determine if we have fasta or gene list input for sequences
	if( guess_input_format( $q->param('sequences') ) eq "fasta" ) {
		# It seems the user input is in FASTA format
		# We check this ...
		if( is_fasta( $q->param('sequences') ) eq "no_fasta" ) {
			$self->stash->{error_msg} = $self->print_error_div("This is not a valid FASTA sequence.");
			$err = 0;
		} else {
			# Count the genes - and check them, they must not be in the annotation database
			my $gene_count = get_gene_count( $self->{'db'}, $q->param('sequences'), $q->param('transcript_library') );
			
			if($gene_count !~ m/^\d+$/) {
				# get_gene_count did not return an integer, print error message
				$self->stash->{error_msg} = $self->print_error_div($gene_count);
				$err = 0;
			} else {
				# Store cleand fasta
				$self->query->param(-name => 'sequences', -value => is_fasta($q->param('sequences') ) );
				# We have FASTA input, so the genes are NOT annotated
				$self->query->param(-name => 'annotated', -value => 0);
				# We do not trust FASTA
				$self->query->param(-name => 'trusted', -value => 1);
				# Set parameter target_count
				$self->query->param(-name => 'target_count', -value => $gene_count);
				# If musthave is not set, this is then the gene count
				if( $q->param('musthave') eq "" ) {
					$self->query->param(-name => 'musthave', -value => $gene_count);
				}
			}
		}
	}
	elsif( guess_input_format( $q->param('sequences') ) eq "gene_list" ) {
		# If it is not FASTA, we have a gene list

		# Count the genes - and check them, they must be in the annotation database
		my $gene_count = get_gene_count( $self->{'db'}, $q->param('sequences'), $q->param('transcript_library') );
		
		if($gene_count !~ m/^\d+$/) {
			# get_gene_count did not return an integer, print error message
			$self->stash->{error_msg} = $self->print_error_div($gene_count);
			$err = 0;
		} else {
			# We have gene name input, so the genes ARE annotated
			$self->query->param(-name => 'annotated', -value => 1);
			# If musthave is not set, this is then the gene count
			if( $q->param('musthave') eq "" ) {
				$self->query->param(-name => 'musthave', -value => $gene_count);
			}
			# Maximum is gene count
			if( $q->param('musthave') > $gene_count) {
				$self->query->param(-name => 'musthave', -value => $gene_count);
			}
			# Store the gene count
			$self->query->param(-name => 'target_count', -value => $gene_count);
		}
		
		# Clean the gene list to a csv line
		my $tmpseq = $q->param('sequences');
		$tmpseq =~ s/\r\n/\n/gm;
		$tmpseq =~ s/\n\n/\n/gm;
		$tmpseq =~ s/\n/,/gm;
		$tmpseq =~ s/,+/,/g;
		$tmpseq =~ s/\s//gm;
		$self->query->param(-name => 'sequences', -value => $tmpseq);
		
		# Check gene names, the list must be unique and no different isoforms of a genes are allowed
		if(check_unique_gene_name($tmpseq) == 1) {
			$self->stash->{error_msg} = $self->print_error_div("Please specify a unique gene list. Do not specify different isoforms of one gene.<br>For spliceform specific silencing please consult the <a href=\"webapp.cgi?page=Help;project=stdwmd#procedure\">manual</a>.");
			$err = 0;
		}
	}
	
	# There must be input specified in the sequences field
	if(! $q->param('sequences') ) {
		$self->stash->{error_msg} = $self->print_error_div("Please specify target genes as FASTA sequence or as a gene list.");
		$err = 0;
	}
	
	if( $q->param('offtargets') eq "" ) {
		$self->query->param(-name => 'offtargets', -value => 0);
	}

	return $err;
}


sub show_results {
	my $self = shift;
	
	# Get CGI query object
	my $q = $self->query();
	my $full_url = $q->url(-relative => 1) . "?page=" . ID;
	
	my %user_param;
	my $html;
	
	# Check input, ID is mandatory
	if(! $q->param('id') ) {
		$self->stash->{error_msg} = $self->print_error_div("Please specify a result ID.");
		$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');
		return;
	}
	
	# collect parameters from ENV
	foreach my $param ($q->param) {
		$user_param{$param} = $q->param($param);
	}

	$html .=  get_results($self, $user_param{'id'}, "html");

	$self->stash->{params} = $html;
	$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');

	return;	
}

sub show_results_xls {
	my $self = shift;

	# Get CGI query object
	my $q = $self->query();
	my $full_url = $q->url(-relative => 1) . "?page=" . ID;

	my %user_param;

	# Check input, ID is mandatory
	if(! $q->param('id') ) {
		$self->stash->{error_msg} = $self->print_error_div("Please specify a result ID.");
		$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');
		return;
	}

	# collect parameters from ENV
	foreach my $param ($q->param) {
		$user_param{$param} = $q->param($param);
	}
	
	print $q->header(
		-type => 'application/vnd.ms-excel',
		-'Content-Disposition' => "attachment; filename=wmd3_results" . $user_param{'id'} . ".xls",
    );
	binmode STDOUT;

	get_results($self, $user_param{'id'}, $q->param('format'));
	
}

1;
