package WMD3::Plugins::Designer::Backend;

###############################################################################
#
# Designer::Backend
#
# Version 3.1
#
# Author: Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#		  Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>
#         Markus Riester
#		  
# Copyright (c) 2005-2008 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use strict;
use warnings;

use AmiRNA::DB::Base;
use AmiRNA::DB::Queue;
use AmiRNA::DB::Results;
use WMD3::WebApp::Error;
use Spreadsheet::WriteExcel;
 

require      Exporter;

our @ISA       = qw(Exporter);
our @EXPORT    = qw(check_job submit_job get_results);


##############################################################################
#
# deliver_stored_results()
#
# If a job was already computed and stored in the database (results table)
# we can deliver these results.
# 
# The results must conform the following rules
#	1. musthave must be identical
#	2. trusted must be 1 in database
#	3. genome release mist be the same
#
##############################################################################
sub deliver_stored_results {
	my $self = shift;
	my %user_param = @_;

# select id, seq_ids, transcript_library, musthave, trusted from results;
# +----+-------------------------------+---------------------+----------+---------+
# | id | seq_ids                       | transcript_library  | musthave | trusted |
# +----+-------------------------------+---------------------+----------+---------+
# |  5 | AT1G01080.1                   | TAIR7_cdna_20070425 |        1 |       1 |
# |  9 | AT1G01070                     | TAIR8_cdna_20080412 |        1 |       0 |
# |  7 | AT1G01090.1                   | TAIR8_cdna_20080412 |        1 |       0 |
# | 10 | AT1G74280,AT1G74290,AT1G74300 | TAIR8_cdna_20080412 |        2 |       0 |
# +----+-------------------------------+---------------------+----------+---------+

# select id from results where seq_ids="AT1G74280,AT1G74290,AT1G74300" and trusted=1 and musthave=2 and transcript_library="TAIR8_cdna_20080412";
}

##############################################################################
#
# check_job() -- Is job already computed and stored in the reuslts table?
#
##############################################################################
sub check_job {
	my $self = shift;
	my %user_param = @_;
	
	my ($results, $job);
	my $id = 0;
	
	# Connect to database
	eval {
		$results = AmiRNA::DB::Results->new($self->{db});
	
		$job = AmiRNA::DB::Job->new( %user_param );
		
		$id = $results->check_if_already_in_db( $job );
	};
	return process_error() if ($@);
	
	return $id;
}

##############################################################################
#
# submit_job()
#
##############################################################################
sub submit_job {
	my $self = shift;
	my %user_param = @_;
	
	my $html = "";
	my ($db, $job, $queue, $results, $job_id);
	
		
	# Connect to database
	eval {
		$queue = AmiRNA::DB::Queue->new($self->{db});
		$results = AmiRNA::DB::Results->new($self->{db});
	
		$job = AmiRNA::DB::Job->new( %user_param );
		
		$job_id = $queue->add_job($job);
	};
	return process_error() if ($@);
	
	$html = "Job successfully submitted (ID: $job_id).<p>You will recieve an E-Mail upon job completion.";

	return $html
}

##############################################################################
#
# get_results()
#
##############################################################################
sub get_results {
	my $self = shift;
	my $job_id = shift;
	my $format = shift;
	
	my $html = "";
	
	# xls
	my ($workbook, $worksheet, $xls_cell_format);
	my $xls_row=0;
	if($format eq "xls") {
		$workbook = Spreadsheet::WriteExcel->new('-');
		$worksheet = $workbook->add_worksheet();
		# Set column width
		$worksheet->set_column('A:A', 2);
		$worksheet->set_column('B:B', 28);
		$worksheet->set_column('E:E', 10);
		$worksheet->set_column('G:G', 10);
		$worksheet->set_column('I:I', 10);
		$worksheet->set_column('K:K', 10);
		# Set header format
		my $xls_wmd3_color = $workbook->set_custom_color(40, 46, 195, 38);
		my $xls_cell_h1_format = $workbook->add_format(bg_color => 'white', color => $xls_wmd3_color, size => 20, bold => 1);
		$worksheet->write($xls_row, 1,  "WMD3 Results", $xls_cell_h1_format);
		$xls_row++;
		my $xls_cell_h2_format = $workbook->add_format(color => $xls_wmd3_color, bold => 1);
		$worksheet->write($xls_row, 1,  "$self->{conf}{web_app}{cgi}?page=Designer;rm=show_results;id=$job_id", $xls_cell_h2_format);
		$xls_row+=2;
		$worksheet->write($xls_row, 1,  "amiRNAs ranked by efficiency and specificity using empirical data.");
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Green amiRNAs tend to have the best properties. Red amiRNAs are fully functional but have less favorable properties like minor off-targets or weak hybridization to their target site.");
		$xls_row+=2;
	}
	
	my $results = AmiRNA::DB::Results->new( $self->{'db'} );
	my ($job, $mirna_objs);
	
	eval {
		($job, $mirna_objs)  = $results->get($job_id);
	};
	return process_error() if ($@);
	
	if(not $job) {
		return "Job ID $job_id not found.";
	}
	
	# Print job information
	$html =<<EOH;
<table>
<tr>
	<td>Transcript library:</td><td>$job->{transcript_library}</td>
</tr><tr>
	<td>Target genes:</td><td>$job->{sequences}</td>
</tr><tr>
	<td>Description:</td><td>$job->{description}</td>
</tr><tr>	
	<td>Min. number of included targets:</td><td>$job->{musthave}</td>
</tr><tr>	
	<td>Accepted off-targets:</td><td>$job->{offtargets}</td>
</tr><tr>	
	<td>Annotated:</td><td>$job->{annotated}</td>
</tr>
EOH

	if(scalar(@$mirna_objs)==0) {
		$html .= "</table>\n" . "<p><b>Unfortunately WMD3 could not design any microRNAs.</b></p>\n";
		return $html;
	}
	else {
		$html .= "<tr><td><a href=\"?page=Designer;project=stdwmd;rm=show_results_xls;id=$job_id;format=xls\">Download xls</a></td></tr></table>";
	}
	
	if($format eq "xls") {
		my $format = $workbook->add_format();
		$format->set_align('left');
		$worksheet->write($xls_row, 1,  "Transcript library:", $format);
		$worksheet->write($xls_row, 2,  $job->{transcript_library}, $format);
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Target genes:", $format);
		$worksheet->write($xls_row, 2,  $job->{sequences}, $format);
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Description:", $format);
		$worksheet->write($xls_row, 2,  $job->{description}, $format);
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Min. number of included targets:", $format);
		$worksheet->write($xls_row, 2,  $job->{musthave}, $format);
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Accepted off-targets:", $format);
		$worksheet->write($xls_row, 2,  $job->{offtargets}, $format);
		$xls_row++;
		$worksheet->write($xls_row, 1,  "Annotated:", $format);
		$worksheet->write($xls_row, 2,  $job->{annotated}, $format);
		$xls_row+=2;
	}
	
	my @micros;
	if ( $job->{'musthave'} eq 1 ) {
		@micros = sort { $a->{score} <=> $b->{score} } @$mirna_objs;
	}
	else {
		@micros = sort { sort_result($a, $b) } @$mirna_objs;
	}

	my $last_targets = '';
	foreach my $micro (@micros) {
		my $classname = 'white';
		if ( $job->{'musthave'} eq 1 ) {
			if ($last_targets eq '') {
				$html .= '<pre>';
				$last_targets = $job->{'sequences'};
			}    
			if ( $micro->{score} <= 15 ) {
				$classname = 'green';
			}
			elsif ( $micro->{score} <= 20 ) {
				$classname = 'yellow';
			}
			elsif ( $micro->{score} <= 25 ) {
				$classname = 'orange';
			}
			elsif ( $micro->{score} > 25 ) {
				$classname = 'red';
			}
		}
		else {
			my @targets = _get_targets($micro);
			my $s_targets = join ", ", @targets;

			if ($s_targets ne $last_targets) {
				$html .= '</pre>' if $last_targets ne ''; 
				$html .= "<br/><b>Targets: $s_targets</b><pre class=\"target_search_results_box\">";
				if($format eq "xls") {
					$xls_row++;
					my $xls_cell_h2_format = $workbook->add_format();
					$xls_cell_h2_format->set_bold(1);
					$worksheet->write($xls_row, 1,  "Targets: $s_targets", $xls_cell_h2_format);
					$xls_row++;
					# Column description
					$worksheet->write($xls_row, 1,  "amiRNA", $xls_cell_h2_format);
					$worksheet->write($xls_row, 2,  "perfect match hybridizatio energy (kcal/mole)", $xls_cell_h2_format);
					$worksheet->write($xls_row, 4,  "target gene id", $xls_cell_h2_format);
					$worksheet->write($xls_row, 5,  "hybridization energy", $xls_cell_h2_format);
					$worksheet->write($xls_row, 6,  "...", $xls_cell_h2_format);
					$xls_row++;
				}
				$last_targets = $s_targets;
			}     
			if ( $micro->{score} <= 25 ) {
				$classname = 'green';
			}
			elsif ( $micro->{score} <= 35 ) {
				$classname = 'yellow';
			}
			elsif ( $micro->{score} <= 45 ) {
				$classname = 'orange';
			}
			elsif ( $micro->{score} > 45 ) {
				$classname = 'red';
			}
		}

		my $url_to_patsearch =  $self->{conf}{web_app}{cgi} 
			. '?page=TargetSearch;rm=compute;project=stdwmd;q=' 
			. $micro->{mirna_pattern} 
			. ';transcript_library='
			. $job->{transcript_library};

		$html .= "<span class=\"${classname}mirna\"><a href=\"$url_to_patsearch\">"
			. $micro->{mirna_pattern}
			. "</a></span>\t"
			. $micro->{optimal_energy} . "\t"
			. $micro->{targets} . "\n";
        
        if($format eq "xls") {
			my $xls_wmd3_color;
			if($classname eq 'green') {
				$xls_wmd3_color = $workbook->set_custom_color(40, 46, 195, 38);
			} elsif($classname eq 'yellow') {
				$xls_wmd3_color = $workbook->set_custom_color(41, 255, 255, 0);
			} elsif($classname eq 'orange') {
				$xls_wmd3_color = $workbook->set_custom_color(42, 255, 165, 0);
			} elsif($classname eq 'red') {
				$xls_wmd3_color = $workbook->set_custom_color(43, 255, 0, 0);
			}
			my $xls_cell_bg_format = $workbook->add_format(bg_color => $xls_wmd3_color);
        	$worksheet->write($xls_row, 0,  "", $xls_cell_bg_format);
			$worksheet->write($xls_row, 1,  $micro->{mirna_pattern});
			$worksheet->write($xls_row, 2,  $micro->{optimal_energy});
			my @targets = split("\t", $micro->{targets});
			my $col_offset=4;
			for(my $i=0; $i<scalar(@targets); $i++) {
				$worksheet->write($xls_row, $i+$col_offset,  $targets[$i]);
			}
			$xls_row++;
		}
	}
		
	
	$html .= "</pre>";

	if($format eq "xls") {
		$workbook->close();
	} else {	
		return $html;
	}
}

###############################################################################
### Sort results by ranking score (low score is better)
sub sort_result {
    my ($a, $b) = @_;
    my @targets_a =  _get_targets($a);
    my @targets_b =  _get_targets($b);
    my $res = scalar(@targets_b) <=> scalar(@targets_a);
    return $res if $res != 0;
    $res = join(' ', @targets_a) cmp 
           join(' ', @targets_b);
    return $res if $res != 0;
    return $a->{score} <=> $b->{score};
}
###############################################################################
### Get targets to print on webpage
sub _get_targets {
    my ($a) = @_;

	my @items = split "\t", $a->{targets};
	my %th = @items;
	my @targets = sort keys %th;
	
	return @targets;
}

1;
