package TargetSearch;

###############################################################################
#
# TargetSearch.pm
#
# Version 3.1
#
# Author: Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#
# Copyright (c) 2008 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use constant ID          => 1;
use constant PLUGIN_NAME => 'TargetSearch';

use base 'WMD3::WebApp::Base';
use WMD3::WebApp::Widgets;
use WMD3::WebApp::Error;
use WMD3::Plugins::TargetSearch::Backend;

use strict;

###############################################################################

# Default form values

# cutoff
my @cutoff_values = (10, 20, 30, 40, 50, 60, 65, 70, 75, 80, 85, 90, 95);
my %cutoff_labels = (
	10 => 10,
	20 => 20,
	30 => 30,
	40 => 40,
	50 => 50,
	60 => 60,
	65 => 65,
	70 => 70,
	75 => 75,
	80 => 80,
	85 => 85,
	90 => 90,
	95 => 95
);

# apply_filter
my @apply_filter_values = (1, 0);
my %apply_filter_labels = (1 => "Yes", 0 => "No");
	
# strand direction
my @direction_values = ("fwd", "rev");
my %direction_labels = (
	"fwd" => "forward", 
	"rev" => "reverse"
);

# allow gaps popup
my @allow_gaps_values = (0, 1);
my %allow_gaps_labels = (1 => "Yes", 0 => "No");

# show only one isoform popup
my @show_only_one_iso_values = (1, 0);
my %show_only_one_iso_labels = (1 => "Yes", 0 => "No");

# sort mode popup
my @sort_mode_values = ("ea", "ed", "ga", "gd");
my %sort_mode_labels = (
	"ea" => "ascending order of Evalue",
	"ed" => "descending order of Evalue",
	"ga" => "ascending order of dG",
	"gd" => "descending order of dG"
);
				
# folding backend
my @folding_backend_values = ("RNAup", "RNAcofold");
my %folding_backend_labels = (
	"RNAup" => "RNAup", 
	"RNAcofold" => "RNAcofold",
);

###############################################################################

my %user_param;

###############################################################################

sub setup {
	my $self = shift;
	$self->start_mode('showform');
	$self->run_modes(
			'showform' => 'showform',
			'compute' => 'compute',
			'show_gfx' => 'show_gfx',
			'showsequence' => 'showsequence'
	);
}

# Execute the following before we execute the requested run mode
sub cgiapp_prerun {
	my $self = shift;

	# Get CGI query object
	my $q = $self->query();
	# Load configuration for plugin and store it
	$self->load_plugin_conf(PLUGIN_NAME);
	# Set current project
	$self->set_project( $q->param('project') || $self->{conf}{web_app}{standard_project} );
	# Setup title, menubar, etc.
	$self->init_view();
}


sub teardown {

}


sub prepareform {
	my $self = shift;
	
	# Get CGI query object
	my $q = $self->query();
	my $full_url = $q->url(-relative => 1) . "?page=" . ID;

	# Default user parameters
	%user_param = (
		one_hit_per_gene   => 1,
		relaxed            => 1,	# < 0 | 1 >
		showdesc           => 50,	# 0 - any, 0 = off
		show_gene_length   => 1,
		calculate_filter   => 1,
		apply_filter       => 1,
		transcript_library => $self->{conf}{amirna}{default_genome},
		cutoff 			   => 70,
		strand 			   => 'rev',
		allow_gaps 		   => 0,
		remove_isoform 	   => 1,
		sort_mode 		   => 'ga',
		folding_backend    => 'RNAcofold',
		mismatches 	 	   => 5,
		temperature 	   => 23,
		flanking 	       => 0,
		microrna_filter_disclosure_state  => 'hidden',
		advanced_options_disclosure_state => 'hidden',
	);

	# Collect parameters from ENV and add to user parameters
	foreach my $param ($q->param) {
		$user_param{$param} = $q->param($param);
	}
	
	# Set calculate_folding depending on user input
	if($user_param{'apply_filter'} eq 1) {
		$user_param{'calculate_folding'} = "passed";
	} else {
		$user_param{'calculate_folding'} = "all";
	}
	
	# SearchOptions object
	my $searchOptions;
	my %local_dbs_description;
	eval {
		$searchOptions = AmiRNA::TargetSearch::SearchOptions->new( $self->{conf}{amirna}{configfile} );
		%local_dbs_description = $searchOptions->get_transcript_libraries();
	};
	if($@) {	
		$self->stash->{params} = process_error();
		$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/results.tmpl');
		return;
	}

	my @local_dbs             = sort { $local_dbs_description{$a} cmp $local_dbs_description{$b} } keys %local_dbs_description;
	$self->stash->{db_popup_menu} = $q->popup_menu(
            -name   => 'transcript_library',
            -id     => 'transcript_library',
            -values => \@local_dbs,
            -labels => \%local_dbs_description,
            -default=> $user_param{transcript_library},
            -class  => 'form_widget',
        );

	# Add the Search textbox
	my $searchbox = $q->textfield(-name=>'q',
                    			-value=>$user_param{'q'},
                    			-class=>'form_widget',
                    			-id=>'q',
	);
	$self->stash->{search} = $searchbox;
	
	# Add the apply_filter popup
	my @apply_filter_values = (1, 0);
	my %apply_filter_labels = (1 => "Yes", 0 => "No");
	$self->stash->{applyfilter_popup_menu} = $q->popup_menu(
            -name   => 'apply_filter',
            -id     => 'apply_filter',
            -values => \@apply_filter_values,
            -labels => \%apply_filter_labels,
            -default=> $user_param{'apply_filter'},
            -class  => 'form_widget',
        );
       
    # Add the cutoff popup
	$self->stash->{cutoff_popup_menu} = $q->popup_menu(
            -name   => 'cutoff',
            -id     => 'cutoff',
            -values => \@cutoff_values,
            -labels => \%cutoff_labels,
            -default=> $user_param{'cutoff'},
            -class  => 'form_widget',
        );
       
    # Add the strand direction popup
	$self->stash->{direction_popup_menu} = $q->popup_menu(
            -name   => 'strand',
            -id     => 'strand',
            -values => \@direction_values,
            -labels => \%direction_labels,
            -default=> $user_param{'strand'},
            -class  => 'form_widget',
        );
       
    # Add the allow gaps popup
	$self->stash->{allow_gaps_popup_menu} = $q->popup_menu(
            -name   => 'allow_gaps',
            -id     => 'allow_gaps',
            -values => \@allow_gaps_values,
            -labels => \%allow_gaps_labels,
            -default=> $user_param{'allow_gaps'},
            -class  => 'form_widget',
        );
        
    # Add the show_only_one_iso popup
	$self->stash->{show_only_one_iso_popup_menu} = $q->popup_menu(
            -name   => 'remove_isoform',
            -id     => 'show_only_one_iso',
            -values => \@show_only_one_iso_values,
            -labels => \%show_only_one_iso_labels,
            -default=> $user_param{'show_only_one_iso'},
            -class  => 'form_widget',
        );
        
    # Add the sort_mode popup
	$self->stash->{sort_mode_popup_menu} = $q->popup_menu(
            -name   => 'sort_mode',
            -id     => 'sort_mode',
            -values => \@sort_mode_values,
            -labels => \%sort_mode_labels,
            -default=> $user_param{'sort_mode'},
            -class  => 'form_widget',
        );
          
    # Add the folding backend popup
	$self->stash->{folding_backend_popup_menu} = $q->popup_menu(
            -name   => 'folding_backend',
            -id     => 'folding_backend',
            -values => \@folding_backend_values,
            -labels => \%folding_backend_labels,
            -default=> $user_param{'folding_backend'},
            -class  => 'form_widget',
        );
                       
	# Set the form
	#
	# Add a POST form and hidden parameters
	$self->stash->{start_form} = $q->start_multipart_form;
	$self->stash->{hidden_fields} = $q->hidden('page', $self->{_plugin_name} )
		. $q->hidden('project', $self->get_current_project )
		. $q->hidden('rm', "compute" );
	$self->stash->{end_form} = $q->endform;
	
	# Set the values
	$self->stash->{mismatches_value} = $user_param{mismatches};
	$self->stash->{temperature_value} = $user_param{temperature};
	$self->stash->{flanking_value} = $user_param{flanking} == 0 ? "&#48;" : $user_param{flanking};
	
	# Set disclosure panels
	$self->stash->{microrna_filter_disclosure_state} = $user_param{'microrna_filter_disclosure_state'};
	$self->stash->{advanced_options_disclosure_state} = $user_param{'advanced_options_disclosure_state'};

	return;
}

sub showform {
	my $self = shift;
	my $q    = $self->query;
	
	$self->prepareform();
	
	$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/view.tmpl');
	
	return;
}


sub check_input_data {
	my $self = shift;
	my $q    = $self->query;

	my $err  = 1;

	# There must be input specified in the search field
	if(! $q->param('q') ) {
		$self->stash->{error_msg} = $self->print_error_div("Please specify a search query.");
		$err = 0;
	}
	
	if($q->param('flanking') eq "") {
		$self->query->param(-name => 'flanking', -value => 0);
	}
	
	return $err;
}


sub compute {
	my $self = shift;
	my $html;
	
	$self->prepareform();
	
	# safe some params
	my $query = $user_param{'q'};
	my $microrna_filter_disclosure_state = $user_param{'microrna_filter_disclosure_state'};
	my $advanced_options_disclosure_state = $user_param{'advanced_options_disclosure_state'};
	
	
	# Compute the results
	# Remove webapp specific parameters
	delete @user_param{"rm", "project", "page", "q", "microrna_filter_disclosure_state", "advanced_options_disclosure_state"};
	
	# Perform target search
	$html .= "<br>" . search($self, $query, %user_param);

	$self->stash->{results} = '<br><hr noshade size="1"><br><h2 style="text-align: left;" class="title">Results</h2>'.$html;
	$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/view.tmpl');
	
	return;
}

sub show_gfx {
	my $self = shift;
	my $q = $self->query();

	my $gene_start   = 0;
	my $target_start = $q->param('target_start');
	my $gene_end     = $q->param('gene_length');
	my $target_length= $q->param('target_end') - $q->param('target_start');
	
	use WMD3::Plugins::TargetSearch::AmiRNAGfx;
	my $img = WMD3::Plugins::TargetSearch::AmiRNAGfx->new();
	$img->draw($gene_start, $gene_end, $target_start, $target_length);
	print "Content-type: image/png\n\n";
	binmode STDOUT;
	# Convert the image to PNG and print it on standard output
	print $img->{img}->png();
}

sub showsequence {
	my $self = shift;
	
	# Get CGI query object
	my $q = $self->query();
	
	# Get the sequence
	my $html .= "<br>" . getseq($self, $q->param('transcript_library'), $q->param('seq_id'));
	
	$self->stash->{transcript_library} = $q->param('transcript_library');
	$self->stash->{seq_id} = $q->param('seq_id');
	$self->stash->{seq} = $html;
	$self->stash->{template} = $self->load_tmpl($self->plugin_conf('template_path').'/seq.tmpl');
	
	return;
}

1;
