package WMD3::Plugins::TargetSearch::Backend;

###############################################################################
#
# TargetSearch::Backend
#
# Version 3.1
#
# Author: Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#		  Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>
#         Markus Riester
#		  
# Copyright (c) 2008 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use strict;
use warnings;

require      Exporter;

use AmiRNA::TargetSearch::SearchOptions;
use AmiRNA::TargetSearch::Vmatch;
use AmiRNA::TargetSearch::GenomeMapper;
use WMD3::WebApp::Error;

our @ISA       = qw(Exporter);
our @EXPORT    = qw(search getseq);


##############################################################################
#
# search() - perform a target search and return HTML
#
##############################################################################

sub search {
	my $self = shift;
	my $query = shift;
	my (%user_param) = @_;
	
	# SearchOptions object
	my $searchOptions = AmiRNA::TargetSearch::SearchOptions->new( $self->{conf}{amirna}{configfile} );
	
	# Create TargetSearch object
	my $targetSearch;
	eval {
		if( $searchOptions->get_search_program() eq "vmatch" ) {
			$targetSearch = AmiRNA::TargetSearch::Vmatch->new($searchOptions, %user_param);
		}
		elsif( $searchOptions->get_search_program() eq "genomemapper" ) {
			$targetSearch = AmiRNA::TargetSearch::GenomeMapper->new($searchOptions, %user_param);
		}
		
		$targetSearch->search( $query );
	};
	return process_error if $@;

	# Assemble HTML
	my $html;
	my $toc;
	my $cntr=0;
	$toc  = "Searching for <b>" . $self->query->param('q') . "</b> in database " . $self->query->param('transcript_library') . "<br><br><br>";
	$toc .= "<b>results_count</b> search results.<br><br>";
	
	eval{
		foreach my $target ( @{ $$targetSearch{searchResults}{targets} } ) {
			if( 
				( ($user_param{'apply_filter'} eq 1) && ($target->{filter_passed} == 1) ) ||		 # Case: Apply filter && Filter passed
				( ($target->{message} =~ /Possible candidate/) && ($user_param{'relaxed'} eq 1) ) || # Case: Possible candidate with <= 3MM
				( ($user_param{'apply_filter'} eq 0) )							                     # Case: Do not apply filter, print all
			) {

				$cntr++;
				$toc .= "<a style=\"padding-left:30px;\" href=\"#" . $target->{seq_id} ."\">" . $target->{seq_id} . "</a><br>\n";
				$html .= "<a name=\"" . $target->{seq_id} . "\" /></a>";
				$html .= "<pre class=\"target_search_results_box\">";
				$html .= "<a href=\"webapp.cgi?page=TargetSearch&rm=showsequence&seq_id=" . $target->{seq_id} . "&transcript_library=" . $user_param{'transcript_library'} . "\">";
				$html .= $target->{fasta_header} . "</a>\n";
				$html .= "Target region: " . $target->{sequence} . "\n";
				$html .= "\n";
				if( $target->{message} ne "passed" ) {
					$html .= "<span class=\"target_search_result_message\">" . $target->{message} . "</span>\n\n";
				}
				$html .= "Hybridization energy: " . $target->{dG} . "kcal/mol (" . sprintf("%.2f", $target->{dG_ratio}) . " %)\n";
				$html .= "\n";
				
				$html .= "<table cellpadding=\"0\" cellspacing=\"0\">";
				$html .= "<tr><td style=\"padding-left:0px;\">Target gene 5'->3'/" . $target->{begin} . "-" . $target->{end} . "\t</td><td style=\"padding-left:18px;\">" . $target->{target_aln} . "</td></tr>\n";
				$html .= "<tr><td style=\"padding-left:0px;\">amiRNA (rev. complement)   \t</td><td style=\"padding-left:18px;\">" . $target->{query_aln} . "</td></tr>\n";
				
				
# 				$html .= "Target gene 3'->5'/" . $target->{begin} . "-" . $target->{end} . "\t" . $target->{target_aln} . "\n";
# 				$html .= "amiRNA (rev. complement)   \t" . $target->{query_aln} . "\n";
				
				my $aln;
				my @target = split //, $target->{target_aln};
				my @query = split //, $target->{query_aln};
				for(my $i=0; $i<scalar(@target); $i++) {
					if($target[$i] eq $query[$i]) {
						$aln .= "*";
					} else {
						$aln .= " ";
					}
					
				}
				$html .= "<tr><td>\t</td><td style=\"padding-left:18px;\">$aln</td></tr>\n\n";
				$html .= "</table>";

				$html .= "<img src=\"" . $self->{conf}{cgi} . "?page=TargetSearch;rm=show_gfx;gene_length=" .$target->{gene_length}
				. ";target_start=" . $target->{begin} . ";target_end=" . $target->{end} . "\" />";
	
				$html .= "</pre>";
				$html .= "<a href=\"#top\">top</a><br><br>";
			}
		}
		$toc .= "</ul><br><br><br><b>Alignments</b><br><br><br>";
		
	};
	return process_error if $@;
	
	$toc =~ s/results_count/$cntr/;
	
	return $toc.$html;
}

##############################################################################
#
# getseq() - retrieve sequence for gene and return HTML
#
##############################################################################

sub getseq {
	my $self = shift;
	my $id = shift;
	my $transcript_library = shift;
	
# 	my $seq = get_cdna()
	my %results;
	my $reshtml;
	
	my $db;
	
	eval {
		# Database connection
		$db = AmiRNA::DB::Base->new( $self->{conf}{amirna}{configfile} );
		$db->connect();
		
		# Create Sequences object
		my $seq_db = AmiRNA::DB::Sequences->new($db);
		
		my %results = ();
		%results = $seq_db->get_cdna($id, $transcript_library);
		
		use Data::Dumper;
		print STDERR "Dump: " . Dumper(%results) . "\n-----------------\n";
		# Output FATA
		use Text::Wrap;
		$Text::Wrap::columns = 72;
		foreach (sort {$a cmp $b} keys %results) {
		print STDERR "\n* in foreach \n\n";
			my @output = (">$_",  $results{ $_ });
			$reshtml .= wrap("", "", @output) . "\n\n";
			print STDERR wrap("", "", @output) . "\n\n";
		}
	};
	$db->disconnect();
	return process_error if $@;
	use Data::Dumper;
	return "<pre>" . $reshtml . "</pre>";
}
1;
