package WMD3::WebApp::Helper;

###############################################################################
#
# WMD3::WebApp::Helper
#
# Version 3.1
#
# Author: Joffrey Fitz <joffrey.fitz@tuebingen.mpg.de>
#
# Copyright (c) 2009 Max Planck Institute for Developmental Biology, 
# 	http://www.eb.tuebingen.mpg.de
#
# This program is free software; you can redistribute it and/or modify it                        
# under the terms of the GNU General Public License as published by the Free                     
# Software Foundation; either version 2, or (at your option) any later 
# version.
#
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details.
#
###############################################################################

use strict;
use warnings;

require Exporter;
use Data::Dumper;

our @ISA       = qw(Exporter);
our @EXPORT    = qw(check_email is_fasta guess_input_format get_gene_count check_unique_gene_name);



##############################################################################
#
# check_email($email)
#
#	Checks if email is syntactical correct, returns 0|1
#	Code adapted from perlfaq9 
#
##############################################################################
sub check_email {
	my $email = shift;
	
	my $atom       = qr{[a-zA-Z0-9_!#\$\%&'*+/=?\^`{}~|\-]+};
    my $dot_atom   = qr{$atom(?:\.$atom)*};
    my $quoted     = qr{"(?:\\[^\r\n]|[^\\"])*"};
    my $local      = qr{(?:$dot_atom|$quoted)};
    my $domain_lit = qr{\[(?:\\\S|[\x21-\x5a\x5e-\x7e])*\]};
    my $domain     = qr{(?:$dot_atom|$domain_lit)};
    my $addr_spec  = qr{$local\@$domain};
    
    $email =~ m/^${addr_spec}$/ ? return 0 : return 1;
}

##############################################################################
#
# guess_input_format($seqs)
#
#	Tries to guess if $seq is a gene list or a fasta sequence, 
#	returns "gene_list" or "fasta"
#
##############################################################################
sub guess_input_format {
	my $seqs = shift;
	
	# Check the trivial case: a FASTA file contains ">"
	# Check with is_fasta if seq is really in fasta format
	if($seqs =~ m/>/) {
		return "fasta";
	} else {
		return "gene_list";
	}
	
}

##############################################################################
#
# is_fasta($seqs)
#
#	Checks if seqs is in FASTA format or not, returns $cleaned_fasta or "no_fasta"
#
##############################################################################
sub is_fasta {
	my $seqs = shift;
	my @lines = split(/\r\n/, $seqs);

	my $cleaned_fasta;
	my $err_code = 0;
	
	for(my $i=0; $i < @lines; $i++) {
		# clean out empty lines
		if($lines[$i] =~ m/^\s*$/) {
			next; 
		}
		# concatenate sequence lines
		if( $lines[$i] !~ m/^\s*>.*$/ and $lines[$i+1] !~ m/^\s*>.*$/) { 
			$cleaned_fasta .= "$lines[$i]";
		}
		# print comment in newline
		elsif($lines[$i] =~ m/^\s*>.*$/ && $i != 0) {
			$cleaned_fasta .= "\n$lines[$i]\n";
		}
		else {
			$cleaned_fasta .= "$lines[$i]\n";
		}
		$cleaned_fasta =~ s/\n\n/\n/g
	}
	$cleaned_fasta .= "\n";
	
	$cleaned_fasta =~ m/(>.*?\n[A-Za-z]+?\n)+/m;
	
	my $match = $&;
	if($match eq $cleaned_fasta) {
		$err_code = "$cleaned_fasta";
	}
	else {
		$err_code = "no_fasta";
	}
	
	return $err_code;
}


##############################################################################
#
# get_gene_count(db, seqs, transcript_library)
#
#	Tries to count the genes in gene list and checks them against the db
#	return: number of genes found (integer) or failure message 
#
##############################################################################
sub get_gene_count {
	my $db = shift;
	my $seqs = shift;
	my $transcript_library = shift;
	
	my $count = 0;
	my $err_code = 0;
	my $err_msg = "";
	
	my $format = guess_input_format($seqs);
	my $seq_db = AmiRNA::DB::Sequences->new( $db );
	
	if($format eq "fasta") {
		my @lines = split(/\r\n/, $seqs);
# 		my @lines = split(/\n/, $seqs);
		my @found_genes;
		for(my $i=0; $i < @lines; $i++) {
			if($lines[$i] =~ m/>(.*)/) {
				my $gene = $1;
				$count++; # We found a gene name, count it
				# Check if gene name exists in db
				my $res;
				eval {
					$res = $seq_db->check_transcript($transcript_library, $gene);
					# We are in FASTA mode, gene must not be found in db
					if($res ne 0) {
						push @found_genes, $gene;
						$err_code = 1;
					}
				};
				if($@) {
					die;
				}
			}
		}
		# Assemble error message
		if($err_code eq 1) {
			if(  scalar(@found_genes) > 1) {
				$err_msg = "You submitted a FASTA sequence, but some genes (" . join(", ", @found_genes) . ") were found in the annotation database. This is a serious error.";
			} else {
				$err_msg = "You submitted a FASTA sequence, but the gene " . join(", ", @found_genes) . " was found in the annotation database. This is a serious error.";
			}
			$err_msg .= "<br>Please specify a gene list for annotated genes, not a FASTA sequence.<br>";
		}
	}
	elsif($format eq "gene_list") {
		# Clean the gene list to a csv line
		$seqs =~ s/\n/,/gm;
		$seqs =~ s/,+/,/g;
		$seqs =~ s/\s//gm;
		
		my @seqs = split(",",$seqs);
		my @not_found_genes;
		
		foreach my $gene (@seqs) {
			my $res;
				$count++;
				eval {
					$res = $seq_db->check_transcript($transcript_library, $gene);
					# We are in FASTA mode, gene must not be found in db
					if($res eq 0) {
						push @not_found_genes, $gene;
						$err_code = 1;
					}
				};
				if($@) {
					die;
				}
		}
		# Assemble error message
		if($err_code eq 1) {
			if(  scalar(@not_found_genes) > 1) {
				$err_msg = "You submitted a gene list, but some genes (<i>" . join(", ", @not_found_genes) . "</i>) were not found in the annotation database.";
			} else {
				$err_msg = "You submitted a gene list, but the gene <i>" . join(", ", @not_found_genes) . "</i> was not found in the annotation database.";
			}
			$err_msg .= "<br>Please specify a gene FASTA sequence for not annotated genes or use the <a href=\"webapp.cgi?page=Help;project=stdwmd#supplementaryTools\">blast tool</a> to determine the correct gene name.<br>";
		}
	}
	else {
		$err_msg = "Wrong input format. Currently only 'fasta' 'and gene_list' are supported.<br>";
		$err_code = 1;
	}
	
	if($err_code == 1) {
		return $err_msg;
	}
	else {
		return $count;
	}
}

##############################################################################
#
# check_unique_gene_name(gene_list_csv)
#
#	Checks the submitted list for duplicate gene names. Returns 0 if all
#	gene names are unique, 1 if there are duplicates or isoforms
#
##############################################################################
sub check_unique_gene_name {
	my $list = shift;
	
	my @genes = split(",", $list);
	# remove isoform suffices
	for(my $i=0; $i<scalar(@genes); $i++) {
		$genes[$i] =~ s/\..*$//;
	}
	my @unique_genes = del_double(@genes);
	return scalar(@genes) == scalar(@unique_genes) ? 0 : 1;
}

sub del_double
{
	my %all;
	$all{$_}=0 for @_;
	return (keys %all);
}

1;
