package AmiRNA::Designer::MiRNA;

use strict;
use warnings;
use Data::Types qw(:all);
use AmiRNA::Error;
use AmiRNA::Config::UserParams;

our $VERSION = '3.1';

################################################################################
### Constructor
sub new {
	my ( $self, $mirna_pattern, $seed_hits, $absolute,  $MM3_candidate, $relaxed_filter_32, $relaxed_filter_28_32, 
		$mutations, $optimal_energy, $targets, $target_seqs, $flanking_seqs, $starting_positions, $type) = @_;


	eval {
		if(	(! is_string($mirna_pattern)) || (! is_string($type)) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "bad string" });
		}
		if( 	(! is_int($absolute)) || (! is_string($MM3_candidate)) || (! is_int($relaxed_filter_32)) ||
			(! is_int($relaxed_filter_28_32)) || (! is_int($mutations)) )
		{
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "bad int" });
		}
		if(	(! is_float($optimal_energy)) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "bad float" });
		}
		if(	(! defined $targets) || (! defined $target_seqs) ) {
			die AmiRNA::Error->new( { errno => "bad parameters", msg => "Designer error", subject => "undefined variable" });
		}
	};
	die if $@;

	$self = {
		mirna_pattern        => $mirna_pattern,
		seed_hits            => $seed_hits,
		absolute             => $absolute,
		maybe                => $MM3_candidate,
		relaxed_filter_32    => $relaxed_filter_32,
		relaxed_filter_28_32 => $relaxed_filter_28_32,
		mutations            => $mutations,
		optimal_energy       => $optimal_energy,
		targets              => $targets,
		target_sequences     => $target_seqs,
		flanking_seqs        => $flanking_seqs,
		starting_positions   => $starting_positions,
		type                 => $type,
		score                => 0,
	};
	bless $self;

	$self->{score} = $self->score;

	return $self;
}


################################################################################
### Scoring
sub score {
    my ($self) = @_;
    if ( $self->{type} eq 'single' ) {
        return $self->_score_single();
    }
    else {
        return $self->_score_multi();
    }
}


################################################################################
### Multi scoring
sub _score_multi {
    my ($self) = @_;
    my $score         = 0;
    my @targets_array = split ( /\t/, $self->{targets} );
    my @hyb_absolute  = ( 0, 0, 0, 0, 0 );
    my @hyb_relative  = ( 0, 0, 0, 0, 0 );
    my (%hyb_abs_matrix) = (
        "00000" => 10,
        "10000" => 2,
        "01000" => 1,
        "00100" => 0,
        "00010" => 1,
        "00001" => 3,
        "11000" => 1,
        "10100" => 2,
        "10010" => 2,
        "10001" => 7,
        "01100" => 1,
        "01010" => 2,
        "01001" => 5,
        "00110" => 1,
        "00101" => 3,
        "00011" => 3,
        "11100" => 2,
        "11010" => 3,
        "11001" => 10,
        "10110" => 3,
        "10101" => 10,
        "10011" => 10,
        "01110" => 2,
        "01101" => 6,
        "01011" => 5,
        "00111" => 5,
        "11110" => 4,
        "11101" => 10,
        "11011" => 10,
        "10111" => 10,
        "01111" => 8,
        "11111" => 10,
    );
    my (%hyb_rel_matrix) = (
        "00000" => 10,
        "10000" => 5,
        "01000" => 1,
        "00100" => 0,
        "00010" => 2,
        "00001" => 3,
        "11000" => 2,
        "10100" => 2,
        "10010" => 5,
        "10001" => 8,
        "01100" => 1,
        "01010" => 2,
        "01001" => 6,
        "00110" => 1,
        "00101" => 4,
        "00011" => 6,
        "11100" => 2,
        "11010" => 6,
        "11001" => 10,
        "10110" => 4,
        "10101" => 10,
        "10011" => 10,
        "01110" => 3,
        "01101" => 8,
        "01011" => 8,
        "00111" => 5,
        "11110" => 8,
        "11101" => 10,
        "11011" => 10,
        "10111" => 10,
        "01111" => 8,
        "11111" => 10,
    );

    # by absolute hybridization energy of amiR
    if ( $self->{optimal_energy} < -46 ) {
        $score += 3;
    }
    elsif ( $self->{optimal_energy} < -44 ) {
        $score += 2;
    }
    elsif ( $self->{optimal_energy} < -38 ) {
        $score += 0;
    }
    elsif ( $self->{optimal_energy} < -34 ) {
        $score += 2;
    }
    elsif ( $self->{optimal_energy} < -30 ) {
        $score += 4;
    }
    elsif ( $self->{optimal_energy} >= -30 ) {
        $score += 7;
    }

    my $i = 1;

    # by absolute and relative hybridization energy of targets with micro
    while ( $i < length(@targets_array) ) {
        my $percent = $targets_array[$i] / $self->{optimal_energy} * 100;
        if ( $targets_array[$i] < -46 ) {
            $hyb_absolute[0] = 1;
        }
        elsif ( $self->{optimal_energy} < -44 ) {
            $hyb_absolute[1] = 1;
        }
        elsif ( $self->{optimal_energy} < -38 ) {
            $hyb_absolute[2] = 1;
        }
        elsif ( $self->{optimal_energy} < -34 ) {
            $hyb_absolute[3] = 1;
        }
        elsif ( $self->{optimal_energy} < -30 ) {
            $hyb_absolute[4] = 1;
        }

        if ( $percent >= 100 ) {
            $hyb_relative[0] = 1;
        }
        elsif ( $percent >= 95 ) {
            $hyb_relative[1] = 1;
        }
        elsif ( $percent >= 80 ) {
            $hyb_relative[2] = 1;
        }
        elsif ( $percent >= 75 ) {
            $hyb_relative[3] = 1;
        }
        elsif ( $percent < 75 ) {
            $hyb_relative[4] = 1;
        }

        $i += 2;
    }
    my $distrib_abs = '';
    foreach (@hyb_absolute) { $distrib_abs .= $_; }
    my $distrib_rel = '';
    foreach (@hyb_relative) { $distrib_rel .= $_; }

    $score += $hyb_abs_matrix{$distrib_abs};
    $score += $hyb_rel_matrix{$distrib_rel};

    # by mismatch positions
    my $tmp_score = 0;
    my @tmp_seqs = split ( /\t/, $self->{target_sequences} );
    foreach (@tmp_seqs) {
        if ( $self->get_number_of_mismatches_in_range( 1, 11, $_ ) == 0 ) {
            if ( $self->get_number_of_mismatches_in_range( 12, 20, $_ ) == 0 )
            {
                $tmp_score += 3;
            }
            elsif (
                $self->get_number_of_mismatches_in_range( 12, 20, $_ ) <= 2 )
            {
                $tmp_score += 0;
            }
            elsif (
                $self->get_number_of_mismatches_in_range( 12, 20, $_ ) <= 4 )
            {
                $tmp_score += 4;
            }
        }
        elsif ( $self->get_number_of_mismatches_in_range( 1, 11, $_ ) > 0 ) {
            if ( $self->get_number_of_mismatches_in_range( 12, 20, $_ ) == 0 )
            {
                $tmp_score += 2;
            }
            elsif (
                $self->get_number_of_mismatches_in_range( 12, 20, $_ ) <= 2 )
            {
                $tmp_score += 3;
            }
            elsif (
                $self->get_number_of_mismatches_in_range( 12, 20, $_ ) <= 4 )
            {
                $tmp_score += 5;
            }

        }
        else { $tmp_score += 8; }
    }
    $tmp_score = $tmp_score / scalar(@tmp_seqs);
    $score += $tmp_score;

    # by other hits
    if ( ( $self->{absolute} - scalar(@tmp_seqs) ) > 14 ) {
        $score += 3;
    }
    elsif ( ( $self->{absolute} - scalar(@tmp_seqs) ) > 7 ) {
        $score += 2;
    }
    elsif ( ( $self->{absolute} - scalar(@tmp_seqs) ) > 2 ) {
        $score += 1;
    }
    elsif ( ( $self->{absolute} - scalar(@tmp_seqs) ) >= 0 ) {
        $score += 0;
    }

    # by terminal sequences
    if (   $self->get_number_of_au_in_range( 0, 2 ) == 3
        && $self->get_number_of_gc_in_range( 16, 18 ) == 3 )
    {
        $score += 0;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 2
        && $self->get_number_of_gc_in_range( 16, 18 ) == 3 )
    {
        $score += 1;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 3
        && $self->get_number_of_gc_in_range( 16, 18 ) == 2 )
    {
        $score += 1;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 2
        && $self->get_number_of_gc_in_range( 16, 18 ) == 2 )
    {
        $score += 2;
    }
    else {
        $score += 3;
    }

    $score += $self->{maybe};
    $score += $self->{relaxed_filter_28_32};
    $score += ($self->{relaxed_filter_32} * 2);
            
    return $score;
}


################################################################################
### Single scoring
sub _score_single {
    my ($self) = @_;
    my $score = 0;
    my ( $target, $dg ) = split "\t", $self->{targets};

    $score += $self->_score_single_by_abs_hybridization_of_amiR();
    $score += $self->_score_single_by_abs_hybridization_of_target($dg);
    $score += $self->_score_single_by_rel_hybridization($dg);
    $score += $self->_score_single_by_number_of_mm();
    $score += $self->_score_single_by_terminal_seqs();
    $score += $self->_score_single_by_other_hits();
    $score += $self->_score_single_by_maybe_hits();

    return $score;
}


################################################################################
### Single scoring tests
sub _score_single_by_abs_hybridization_of_amiR {
    my ($self) = @_;
    my $score = 4;

    if ( $self->{optimal_energy} < -47 ) {
        $score = 3;
    }
    elsif ( $self->{optimal_energy} < -44 ) {
        $score = 1;
    }
    elsif ( $self->{optimal_energy} < -38 ) {
        $score = 0;
    }
    elsif ( $self->{optimal_energy} < -35 ) {
        $score = 2;
    }
    elsif ( $self->{optimal_energy} < -31 ) {
	$score = 4;
    }
    elsif ( $self->{optimal_energy} >= -31 ) {
	$score = 5;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_abs_hybridization_of_target {
    my ( $self, $dg ) = @_;
    my $score = 3;

    if ( $dg < -44 ) {
        $score = 3;
    }
    elsif ( $dg < -41 ) {
        $score = 1;
    }
    elsif ( $dg < -36 ) {
        $score = 0;
    }
    elsif ( $dg < -33 ) {
        $score = 2;
    }
    elsif ( $dg < -31 ) {
        $score = 4;
    }
    elsif ( $dg >= -31 ) {
        $score = 5;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_rel_hybridization {
    my ( $self, $dg ) = @_;

    my $score = 3;

    # by relative hybridization energy
    my $percent = $dg / $self->{optimal_energy} * 100;
    if ( $percent >= 99 ) {
        $score = 4;
    }
    elsif ( $percent >= 95 ) {
        $score = 1;
    }
    elsif ( $percent >= 80 ) {
        $score = 0;
    }
    elsif ( $percent >= 75 ) {
        $score = 2;
    }
    elsif ( $percent < 75 ) {
        $score = 3;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_number_of_mm {
    my ( $self, $dg ) = @_;
    my $score = 3;
    if ( $self->get_number_of_mismatches_in_single == 1 ) {
        if ( $self->get_number_of_mismatches_in_single( 17, 20 ) == 1 ) {
            $score = 0;
        }
        elsif ( $self->get_number_of_mismatches_in_single( 12, 15 ) == 1 ) {
            $score = 1;
        }
	elsif ( $self->get_number_of_mismatches_in_single( 16, 16 ) == 1 ) {
		$score = 2;
	}
        elsif ( $self->get_number_of_mismatches_in_single( 1, 11 ) == 1 ) {
            $score = 3;
        }
        elsif ( $self->get_number_of_mismatches_in_single( 0, 0 ) == 1 ) {
            $score = 3;
        }
    }
    elsif ( $self->get_number_of_mismatches_in_single == 2 ) {
        if ( $self->get_number_of_mismatches_in_single( 17, 20 ) >= 1 ) {
            if ( $self->get_number_of_mismatches_in_single( 17, 20 ) == 2 ) {
                $score = 1;
            }
            elsif ( $self->get_number_of_mismatches_in_single( 12, 15 ) == 1 )
            {
                $score = 1;
            }
	    elsif ( $self->get_number_of_mismatches_in_single( 16, 16 ) == 1 ) {
	        $score = 2;
	    }
            elsif ( $self->get_number_of_mismatches_in_single( 1, 11 ) == 1 )
            {
                $score = 3;
            }
            elsif ( $self->get_number_of_mismatches_in_single( 0, 0 ) == 1 ) {
                $score = 0;
            }
        }
        elsif ( $self->get_number_of_mismatches_in_single( 12, 15 ) >= 1 ) {
            if ( $self->get_number_of_mismatches_in_single( 12, 15 ) == 2 ) {
                $score = 2;
            }
	    elsif ( $self->get_number_of_mismatches_in_single( 16, 16 ) == 1 ) {
	        $score = 3;
	    }
            elsif ( $self->get_number_of_mismatches_in_single( 1, 11 ) == 1 )
            {
                $score = 3;
            }
            elsif ( $self->get_number_of_mismatches_in_single( 0, 0 ) == 1 ) {
                $score = 2;
            }
        }
        elsif ( $self->get_number_of_mismatches_in_single( 1, 11 ) == 1 ) {
            if ( $self->get_number_of_mismatches_in_single( 0, 0 ) == 1 ) {
                $score = 3;
            }
        }
    }
    elsif ( $self->get_number_of_mismatches_in_single == 0 ) {
        $score = 6;
    }
    else {
        $score = 4;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_terminal_seqs {
    my ($self) = @_;
    my $score = 4;

    if (   $self->get_number_of_au_in_range( 0, 2 ) == 3
        && $self->get_number_of_gc_in_range( 16, 18 ) == 3 )
    {
        $score = 0;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 2
        && $self->get_number_of_gc_in_range( 16, 18 ) == 3 )
    {
        $score = 1;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 3
        && $self->get_number_of_gc_in_range( 16, 18 ) == 2 )
    {
        $score = 1;
    }
    elsif ($self->get_number_of_au_in_range( 0, 2 ) == 2
        && $self->get_number_of_gc_in_range( 16, 18 ) == 2 )
    {
        $score = 2;
    }
    else {
        $score = 4;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_other_hits {
    my ($self) = @_;
    my $score = 3;

    if ( $self->{absolute} > 14 ) {
        $score = 3;
    }
    elsif ( $self->{absolute} > 7 ) {
        $score = 2;
    }
    elsif ( $self->{absolute} > 2 ) {
        $score = 1;
    }
    elsif ( $self->{absolute} >= 0 ) {
        $score = 0;
    }
    return $score;
}

################################################################################
### scoring subfunction
sub _score_single_by_maybe_hits {
    my ($self) = @_;
    my $score = 0;
    $score += $self->{maybe};
    $score += $self->{relaxed_filter_28_32};
    $score += ($self->{relaxed_filter_32} * 2);
    return $score;
}


################################################################################
### Count mismatches and CG content
sub get_number_of_mismatches_in_single {
    my ( $self, $from, $to ) = @_;
    if ( !defined $from ) { $from = 0; }
    if ( !defined $to )   { $to   = length( $self->{mirna_pattern} ) - 1; }
    my $target = uc( $self->{target_sequences} );
    
    warn "target empty" if $target eq '';
    $self->get_number_of_mismatches_in_range( $from, $to, $target );
}

################################################################################
### Mismatch counter
sub get_number_of_mismatches_in_range {
    my ( $self, $from, $to, $target ) = @_;
    my $mirna = uc( $self->{mirna_pattern} );
    $target = reverse($target);
    $target =~ tr/ACGT/TGCA/;
    my $mm = 0;
    warn "target not defined" if !defined $target;
    warn "mirna  not defined" if !defined $mirna;
    warn "from   not defined" if !defined $from;
    warn "to     not defined" if !defined $to;
    for my $i ( $from .. $to ) {
        if (length($mirna) < $i) {
            warn "MIRNA: $mirna, $i";
        }
        if (length($target) < $i) {
            warn "TARGET: $target, $i";
        }
        if ( substr( $mirna, $i, 1 ) ne substr( $target, $i, 1 ) ) {
            $mm++;
        }
    }
    return $mm;
}

################################################################################
### AT content counter
sub get_number_of_au_in_range {
    my ( $self, $from, $to ) = @_;
    return $self->_get_number_of_matches_in_range( $from, $to, ( 'A', 'T' ) );
}

################################################################################
### GC content counter
sub get_number_of_gc_in_range {
    my ( $self, $from, $to ) = @_;
    return $self->_get_number_of_matches_in_range( $from, $to, ( 'G', 'C' ) );
}

################################################################################
### Match counter
sub _get_number_of_matches_in_range {
    my ( $self, $from, $to, @bases ) = @_;
    my $m = 0;
    for my $i ( $from .. $to ) {
        foreach my $base (@bases) {
            if ( substr( $self->{mirna_pattern}, $i, 1 ) eq uc($base) ) {
                $m++;
            }
        }
    }
    return $m;
}
1;

__END__

=head1 NAME

 AmiRNA::Designer::MiRNA

=head1 DESCRIPTION

 Container for filtered amiRNAs with scoring function.


=head1 METHODS

=over

=item AmiRNA::Designer::MiRNA-E<gt>new( $mirna_pattern, $seed_hits, $absolute,  $MM3_candidate, $relaxed_filter_32, 
					$relaxed_filter_28_32, $mutations, $optimal_energy, $targets, $target_seqs, 
					$flanking_seqs, $starting_positions, $type )

 Constructs MiRNA object.

 Parameters:
    mirna_pattern        => miRNA sequence
    seed_hits            => 
    absolute             => total number of targets before filter
    MM3_candidate        => number of targets with up to 3 mismatches not passing filter
    relaxed_filter_32    => number of targets passing relaxed filter (dG > -32)
    relaxed_filter_28_32 => number of targets passing stronlgy relaxed filter (dG > -28)
    mutations            => number of mutations/mismatches of amiRNA to target
    optimal_energy       => hybridization energy for mismatch free miRNA/target binding
    targets              => target IDs, comma seperated
    target_seqs          => target sequences, comma seperated
    flanking_seqs        => flanking sequences of targets, comma seperated
    starting_positions   => position of binding site withn target gene
    type                 => 


=item AmiRNA::Designer::MiRNA-E<gt>score()

 Score calculated amiRNA according to empirical rules.


=back


=head1 DEPENDENCIES

=over

=item L<Data::Types>

=back


=head1 SEE ALSO

=over

=item L<AmiRNA::Designer::AmiRNADesigner>

=item L<AmiRNA::TargetSearch::MIRNAFilter>

=item L<AmiRNA::Error>

=back


=head1 AUTHOR

=over

=item Stephan Ossowski <stephan.ossowski@tuebingen.mpg.de>

=item Markus Riester, <mriester@gmx.de>

=item Joffrey Fitz, <joffrey.fitz@tuebingen.mpg.de>

=back


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2005-2009 by Max Planck Institute for Developmental Biology,Tuebingen.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut

# vim: ft=perl sw=4 ts=4 expandtab
